@@ -155,7 +155,6 @@ All three steps may be performed with one sub-command::
     Run container-sharder on all nodes to shard the container.
 
 """
-from __future__ import print_function
 import argparse
 import json
 import os.path
@@ -163,23 +162,23 @@ import sys
 import time
 from contextlib import contextmanager
 
-from six.moves import input
-
-
 from swift.common.utils import Timestamp, get_logger, ShardRange, readconf, \
-    ShardRangeList
+    ShardRangeList, non_negative_int, config_positive_int_value
 from swift.container.backend import ContainerBroker, UNSHARDED
 from swift.container.sharder import make_shard_ranges, sharding_enabled, \
     CleavingContext, process_compactible_shard_sequences, \
     find_compactible_shard_sequences, find_overlapping_ranges, \
     find_paths, rank_paths, finalize_shrinking, DEFAULT_SHARDER_CONF, \
-    ContainerSharderConf, find_paths_with_gaps
+    ContainerSharderConf, find_paths_with_gaps, combine_shard_ranges, \
+    update_own_shard_range_stats
 
 EXIT_SUCCESS = 0
 EXIT_ERROR = 1
 EXIT_INVALID_ARGS = 2  # consistent with argparse exit code for invalid args
 EXIT_USER_QUIT = 3
 
+MIN_SHARD_RANGE_AGE_FOR_REPAIR = 4 * 3600
+
 # Some CLI options derive their default values from DEFAULT_SHARDER_CONF if
 # they have not been set. It is therefore important that the CLI parser
 # provides None as a default so that we can detect that no value was set on the
@@ -206,14 +205,36 @@ class InvalidSolutionException(ManageShardRangesException):
         self.overlapping_donors = overlapping_donors
 
 
+def wrap_for_argparse(func, msg=None):
+    """
+    Wrap the given ``func`` to catch any ``ValueError`` and raise an
+    ``argparse.ArgumentTypeError`` instead.
+
+    :param func: a function.
+    :param msg: an optional message to use with any exception that is used; if
+        not given then the string representation of the ValueError will be
+        used.
+    :return: a function wrapper.
+    """
+    def wrapped_func(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except ValueError as err:
+            raise argparse.ArgumentTypeError(str(err) if msg is None else msg)
+    return wrapped_func
+
+
 def _proceed(args):
     if args.dry_run:
         choice = 'no'
     elif args.yes:
         choice = 'yes'
     else:
-        choice = input('Do you want to apply these changes to the container '
-                       'DB? [yes/N]')
+        try:
+            choice = input('Do you want to apply these changes to the '
+                           'container DB? [yes/N]')
+        except (EOFError, KeyboardInterrupt):
+            choice = 'no'
     if choice != 'yes':
         print('No changes applied')
 
@@ -356,6 +377,9 @@ def db_info(broker, args):
            if own_sr else None))
     db_state = broker.get_db_state()
     print('db_state = %s' % db_state)
+    info = broker.get_info()
+    print('object_count = %d' % info['object_count'])
+    print('bytes_used = %d' % info['bytes_used'])
     if db_state == 'sharding':
         print('Retiring db id: %s' % broker.get_brokers()[0].get_info()['id'])
         print('Cleaving context: %s' %
@@ -383,9 +407,13 @@ def delete_shard_ranges(broker, args):
             print('  - %d existing shard ranges have started sharding' %
                   [sr.state != ShardRange.FOUND
                    for sr in shard_ranges].count(True))
-        choice = input('Do you want to show the existing ranges [s], '
-                       'delete the existing ranges [yes] '
-                       'or quit without deleting [q]? ')
+        try:
+            choice = input('Do you want to show the existing ranges [s], '
+                           'delete the existing ranges [yes] '
+                           'or quit without deleting [q]? ')
+        except (EOFError, KeyboardInterrupt):
+            choice = 'q'
+
         if choice == 's':
             show_shard_ranges(broker, args)
             continue
@@ -406,6 +434,40 @@ def delete_shard_ranges(broker, args):
     return EXIT_SUCCESS
 
 
+def merge_shard_ranges(broker, args):
+    _check_own_shard_range(broker, args)
+    shard_data = _load_and_validate_shard_data(args, require_index=False)
+    new_shard_ranges = ShardRangeList([ShardRange.from_dict(sr)
+                                       for sr in shard_data])
+    new_shard_ranges.sort(key=ShardRange.sort_key)
+
+    # do some checks before merging...
+    existing_shard_ranges = ShardRangeList(
+        broker.get_shard_ranges(include_deleted=True))
+    outcome = combine_shard_ranges(new_shard_ranges, existing_shard_ranges)
+    if args.verbose:
+        print('This change will result in the following shard ranges in the '
+              'affected namespace:')
+        print(json.dumps([dict(sr) for sr in outcome], indent=2))
+    overlaps = find_overlapping_ranges(outcome)
+    if overlaps:
+        print('WARNING: this change will result in shard ranges overlaps!')
+    paths_with_gaps = find_paths_with_gaps(outcome)
+    gaps = [gap for start_path, gap, end_path in paths_with_gaps
+            if existing_shard_ranges.includes(gap)]
+    if gaps:
+        print('WARNING: this change will result in shard ranges gaps!')
+
+    if not _proceed(args):
+        return EXIT_USER_QUIT
+
+    with broker.updated_timeout(args.replace_timeout):
+        broker.merge_shard_ranges(new_shard_ranges)
+    print('Injected %d shard ranges.' % len(new_shard_ranges))
+    print('Run container-replicator to replicate them to other nodes.')
+    return EXIT_SUCCESS
+
+
 def _replace_shard_ranges(broker, args, shard_data, timeout=0):
     own_shard_range = _check_own_shard_range(broker, args)
     shard_ranges = make_shard_ranges(
@@ -449,6 +511,8 @@ def _enable_sharding(broker, own_shard_range, args):
     if own_shard_range.update_state(ShardRange.SHARDING):
         own_shard_range.epoch = Timestamp.now()
         own_shard_range.state_timestamp = own_shard_range.epoch
+    # initialise own_shard_range with current broker object stats...
+    update_own_shard_range_stats(broker, own_shard_range)
 
     with broker.updated_timeout(args.enable_timeout):
         broker.merge_shard_ranges([own_shard_range])
@@ -543,6 +607,57 @@ def compact_shard_ranges(broker, args):
     return EXIT_SUCCESS
 
 
+def _remove_illegal_overlapping_donors(
+        acceptor_path, overlapping_donors, args):
+    # Check parent-children relationship in overlaps between acceptors and
+    # donors, remove any overlapping parent or child shard range from donors.
+    # Note: we can use set() here, since shard range object is hashed by
+    # id and all shard ranges in overlapping_donors are unique already.
+    parent_child_donors = set()
+    for acceptor in acceptor_path:
+        parent_child_donors.update(
+            [donor for donor in overlapping_donors
+                if acceptor.is_child_of(donor) or donor.is_child_of(acceptor)])
+    if parent_child_donors:
+        overlapping_donors = ShardRangeList(
+            [sr for sr in overlapping_donors
+                if sr not in parent_child_donors])
+        print('%d donor shards ignored due to parent-child relationship '
+              'checks' % len(parent_child_donors))
+
+    # Check minimum age requirement in overlaps between acceptors and donors.
+    if args.min_shard_age == 0:
+        return acceptor_path, overlapping_donors
+    ts_now = Timestamp.now()
+    # Remove overlapping donor shard ranges who were created recently within
+    # 'min_shard_age' age limit.
+    qualified_donors = ShardRangeList(
+        [sr for sr in overlapping_donors
+            if float(sr.timestamp) + args.min_shard_age < float(ts_now)])
+    young_donors = len(overlapping_donors) - len(qualified_donors)
+    if young_donors > 0:
+        print('%d overlapping donor shards ignored due to minimum age '
+              'limit' % young_donors)
+    if not qualified_donors:
+        return acceptor_path, None
+    # Remove those overlapping donors whose overlapping acceptors were created
+    # within age limit.
+    donors_with_young_overlap_acceptor = set()
+    for acceptor_sr in acceptor_path:
+        if float(acceptor_sr.timestamp) + args.min_shard_age < float(ts_now):
+            continue
+        donors_with_young_overlap_acceptor.update(
+            [sr for sr in qualified_donors if acceptor_sr.overlaps(sr)])
+    if donors_with_young_overlap_acceptor:
+        qualified_donors = ShardRangeList(
+            [sr for sr in qualified_donors
+             if sr not in donors_with_young_overlap_acceptor])
+        print('%d donor shards ignored due to existence of overlapping young '
+              'acceptors' % len(donors_with_young_overlap_acceptor))
+
+    return acceptor_path, qualified_donors
+
+
 def _find_overlapping_donors(shard_ranges, own_sr, args):
     shard_ranges = ShardRangeList(shard_ranges)
     if ShardRange.SHARDING in shard_ranges.states:
@@ -593,7 +708,8 @@ def _find_overlapping_donors(shard_ranges, own_sr, args):
             'Isolated cleaved and/or active shard ranges in donor ranges',
             acceptor_path, overlapping_donors)
 
-    return acceptor_path, overlapping_donors
+    return _remove_illegal_overlapping_donors(
+        acceptor_path, overlapping_donors, args)
 
 
 def _fix_gaps(broker, args, paths_with_gaps):
@@ -775,13 +891,6 @@ def analyze_shard_ranges(args):
     return EXIT_SUCCESS
 
 
-def _positive_int(arg):
-    val = int(arg)
-    if val <= 0:
-        raise argparse.ArgumentTypeError('must be > 0')
-    return val
-
-
 def _add_find_args(parser):
     parser.add_argument(
         'rows_per_shard', nargs='?', type=int, default=USE_SHARDER_DEFAULT,
@@ -790,7 +899,8 @@ def _add_find_args(parser):
         'given in a conf file specified with --config, otherwise %s.'
         % DEFAULT_SHARDER_CONF['rows_per_shard'])
     parser.add_argument(
-        '--minimum-shard-size', type=_positive_int,
+        '--minimum-shard-size',
+        type=wrap_for_argparse(config_positive_int_value, 'must be > 0'),
         default=USE_SHARDER_DEFAULT,
         help='Minimum size of the final shard range. If this is greater than '
         'one then the final shard range may be extended to more than '
@@ -839,11 +949,12 @@ def _add_prompt_args(parser):
 
 
 def _add_max_expanding_arg(parser):
-    parser.add_argument('--max-expanding', nargs='?',
-                        type=_positive_int,
-                        default=USE_SHARDER_DEFAULT,
-                        help='Maximum number of shards that should be '
-                             'expanded. Defaults to unlimited.')
+    parser.add_argument(
+        '--max-expanding', nargs='?',
+        type=wrap_for_argparse(config_positive_int_value, 'must be > 0'),
+        default=USE_SHARDER_DEFAULT,
+        help='Maximum number of shards that should be '
+        'expanded. Defaults to unlimited.')
 
 
 def _make_parser():
@@ -905,6 +1016,22 @@ def _make_parser():
         'info', help='Print container db info')
     info_parser.set_defaults(func=db_info)
 
+    # merge
+    merge_parser = subparsers.add_parser(
+        'merge',
+        help='Merge shard range(s) from file with existing shard ranges. This '
+             'subcommand should only be used if you are confident that you '
+             'know what you are doing. Shard ranges should not typically be '
+             'modified in this way.')
+    merge_parser.add_argument('input', metavar='input_file',
+                              type=str, help='Name of file')
+    merge_parser.add_argument(
+        '--replace-timeout', type=int, default=600,
+        help='Minimum DB timeout to use when merging shard ranges.')
+    _add_account_prefix_arg(merge_parser)
+    _add_prompt_args(merge_parser)
+    merge_parser.set_defaults(func=merge_shard_ranges)
+
     # replace
     replace_parser = subparsers.add_parser(
         'replace',
@@ -940,32 +1067,35 @@ def _make_parser():
              'of rows. This command only works on root containers.')
     _add_prompt_args(compact_parser)
     compact_parser.add_argument(
-        '--shrink-threshold', nargs='?', type=_positive_int,
+        '--shrink-threshold', nargs='?',
+        type=wrap_for_argparse(config_positive_int_value, 'must be > 0'),
         default=USE_SHARDER_DEFAULT,
         help='The number of rows below which a shard can qualify for '
-             'shrinking. '
-             'Defaults to %d' % DEFAULT_SHARDER_CONF['shrink_threshold'])
+        'shrinking. '
+        'Defaults to %d' % DEFAULT_SHARDER_CONF['shrink_threshold'])
     compact_parser.add_argument(
-        '--expansion-limit', nargs='?', type=_positive_int,
+        '--expansion-limit', nargs='?',
+        type=wrap_for_argparse(config_positive_int_value, 'must be > 0'),
         default=USE_SHARDER_DEFAULT,
         help='Maximum number of rows for an expanding shard to have after '
-             'compaction has completed. '
-             'Defaults to %d' % DEFAULT_SHARDER_CONF['expansion_limit'])
+        'compaction has completed. '
+        'Defaults to %d' % DEFAULT_SHARDER_CONF['expansion_limit'])
     # If just one donor shard is chosen to shrink to an acceptor then the
     # expanded acceptor will handle object listings as soon as the donor shard
     # has shrunk. If more than one donor shard are chosen to shrink to an
     # acceptor then the acceptor may not handle object listings for some donor
     # shards that have shrunk until *all* donors have shrunk, resulting in
     # temporary gap(s) in object listings where the shrunk donors are missing.
-    compact_parser.add_argument('--max-shrinking', nargs='?',
-                                type=_positive_int,
-                                default=USE_SHARDER_DEFAULT,
-                                help='Maximum number of shards that should be '
-                                     'shrunk into each expanding shard. '
-                                     'Defaults to 1. Using values greater '
-                                     'than 1 may result in temporary gaps in '
-                                     'object listings until all selected '
-                                     'shards have shrunk.')
+    compact_parser.add_argument(
+        '--max-shrinking', nargs='?',
+        type=wrap_for_argparse(config_positive_int_value, 'must be > 0'),
+        default=USE_SHARDER_DEFAULT,
+        help='Maximum number of shards that should be '
+        'shrunk into each expanding shard. '
+        'Defaults to 1. Using values greater '
+        'than 1 may result in temporary gaps in '
+        'object listings until all selected '
+        'shards have shrunk.')
     _add_max_expanding_arg(compact_parser)
     compact_parser.set_defaults(func=compact_shard_ranges)
 
@@ -975,6 +1105,14 @@ def _make_parser():
         help='Repair overlapping shard ranges. No action will be taken '
              'without user confirmation unless the -y option is used.')
     _add_prompt_args(repair_parser)
+    repair_parser.add_argument(
+        '--min-shard-age', nargs='?',
+        type=wrap_for_argparse(non_negative_int, 'must be >= 0'),
+        default=MIN_SHARD_RANGE_AGE_FOR_REPAIR,
+        help='Minimum age of a shard for it to be considered as an overlap '
+        'that is due for repair. Overlapping shards younger than this '
+        'age will be ignored. Value of 0 means no recent shards will be '
+        'ignored. Defaults to %d.' % MIN_SHARD_RANGE_AGE_FOR_REPAIR)
     # TODO: maybe this should be a separate subcommand given that it needs
     #  some extra options vs repairing overlaps?
     repair_parser.add_argument(
@@ -988,6 +1126,14 @@ def _make_parser():
         'analyze',
         help='Analyze shard range json data read from file. Use -v to see '
              'more detailed analysis.')
+    analyze_parser.add_argument(
+        '--min-shard-age', nargs='?',
+        type=wrap_for_argparse(non_negative_int, 'must be >= 0'),
+        default=0,
+        help='Minimum age of a shard for it to be considered as an overlap '
+        'that is due for repair. Overlapping shards younger than this '
+        'age will be ignored. Value of 0 means no recent shards will be '
+        'ignored. Defaults to 0.')
     analyze_parser.set_defaults(func=analyze_shard_ranges)
 
     return parser
