@@ -21,12 +21,9 @@ import json
 import logging
 import os
 from uuid import uuid4
-import sys
 import time
 import errno
-import six
-import six.moves.cPickle as pickle
-from swift import gettext_ as _
+import pickle  # nosec: B403
 from tempfile import mkstemp
 
 from eventlet import sleep, Timeout
@@ -56,28 +53,14 @@ SQLITE_ARG_LIMIT = 999
 RECLAIM_PAGE_SIZE = 10000
 
 
-def utf8encode(*args):
-    return [(s.encode('utf8') if isinstance(s, six.text_type) else s)
-            for s in args]
-
-
 def native_str_keys_and_values(metadata):
-    if six.PY2:
-        uni_keys = [k for k in metadata if isinstance(k, six.text_type)]
-        for k in uni_keys:
-            sv = metadata[k]
-            del metadata[k]
-            metadata[k.encode('utf-8')] = [
-                x.encode('utf-8') if isinstance(x, six.text_type) else x
-                for x in sv]
-    else:
-        bin_keys = [k for k in metadata if isinstance(k, six.binary_type)]
-        for k in bin_keys:
-            sv = metadata[k]
-            del metadata[k]
-            metadata[k.decode('utf-8')] = [
-                x.decode('utf-8') if isinstance(x, six.binary_type) else x
-                for x in sv]
+    bin_keys = [k for k in metadata if isinstance(k, bytes)]
+    for k in bin_keys:
+        sv = metadata[k]
+        del metadata[k]
+        metadata[k.decode('utf-8')] = [
+            x.decode('utf-8') if isinstance(x, bytes) else x
+            for x in sv]
 
 
 ZERO_LIKE_VALUES = {None, '', 0, '0'}
@@ -130,19 +113,30 @@ class DatabaseAlreadyExists(sqlite3.DatabaseError):
 
 class GreenDBConnection(sqlite3.Connection):
     """SQLite DB Connection handler that plays well with eventlet."""
+    # slots are needed for python 3.11.0 (there's an issue fixed in 3.11.1,
+    # see https://github.com/python/cpython/issues/99886)
+    __slots__ = ('timeout', 'db_file')
 
     def __init__(self, database, timeout=None, *args, **kwargs):
         if timeout is None:
             timeout = BROKER_TIMEOUT
         self.timeout = timeout
         self.db_file = database
-        super(GreenDBConnection, self).__init__(database, 0, *args, **kwargs)
+        super(GreenDBConnection, self).__init__(
+            database, timeout=0, *args, **kwargs)
 
     def cursor(self, cls=None):
         if cls is None:
             cls = GreenDBCursor
         return sqlite3.Connection.cursor(self, cls)
 
+    def execute(self, *args, **kwargs):
+        # py311 stopped calling self.cursor() to get the cursor;
+        # see https://github.com/python/cpython/pull/31351
+        curs = self.cursor()
+        curs.execute(*args, **kwargs)
+        return curs
+
     def commit(self):
         return _db_timeout(
             self.timeout, self.db_file,
@@ -151,6 +145,9 @@ class GreenDBConnection(sqlite3.Connection):
 
 class GreenDBCursor(sqlite3.Cursor):
     """SQLite Cursor handler that plays well with eventlet."""
+    # slots are needed for python 3.11.0 (there's an issue fixed in 3.11.1,
+    # see https://github.com/python/cpython/issues/99886)
+    __slots__ = ('timeout', 'db_file')
 
     def __init__(self, *args, **kwargs):
         self.timeout = args[0].timeout
@@ -162,6 +159,9 @@ class GreenDBCursor(sqlite3.Cursor):
             self.timeout, self.db_file, lambda: sqlite3.Cursor.execute(
                 self, *args, **kwargs))
 
+    # NB: executemany and executescript are *not* greened, and never have been
+    # (as far as I can tell)
+
 
 def dict_factory(crs, row):
     """
@@ -203,9 +203,9 @@ def get_db_connection(path, timeout=30, logger=None, okay_to_create=False):
         connect_time = time.time()
         conn = sqlite3.connect(path, check_same_thread=False,
                                factory=GreenDBConnection, timeout=timeout)
-        if QUERY_LOGGING and logger and not six.PY2:
+        if QUERY_LOGGING and logger:
             conn.set_trace_callback(logger.debug)
-        if path != ':memory:' and not okay_to_create:
+        if not okay_to_create:
             # attempt to detect and fail when connect creates the db file
             stat = os.stat(path)
             if stat.st_size == 0 and stat.st_ctime >= connect_time:
@@ -359,17 +359,13 @@ class DatabaseBroker(object):
         :param put_timestamp: internalized timestamp of initial PUT request
         :param storage_policy_index: only required for containers
         """
-        if self._db_file == ':memory:':
-            tmp_db_file = None
-            conn = get_db_connection(self._db_file, self.timeout, self.logger)
-        else:
-            mkdirs(self.db_dir)
-            fd, tmp_db_file = mkstemp(suffix='.tmp', dir=self.db_dir)
-            os.close(fd)
-            conn = sqlite3.connect(tmp_db_file, check_same_thread=False,
-                                   factory=GreenDBConnection, timeout=0)
-            if QUERY_LOGGING and not six.PY2:
-                conn.set_trace_callback(self.logger.debug)
+        mkdirs(self.db_dir)
+        fd, tmp_db_file = mkstemp(suffix='.tmp', dir=self.db_dir)
+        os.close(fd)
+        conn = sqlite3.connect(tmp_db_file, check_same_thread=False,
+                               factory=GreenDBConnection, timeout=0)
+        if QUERY_LOGGING:
+            conn.set_trace_callback(self.logger.debug)
         # creating dbs implicitly does a lot of transactions, so we
         # pick fast, unsafe options here and do a big fsync at the end.
         with closing(conn.cursor()) as cur:
@@ -486,32 +482,32 @@ class DatabaseBroker(object):
                 raise
             quar_path = "%s-%s" % (quar_path, uuid4().hex)
             renamer(self.db_dir, quar_path, fsync=False)
-        detail = _('Quarantined %(db_dir)s to %(quar_path)s due to '
-                   '%(reason)s') % {'db_dir': self.db_dir,
-                                    'quar_path': quar_path,
-                                    'reason': reason}
+        detail = ('Quarantined %(db_dir)s to %(quar_path)s due to '
+                  '%(reason)s') % {'db_dir': self.db_dir,
+                                   'quar_path': quar_path,
+                                   'reason': reason}
         self.logger.error(detail)
         raise sqlite3.DatabaseError(detail)
 
-    def possibly_quarantine(self, exc_type, exc_value, exc_traceback):
+    def possibly_quarantine(self, err):
         """
         Checks the exception info to see if it indicates a quarantine situation
         (malformed or corrupted database). If not, the original exception will
         be reraised. If so, the database will be quarantined and a new
         sqlite3.DatabaseError will be raised indicating the action taken.
         """
-        if 'database disk image is malformed' in str(exc_value):
+        if 'database disk image is malformed' in str(err):
             exc_hint = 'malformed database'
-        elif 'malformed database schema' in str(exc_value):
+        elif 'malformed database schema' in str(err):
             exc_hint = 'malformed database'
-        elif ' is not a database' in str(exc_value):
+        elif ' is not a database' in str(err):
             # older versions said 'file is not a database'
             # now 'file is encrypted or is not a database'
             exc_hint = 'corrupted database'
-        elif 'disk I/O error' in str(exc_value):
+        elif 'disk I/O error' in str(err):
             exc_hint = 'disk error while accessing database'
         else:
-            six.reraise(exc_type, exc_value, exc_traceback)
+            raise err
 
         self.quarantine(exc_hint)
 
@@ -541,12 +537,12 @@ class DatabaseBroker(object):
     def get(self):
         """Use with the "with" statement; returns a database connection."""
         if not self.conn:
-            if self.db_file != ':memory:' and os.path.exists(self.db_file):
+            if os.path.exists(self.db_file):
                 try:
                     self.conn = get_db_connection(self.db_file, self.timeout,
                                                   self.logger)
-                except (sqlite3.DatabaseError, DatabaseConnectionError):
-                    self.possibly_quarantine(*sys.exc_info())
+                except (sqlite3.DatabaseError, DatabaseConnectionError) as e:
+                    self.possibly_quarantine(e)
             else:
                 raise DatabaseConnectionError(self.db_file, "DB doesn't exist")
         conn = self.conn
@@ -555,12 +551,12 @@ class DatabaseBroker(object):
             yield conn
             conn.rollback()
             self.conn = conn
-        except sqlite3.DatabaseError:
+        except sqlite3.DatabaseError as e:
             try:
                 conn.close()
             except Exception:
                 pass
-            self.possibly_quarantine(*sys.exc_info())
+            self.possibly_quarantine(e)
         except (Exception, Timeout):
             conn.close()
             raise
@@ -569,7 +565,7 @@ class DatabaseBroker(object):
     def lock(self):
         """Use with the "with" statement; locks a database."""
         if not self.conn:
-            if self.db_file != ':memory:' and os.path.exists(self.db_file):
+            if os.path.exists(self.db_file):
                 self.conn = get_db_connection(self.db_file, self.timeout,
                                               self.logger)
             else:
@@ -581,16 +577,15 @@ class DatabaseBroker(object):
         conn.execute('BEGIN IMMEDIATE')
         try:
             yield True
-        except (Exception, Timeout):
-            pass
-        try:
-            conn.execute('ROLLBACK')
-            conn.isolation_level = orig_isolation_level
-            self.conn = conn
-        except (Exception, Timeout):
-            logging.exception(
-                _('Broker error trying to rollback locked connection'))
-            conn.close()
+        finally:
+            try:
+                conn.execute('ROLLBACK')
+                conn.isolation_level = orig_isolation_level
+                self.conn = conn
+            except (Exception, Timeout):
+                logging.exception(
+                    'Broker error trying to rollback locked connection')
+                conn.close()
 
     def _new_db_id(self):
         device_name = os.path.basename(self.get_device_path())
@@ -637,7 +632,7 @@ class DatabaseBroker(object):
 
         :returns: True if the DB is considered to be deleted, False otherwise
         """
-        if self.db_file != ':memory:' and not os.path.exists(self.db_file):
+        if not os.path.exists(self.db_file):
             return True
         self._commit_puts_stale_ok()
         with self.get() as conn:
@@ -714,22 +709,26 @@ class DatabaseBroker(object):
                 return -1
             return row['sync_point']
 
-    def get_syncs(self, incoming=True):
+    def get_syncs(self, incoming=True, include_timestamp=False):
         """
         Get a serialized copy of the sync table.
 
         :param incoming: if True, get the last incoming sync, otherwise get
                          the last outgoing sync
-        :returns: list of {'remote_id', 'sync_point'}
+        :param include_timestamp: If True include the updated_at timestamp
+        :returns: list of {'remote_id', 'sync_point'} or
+                 {'remote_id', 'sync_point', 'updated_at'}
+                 if include_timestamp is True.
         """
         with self.get() as conn:
+            columns = 'remote_id, sync_point'
+            if include_timestamp:
+                columns += ', updated_at'
             curs = conn.execute('''
-                SELECT remote_id, sync_point FROM %s_sync
-            ''' % ('incoming' if incoming else 'outgoing'))
-            result = []
-            for row in curs:
-                result.append({'remote_id': row[0], 'sync_point': row[1]})
-            return result
+                SELECT %s FROM %s_sync
+            ''' % (columns, 'incoming' if incoming else 'outgoing'))
+            curs.row_factory = dict_factory
+            return [r for r in curs]
 
     def get_max_row(self, table=None):
         if not table:
@@ -770,8 +769,8 @@ class DatabaseBroker(object):
         """
         Put a record into the DB. If the DB has an associated pending file with
         space then the record is appended to that file and a commit to the DB
-        is deferred. If the DB is in-memory or its pending file is full then
-        the record will be committed immediately.
+        is deferred. If its pending file is full then the record will be
+        committed immediately.
 
         :param record: a record to be added to the DB.
         :raises DatabaseConnectionError: if the DB file does not exist or if
@@ -779,9 +778,6 @@ class DatabaseBroker(object):
         :raises LockTimeout: if a timeout occurs while waiting to take a lock
             to write to the pending file.
         """
-        if self._db_file == ':memory:':
-            self.merge_items([record])
-            return
         if not os.path.exists(self.db_file):
             raise DatabaseConnectionError(self.db_file, "DB doesn't exist")
         if self.skip_commits:
@@ -807,8 +803,7 @@ class DatabaseBroker(object):
                     fp.flush()
 
     def _skip_commit_puts(self):
-        return (self._db_file == ':memory:' or self.skip_commits or not
-                os.path.exists(self.pending_file))
+        return self.skip_commits or not os.path.exists(self.pending_file)
 
     def _commit_puts(self, item_list=None):
         """
@@ -837,15 +832,12 @@ class DatabaseBroker(object):
             for entry in fp.read().split(b':'):
                 if entry:
                     try:
-                        if six.PY2:
-                            data = pickle.loads(base64.b64decode(entry))
-                        else:
-                            data = pickle.loads(base64.b64decode(entry),
-                                                encoding='utf8')
+                        data = pickle.loads(base64.b64decode(entry),
+                                            encoding='utf8')  # nosec: B301
                         self._commit_puts_load(item_list, data)
                     except Exception:
                         self.logger.exception(
-                            _('Invalid pending entry %(file)s: %(entry)s'),
+                            'Invalid pending entry %(file)s: %(entry)s',
                             {'file': self.pending_file, 'entry': entry})
             if item_list:
                 self.merge_items(item_list)
@@ -922,7 +914,7 @@ class DatabaseBroker(object):
         within 512k of a boundary, it allocates to the next boundary.
         Boundaries are 2m, 5m, 10m, 25m, 50m, then every 50m after.
         """
-        if not DB_PREALLOCATION or self._db_file == ':memory:':
+        if not DB_PREALLOCATION:
             return
         MB = (1024 * 1024)
 
