@@ -123,11 +123,9 @@ the file are simply ignored).
 __all__ = ['FormPost', 'filter_factory', 'READ_CHUNK_SIZE', 'MAX_VALUE_LENGTH']
 
 import hmac
-import hashlib
 from time import time
 
-import six
-from six.moves.urllib.parse import quote
+from urllib.parse import quote
 
 from swift.common.constraints import valid_api_version
 from swift.common.exceptions import MimeInvalid
@@ -136,10 +134,11 @@ from swift.common.digest import get_allowed_digests, \
     extract_digest_and_algorithm, DEFAULT_ALLOWED_DIGESTS
 from swift.common.utils import streq_const_time, parse_content_disposition, \
     parse_mime_headers, iter_multipart_mime_documents, reiterate, \
-    close_if_possible, get_logger
+    closing_if_possible, get_logger, InputProxy
 from swift.common.registry import register_swift_info
-from swift.common.wsgi import make_pre_authed_env
+from swift.common.wsgi import WSGIContext, make_pre_authed_env
 from swift.common.swob import HTTPUnauthorized, wsgi_to_str, str_to_wsgi
+from swift.common.http import is_success
 from swift.proxy.controllers.base import get_account_info, get_container_info
 
 
@@ -159,7 +158,7 @@ class FormUnauthorized(Exception):
     pass
 
 
-class _CappedFileLikeObject(object):
+class _CappedFileLikeObject(InputProxy):
     """
     A file-like object wrapping another file-like object that raises
     an EOFError if the amount of data read exceeds a given
@@ -171,26 +170,15 @@ class _CappedFileLikeObject(object):
     """
 
     def __init__(self, fp, max_file_size):
-        self.fp = fp
+        super().__init__(fp)
         self.max_file_size = max_file_size
-        self.amount_read = 0
         self.file_size_exceeded = False
 
-    def read(self, size=None):
-        ret = self.fp.read(size)
-        self.amount_read += len(ret)
-        if self.amount_read > self.max_file_size:
+    def chunk_update(self, chunk, eof, *args, **kwargs):
+        if self.bytes_received > self.max_file_size:
             self.file_size_exceeded = True
             raise EOFError('max_file_size exceeded')
-        return ret
-
-    def readline(self):
-        ret = self.fp.readline()
-        self.amount_read += len(ret)
-        if self.amount_read > self.max_file_size:
-            self.file_size_exceeded = True
-            raise EOFError('max_file_size exceeded')
-        return ret
+        return chunk
 
 
 class FormPost(object):
@@ -248,9 +236,7 @@ class FormPost(object):
                      ('Content-Length', str(len(body)))))
                 return [body]
             except (FormInvalid, EOFError) as err:
-                body = 'FormPost: %s' % err
-                if six.PY3:
-                    body = body.encode('utf-8')
+                body = ('FormPost: %s' % err).encode('utf-8')
                 start_response(
                     '400 Bad Request',
                     (('Content-Type', 'text/plain'),
@@ -272,12 +258,12 @@ class FormPost(object):
         :returns: status_line, headers_list, body
         """
         keys = self._get_keys(env)
-        if six.PY3:
-            boundary = boundary.encode('utf-8')
+        boundary = boundary.encode('utf-8')
         status = message = ''
         attributes = {}
         file_attributes = {}
         subheaders = []
+        resp_body = None
         file_count = 0
         for fp in iter_multipart_mime_documents(
                 env['wsgi.input'], boundary, read_chunk_size=READ_CHUNK_SIZE):
@@ -302,9 +288,10 @@ class FormPost(object):
                         'content-encoding' in hdrs:
                     file_attributes['content-encoding'] = \
                         hdrs['Content-Encoding']
-                status, subheaders = \
+                status, subheaders, resp_body = \
                     self._perform_subrequest(env, file_attributes, fp, keys)
-                if not status.startswith('2'):
+                status_code = int(status.split(' ', 1)[0])
+                if not is_success(status_code):
                     break
             else:
                 data = b''
@@ -317,14 +304,14 @@ class FormPost(object):
                     data += chunk
                 while fp.read(READ_CHUNK_SIZE):
                     pass
-                if six.PY3:
-                    data = data.decode('utf-8')
+                data = data.decode('utf-8')
                 if 'name' in attrs:
                     attributes[attrs['name'].lower()] = data.rstrip('\r\n--')
         if not status:
             status = '400 Bad Request'
             message = 'no files to process'
 
+        status_code = int(status.split(' ', 1)[0])
         headers = [(k, v) for k, v in subheaders
                    if k.lower().startswith('access-control')]
 
@@ -333,21 +320,21 @@ class FormPost(object):
             body = status
             if message:
                 body = status + '\r\nFormPost: ' + message.title()
+            body = body.encode('utf-8')
+            if not is_success(status_code) and resp_body:
+                body = resp_body
             headers.extend([('Content-Type', 'text/plain'),
                             ('Content-Length', len(body))])
-            if six.PY3:
-                body = body.encode('utf-8')
             return status, headers, body
-        status = status.split(' ', 1)[0]
         if '?' in redirect:
             redirect += '&'
         else:
             redirect += '?'
-        redirect += 'status=%s&message=%s' % (quote(status), quote(message))
+        redirect += 'status=%s&message=%s' % (quote(str(status_code)),
+                                              quote(message))
         body = '<html><body><p><a href="%s">' \
                'Click to continue...</a></p></body></html>' % redirect
-        if six.PY3:
-            body = body.encode('utf-8')
+        body = body.encode('utf-8')
         headers.extend(
             [('Location', redirect), ('Content-Length', str(len(body)))])
         return '303 See Other', headers, body
@@ -409,8 +396,7 @@ class FormPost(object):
             attributes.get('max_file_size') or '0',
             attributes.get('max_file_count') or '0',
             attributes.get('expires') or '0')
-        if six.PY3:
-            hmac_body = hmac_body.encode('utf-8')
+        hmac_body = hmac_body.encode('utf-8')
 
         has_valid_sig = False
         signature = attributes.get('signature', '')
@@ -420,35 +406,25 @@ class FormPost(object):
             raise FormUnauthorized('invalid signature')
         if hash_name not in self.allowed_digests:
             raise FormUnauthorized('invalid signature')
-        hash_algorithm = getattr(hashlib, hash_name) if six.PY2 else hash_name
 
         for key in keys:
             # Encode key like in swift.common.utls.get_hmac.
-            if not isinstance(key, six.binary_type):
+            if not isinstance(key, bytes):
                 key = key.encode('utf8')
-            sig = hmac.new(key, hmac_body, hash_algorithm).hexdigest()
+            sig = hmac.new(key, hmac_body, hash_name).hexdigest()
             if streq_const_time(sig, signature):
                 has_valid_sig = True
         if not has_valid_sig:
             raise FormUnauthorized('invalid signature')
         self.logger.increment('formpost.digests.%s' % hash_name)
-
-        substatus = [None]
-        subheaders = [None]
-
+        wsgi_ctx = WSGIContext(self.app)
         wsgi_input = subenv['wsgi.input']
-
-        def _start_response(status, headers, exc_info=None):
-            if wsgi_input.file_size_exceeded:
-                raise EOFError("max_file_size exceeded")
-
-            substatus[0] = status
-            subheaders[0] = headers
-
-        # reiterate to ensure the response started,
-        # but drop any data on the floor
-        close_if_possible(reiterate(self.app(subenv, _start_response)))
-        return substatus[0], subheaders[0]
+        resp = wsgi_ctx._app_call(subenv)
+        if wsgi_input.file_size_exceeded:
+            raise EOFError("max_file_size exceeded")
+        with closing_if_possible(reiterate(resp)):
+            body = b''.join(resp)
+        return wsgi_ctx._response_status, wsgi_ctx._response_headers, body
 
     def _get_keys(self, env):
         """
