@@ -17,7 +17,7 @@ from swift.common.http import is_success
 from swift.common.middleware import acl as swift_acl
 from swift.common.request_helpers import get_sys_meta_prefix
 from swift.common.swob import HTTPNotFound, HTTPForbidden, HTTPUnauthorized
-from swift.common.utils import register_swift_info
+from swift.common.utils import config_read_reseller_options, list_from_csv
 from swift.proxy.controllers.base import get_account_info
 import functools
 
@@ -31,21 +31,21 @@ UNKNOWN_ID = '_unknown'
 class KeystoneAuth(object):
     """Swift middleware to Keystone authorization system.
 
-    In Swift's proxy-server.conf add this middleware to your pipeline::
-
-        [pipeline:main]
-        pipeline = catch_errors cache authtoken keystoneauth proxy-server
-
-    Make sure you have the authtoken middleware before the
-    keystoneauth middleware.
+    In Swift's proxy-server.conf add this keystoneauth middleware and the
+    authtoken middleware to your pipeline. Make sure you have the authtoken
+    middleware before the keystoneauth middleware.
 
     The authtoken middleware will take care of validating the user and
     keystoneauth will authorize access.
 
-    The authtoken middleware is shipped directly with keystone it
-    does not have any other dependences than itself so you can either
+    The sample proxy-server.conf shows a sample pipeline that uses keystone.
+
+    :download:`proxy-server.conf-sample </../../etc/proxy-server.conf-sample>`
+
+    The authtoken middleware is shipped with keystonemiddleware - it
+    does not have any other dependencies than itself so you can either
     install it by copying the file directly in your python path or by
-    installing keystone.
+    installing keystonemiddleware.
 
     If support is required for unvalidated users (as with anonymous
     access) or for formpost/staticweb/tempurl middleware, authtoken will
@@ -65,25 +65,70 @@ class KeystoneAuth(object):
         use = egg:swift#keystoneauth
         operator_roles = admin, swiftoperator
 
-    This maps tenants to account in Swift.
-
-    The user whose able to give ACL / create Containers permissions
-    will be the one that are inside the ``operator_roles``
+    The user who is able to give ACL / create Containers permissions
+    will be the user with a role listed in the ``operator_roles``
     setting which by default includes the admin and the swiftoperator
     roles.
 
+    The keystoneauth middleware maps a Keystone project/tenant to an account
+    in Swift by adding a prefix (``AUTH_`` by default) to the tenant/project
+    id.. For example, if the project id is ``1234``, the path is
+    ``/v1/AUTH_1234``.
+
     If you need to have a different reseller_prefix to be able to
     mix different auth servers you can configure the option
     ``reseller_prefix`` in your keystoneauth entry like this::
 
         reseller_prefix = NEWAUTH
 
-    The keystoneauth middleware supports cross-tenant access control using
-    the syntax ``<tenant>:<user>`` to specify a grantee in container Access
-    Control Lists (ACLs). For a request to be granted by an ACL, the grantee
+    Don't forget to also update the Keystone service endpoint configuration to
+    use NEWAUTH in the path.
+
+    It is possible to have several accounts associated with the same project.
+    This is done by listing several prefixes as shown in the following
+    example::
+
+        reseller_prefix = AUTH, SERVICE
+
+    This means that for project id '1234', the paths '/v1/AUTH_1234' and
+    '/v1/SERVICE_1234' are associated with the project and are authorized
+    using roles that a user has with that project. The core use of this feature
+    is that it is possible to provide different rules for each account
+    prefix. The following parameters may be prefixed with the appropriate
+    prefix::
+
+        operator_roles
+        service_roles
+
+    For backward compatibility, if either of these parameters is specified
+    without a prefix then it applies to all reseller_prefixes. Here is an
+    example, using two prefixes::
+
+        reseller_prefix = AUTH, SERVICE
+        # The next three lines have identical effects (since the first applies
+        # to both prefixes).
+        operator_roles = admin, swiftoperator
+        AUTH_operator_roles = admin, swiftoperator
+        SERVICE_operator_roles = admin, swiftoperator
+        # The next line only applies to accounts with the SERVICE prefix
+        SERVICE_operator_roles = admin, some_other_role
+
+    X-Service-Token tokens are supported by the inclusion of the service_roles
+    configuration option. When present, this option requires that the
+    X-Service-Token header supply a token from a user who has a role listed
+    in service_roles. Here is an example configuration::
+
+        reseller_prefix = AUTH, SERVICE
+        AUTH_operator_roles = admin, swiftoperator
+        SERVICE_operator_roles = admin, swiftoperator
+        SERVICE_service_roles = service
+
+    The keystoneauth middleware supports cross-tenant access control using the
+    syntax ``<tenant>:<user>`` to specify a grantee in container Access Control
+    Lists (ACLs). For a request to be granted by an ACL, the grantee
     ``<tenant>`` must match the UUID of the tenant to which the request
-    token is scoped and the grantee ``<user>`` must match the UUID of the
-    user authenticated by the request token.
+    X-Auth-Token is scoped and the grantee ``<user>`` must match the UUID of
+    the user authenticated by that token.
 
     Note that names must no longer be used in cross-tenant ACLs because with
     the introduction of domains in keystone names are no longer globally
@@ -92,7 +137,7 @@ class KeystoneAuth(object):
     For backwards compatibility, ACLs using names will be granted by
     keystoneauth when it can be established that the grantee tenant,
     the grantee user and the tenant being accessed are either not yet in a
-    domain (e.g. the request token has been obtained via the keystone v2
+    domain (e.g. the X-Auth-Token has been obtained via the keystone v2
     API) or are all in the default domain to which legacy accounts would
     have been migrated. The default domain is identified by its UUID,
     which by default has the value ``default``. This can be changed by
@@ -114,6 +159,14 @@ class KeystoneAuth(object):
     keystoneauth will assume that the tenant may not be in the default domain
     and therefore not match names in ACLs for that account.
 
+    By default, middleware higher in the WSGI pipeline may override auth
+    processing, useful for middleware such as tempurl and formpost. If you know
+    you're not going to use such middleware and you want a bit of extra
+    security you can disable this behaviour by setting the ``allow_overrides``
+    option to ``false``::
+
+        allow_overrides = false
+
     :param app: The next WSGI app in the pipeline
     :param conf: The dict of configuration values
     """
@@ -121,15 +174,23 @@ class KeystoneAuth(object):
         self.app = app
         self.conf = conf
         self.logger = swift_utils.get_logger(conf, log_route='keystoneauth')
-        self.reseller_prefix = conf.get('reseller_prefix', 'AUTH_').strip()
-        if self.reseller_prefix and self.reseller_prefix[-1] != '_':
-            self.reseller_prefix += '_'
-        self.operator_roles = conf.get('operator_roles',
-                                       'admin, swiftoperator').lower()
+        self.reseller_prefixes, self.account_rules = \
+            config_read_reseller_options(conf,
+                                         dict(operator_roles=['admin',
+                                                              'swiftoperator'],
+                                              service_roles=[],
+                                              project_reader_roles=[]))
         self.reseller_admin_role = conf.get('reseller_admin_role',
                                             'ResellerAdmin').lower()
+        self.system_reader_roles = {role.lower() for role in list_from_csv(
+            conf.get('system_reader_roles', ''))}
+
         config_is_admin = conf.get('is_admin', "false").lower()
-        self.is_admin = swift_utils.config_true_value(config_is_admin)
+        if swift_utils.config_true_value(config_is_admin):
+            self.logger.warning("The 'is_admin' option for keystoneauth is no "
+                                "longer supported. Remove the 'is_admin' "
+                                "option from your keystoneauth config")
+
         config_overrides = conf.get('allow_overrides', 't').lower()
         self.allow_overrides = swift_utils.config_true_value(config_overrides)
         self.default_domain_id = conf.get('default_domain_id', 'default')
@@ -137,25 +198,24 @@ class KeystoneAuth(object):
             conf.get('allow_names_in_acls', 'true'))
 
     def __call__(self, environ, start_response):
-        identity = self._keystone_identity(environ)
+        env_identity = self._keystone_identity(environ)
 
         # Check if one of the middleware like tempurl or formpost have
         # set the swift.authorize_override environ and want to control the
         # authentication
         if (self.allow_overrides and
                 environ.get('swift.authorize_override', False)):
-            msg = 'Authorizing from an overriding middleware (i.e: tempurl)'
+            msg = 'Authorizing from an overriding middleware'
             self.logger.debug(msg)
             return self.app(environ, start_response)
 
-        if identity:
-            self.logger.debug('Using identity: %r', identity)
-            environ['keystone.identity'] = identity
-            environ['REMOTE_USER'] = identity.get('tenant')
-            env_identity = self._integral_keystone_identity(environ)
+        if env_identity:
+            self.logger.debug('Using identity: %r', env_identity)
+            environ['REMOTE_USER'] = env_identity.get('tenant')
+            environ['keystone.identity'] = env_identity
             environ['swift.authorize'] = functools.partial(
                 self.authorize, env_identity)
-            user_roles = (r.lower() for r in identity.get('roles', []))
+            user_roles = (r.lower() for r in env_identity.get('roles', []))
             if self.reseller_admin_role in user_roles:
                 environ['reseller_request'] = True
         else:
@@ -179,33 +239,20 @@ class KeystoneAuth(object):
 
     def _keystone_identity(self, environ):
         """Extract the identity from the Keystone auth component."""
-        # In next release, we would add user id in env['keystone.identity'] by
-        # using _integral_keystone_identity to replace current
-        # _keystone_identity. The purpose of keeping it in this release it for
-        # back compatibility.
-        if environ.get('HTTP_X_IDENTITY_STATUS') != 'Confirmed':
+        if (environ.get('HTTP_X_IDENTITY_STATUS') != 'Confirmed'
+            or environ.get(
+                'HTTP_X_SERVICE_IDENTITY_STATUS') not in (None, 'Confirmed')):
             return
-        roles = []
-        if 'HTTP_X_ROLES' in environ:
-            roles = environ['HTTP_X_ROLES'].split(',')
-        identity = {'user': environ.get('HTTP_X_USER_NAME'),
-                    'tenant': (environ.get('HTTP_X_TENANT_ID'),
-                               environ.get('HTTP_X_TENANT_NAME')),
-                    'roles': roles}
-        return identity
-
-    def _integral_keystone_identity(self, environ):
-        """Extract the identity from the Keystone auth component."""
-        if environ.get('HTTP_X_IDENTITY_STATUS') != 'Confirmed':
-            return
-        roles = []
-        if 'HTTP_X_ROLES' in environ:
-            roles = environ['HTTP_X_ROLES'].split(',')
+        roles = list_from_csv(environ.get('HTTP_X_ROLES', ''))
+        service_roles = list_from_csv(environ.get('HTTP_X_SERVICE_ROLES', ''))
         identity = {'user': (environ.get('HTTP_X_USER_ID'),
                              environ.get('HTTP_X_USER_NAME')),
-                    'tenant': (environ.get('HTTP_X_TENANT_ID'),
-                               environ.get('HTTP_X_TENANT_NAME')),
-                    'roles': roles}
+                    'tenant': (environ.get('HTTP_X_PROJECT_ID',
+                                           environ.get('HTTP_X_TENANT_ID')),
+                               environ.get('HTTP_X_PROJECT_NAME',
+                                           environ.get('HTTP_X_TENANT_NAME'))),
+                    'roles': roles,
+                    'service_roles': service_roles}
         token_info = environ.get('keystone.token_info', {})
         auth_version = 0
         user_domain = project_domain = (None, None)
@@ -223,17 +270,31 @@ class KeystoneAuth(object):
         identity['auth_version'] = auth_version
         return identity
 
-    def _get_account_for_tenant(self, tenant_id):
-        return '%s%s' % (self.reseller_prefix, tenant_id)
+    def _get_account_name(self, prefix, tenant_id):
+        return '%s%s' % (prefix, tenant_id)
 
-    def _reseller_check(self, account, tenant_id):
-        """Check reseller prefix."""
-        return account == self._get_account_for_tenant(tenant_id)
+    def _account_matches_tenant(self, account, tenant_id):
+        """Check if account belongs to a project/tenant"""
+        for prefix in self.reseller_prefixes:
+            if self._get_account_name(prefix, tenant_id) == account:
+                return True
+        return False
+
+    def _get_account_prefix(self, account):
+        """Get the prefix of an account"""
+        # Empty prefix matches everything, so try to match others first
+        for prefix in [pre for pre in self.reseller_prefixes if pre != '']:
+            if account.startswith(prefix):
+                return prefix
+        if '' in self.reseller_prefixes:
+            return ''
+        return None
 
     def _get_project_domain_id(self, environ):
         info = get_account_info(environ, self.app, 'KS')
         domain_id = info.get('sysmeta', {}).get('project-domain-id')
-        exists = is_success(info.get('status', 0))
+        exists = (is_success(info.get('status', 0))
+                  and info.get('account_really_exists', True))
         return exists, domain_id
 
     def _set_project_domain_id(self, req, path_parts, env_identity):
@@ -250,12 +311,12 @@ class KeystoneAuth(object):
         method = req.method
         if (obj or (container and method != 'PUT')
                 or method not in ['PUT', 'POST']):
-                return
+            return
 
         tenant_id, tenant_name = env_identity['tenant']
         exists, sysmeta_id = self._get_project_domain_id(req.environ)
         req_has_id, req_id, new_id = False, None, None
-        if self._reseller_check(account, tenant_id):
+        if self._account_matches_tenant(account, tenant_id):
             # domain id can be inferred from request (may be None)
             req_has_id = True
             req_id = env_identity['project_domain'][0]
@@ -269,9 +330,9 @@ class KeystoneAuth(object):
             # unknown domain, update if req confirms domain
             new_id = req_id or ''
         elif req_has_id and sysmeta_id != req_id:
-            self.logger.warn("Inconsistent project domain id: " +
-                             "%s in token vs %s in account metadata."
-                             % (req_id, sysmeta_id))
+            self.logger.warning("Inconsistent project domain id: " +
+                                "%s in token vs %s in account metadata."
+                                % (req_id, sysmeta_id))
 
         if new_id is not None:
             req.headers[PROJECT_DOMAIN_ID_SYSMETA_HEADER] = new_id
@@ -290,7 +351,7 @@ class KeystoneAuth(object):
         # request user and scoped project are both in default domain
         tenant_id, tenant_name = identity['tenant']
         version, account, container, obj = path_parts
-        if self._reseller_check(account, tenant_id):
+        if self._account_matches_tenant(account, tenant_id):
             # account == scoped project, so account is also in default domain
             allow = True
         else:
@@ -334,11 +395,15 @@ class KeystoneAuth(object):
         return None
 
     def authorize(self, env_identity, req):
+        # Cleanup - make sure that a previously set swift_owner setting is
+        # cleared now. This might happen for example with COPY requests.
+        req.environ.pop('swift_owner', None)
+
         tenant_id, tenant_name = env_identity['tenant']
         user_id, user_name = env_identity['user']
         referrers, roles = swift_acl.parse_acl(getattr(req, 'acl', None))
 
-        #allow OPTIONS requests to proceed as normal
+        # allow OPTIONS requests to proceed as normal
         if req.method == 'OPTIONS':
             return
 
@@ -351,15 +416,29 @@ class KeystoneAuth(object):
         self._set_project_domain_id(req, part, env_identity)
 
         user_roles = [r.lower() for r in env_identity.get('roles', [])]
+        user_service_roles = [r.lower() for r in env_identity.get(
+                              'service_roles', [])]
 
-        # Give unconditional access to a user with the reseller_admin
-        # role.
+        # Give unconditional access to a user with the reseller_admin role.
         if self.reseller_admin_role in user_roles:
             msg = 'User %s has reseller admin authorizing'
             self.logger.debug(msg, tenant_id)
             req.environ['swift_owner'] = True
             return
 
+        # Being in system_reader_roles is almost as good as reseller_admin.
+        if self.system_reader_roles.intersection(user_roles):
+            # Note that if a system reader is trying to write, we're letting
+            # the request fall on other access checks below. This way,
+            # a compliance auditor can write a log file as a normal member.
+            if req.method in ('GET', 'HEAD'):
+                msg = 'User %s has system reader authorizing'
+                self.logger.debug(msg, tenant_id)
+                # We aren't setting 'swift_owner' nor 'reseller_request'
+                # because they are only ever used for something that modifies
+                # the contents of the cluster (setting ACL, deleting accounts).
+                return
+
         # If we are not reseller admin and user is trying to delete its own
         # account then deny it.
         if not container and not obj and req.method == 'DELETE':
@@ -388,31 +467,54 @@ class KeystoneAuth(object):
 
         # Check if a user tries to access an account that does not match their
         # token
-        if not self._reseller_check(account, tenant_id):
+        if not self._account_matches_tenant(account, tenant_id):
             log_msg = 'tenant mismatch: %s != %s'
             self.logger.debug(log_msg, account, tenant_id)
             return self.denied_response(req)
 
-        # Check the roles the user is belonging to. If the user is
-        # part of the role defined in the config variable
-        # operator_roles (like admin) then it will be
-        # promoted as an admin of the account/tenant.
-        for role in self.operator_roles.split(','):
-            role = role.strip()
-            if role in user_roles:
-                log_msg = 'allow user with role %s as account admin'
-                self.logger.debug(log_msg, role)
-                req.environ['swift_owner'] = True
-                return
-
-        # If user is of the same name of the tenant then make owner of it.
-        if self.is_admin and user_name == tenant_name:
-            self.logger.warning("the is_admin feature has been deprecated "
-                                "and will be removed in the future "
-                                "update your config file")
+        # Compare roles from tokens against the configuration options:
+        #
+        # X-Auth-Token role  Has specified  X-Service-Token role  Grant
+        # in operator_roles? service_roles? in service_roles?     swift_owner?
+        # ------------------ -------------- --------------------  ------------
+        # yes                yes            yes                   yes
+        # yes                yes            no                    no
+        # yes                no             don't care            yes
+        # no                 don't care     don't care            no
+        # ------------------ -------------- --------------------  ------------
+        account_prefix = self._get_account_prefix(account)
+        operator_roles = self.account_rules[account_prefix]['operator_roles']
+        have_operator_role = set(operator_roles).intersection(
+            set(user_roles))
+        service_roles = self.account_rules[account_prefix]['service_roles']
+        have_service_role = set(service_roles).intersection(
+            set(user_service_roles))
+        allowed = False
+        if have_operator_role and (service_roles and have_service_role):
+            allowed = True
+        elif have_operator_role and not service_roles:
+            allowed = True
+        if allowed:
+            log_msg = 'allow user with role(s) %s as account admin'
+            self.logger.debug(log_msg, ','.join(have_operator_role.union(
+                                                have_service_role)))
             req.environ['swift_owner'] = True
             return
 
+        # The project_reader_roles is almost as good as operator_roles. But
+        # it does not work with service tokens and does not get 'swift_owner'.
+        # And, it only serves GET requests, obviously.
+        project_reader_roles = self.account_rules[account_prefix][
+            'project_reader_roles']
+        have_reader_role = set(project_reader_roles).intersection(
+            set(user_roles))
+        if have_reader_role:
+            if req.method in ('GET', 'HEAD'):
+                msg = 'User %s with role(s) %s has project reader authorizing'
+                self.logger.debug(msg, tenant_id,
+                                  ','.join(project_reader_roles))
+                return
+
         if acl_authorized is not None:
             return self.denied_response(req)
 
@@ -438,12 +540,13 @@ class KeystoneAuth(object):
         except ValueError:
             return HTTPNotFound(request=req)
 
-        #allow OPTIONS requests to proceed as normal
+        # allow OPTIONS requests to proceed as normal
         if req.method == 'OPTIONS':
             return
 
         is_authoritative_authz = (account and
-                                  account.startswith(self.reseller_prefix))
+                                  (self._get_account_prefix(account) in
+                                   self.reseller_prefixes))
         if not is_authoritative_authz:
             return self.denied_response(req)
 
@@ -494,7 +597,6 @@ def filter_factory(global_conf, **local_conf):
     """Returns a WSGI filter app for use with paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
-    register_swift_info('keystoneauth')
 
     def auth_filter(app):
         return KeystoneAuth(app, conf)
