@@ -65,16 +65,14 @@ import os
 import re
 import time
 
-import six
-
 from swift.common import constraints
-from swift.common.swob import Range, bytes_to_wsgi, normalize_etag, wsgi_to_str
-from swift.common.utils import json, public, reiterate, md5
-from swift.common.db import utf8encode
+from swift.common.swob import Range, bytes_to_wsgi, normalize_etag, \
+    wsgi_to_str
+from swift.common.utils import json, public, reiterate, md5, Timestamp
 from swift.common.request_helpers import get_container_update_override_key, \
     get_param
 
-from six.moves.urllib.parse import quote, urlparse
+from urllib.parse import quote, urlparse
 
 from swift.common.middleware.s3api.controllers.base import Controller, \
     bucket_operation, object_operation, check_container_existence
@@ -82,8 +80,7 @@ from swift.common.middleware.s3api.s3response import InvalidArgument, \
     ErrorResponse, MalformedXML, BadDigest, KeyTooLongError, \
     InvalidPart, BucketAlreadyExists, EntityTooSmall, InvalidPartOrder, \
     InvalidRequest, HTTPOk, HTTPNoContent, NoSuchKey, NoSuchUpload, \
-    NoSuchBucket, BucketAlreadyOwnedByYou
-from swift.common.middleware.s3api.exception import BadSwiftRequest
+    NoSuchBucket, BucketAlreadyOwnedByYou, ServiceUnavailable
 from swift.common.middleware.s3api.utils import unique_id, \
     MULTIUPLOAD_SUFFIX, S3Timestamp, sysmeta_header
 from swift.common.middleware.s3api.etree import Element, SubElement, \
@@ -97,6 +94,17 @@ MAX_COMPLETE_UPLOAD_BODY_SIZE = 2048 * 1024
 
 
 def _get_upload_info(req, app, upload_id):
+    """
+    Make a HEAD request for existing upload object metadata. Tries the upload
+    marker first, and then falls back to the manifest object.
+
+    :param req: an S3Request object.
+    :param app: the wsgi app.
+    :param upload_id: the upload id.
+    :returns: a tuple of (S3Response, boolean) where the boolean is True if the
+        response is from the upload marker and False otherwise.
+    :raises: NoSuchUpload if neither the marker nor the manifest were found.
+    """
 
     container = req.container_name + MULTIUPLOAD_SUFFIX
     obj = '%s/%s' % (req.object_name, upload_id)
@@ -107,14 +115,17 @@ def _get_upload_info(req, app, upload_id):
     # it off for now...
     copy_source = req.headers.pop('X-Amz-Copy-Source', None)
     try:
-        return req.get_response(app, 'HEAD', container=container, obj=obj)
+        resp = req.get_response(app, 'HEAD', container=container, obj=obj)
+        return resp, True
     except NoSuchKey:
+        # ensure consistent path and policy are logged despite manifest HEAD
         upload_marker_path = req.environ.get('s3api.backend_path')
+        policy_index = req.policy_index
         try:
             resp = req.get_response(app, 'HEAD')
             if resp.sysmeta_headers.get(sysmeta_header(
                     'object', 'upload-id')) == upload_id:
-                return resp
+                return resp, False
         except NoSuchKey:
             pass
         finally:
@@ -122,6 +133,8 @@ def _get_upload_info(req, app, upload_id):
             # path, so put it back
             if upload_marker_path is not None:
                 req.environ['s3api.backend_path'] = upload_marker_path
+            if policy_index is not None:
+                req.policy_index = policy_index
         raise NoSuchUpload(upload_id=upload_id)
     finally:
         # ...making sure to restore any copy-source before returning
@@ -180,15 +193,7 @@ class PartController(Controller):
             raise InvalidArgument('ResourceType', 'partNumber',
                                   'Unexpected query string parameter')
 
-        try:
-            part_number = int(get_param(req, 'partNumber'))
-            if part_number < 1 or self.conf.max_upload_part_num < part_number:
-                raise Exception()
-        except Exception:
-            err_msg = 'Part number must be an integer between 1 and %d,' \
-                      ' inclusive' % self.conf.max_upload_part_num
-            raise InvalidArgument('partNumber', get_param(req, 'partNumber'),
-                                  err_msg)
+        part_number = req.validate_part_number()
 
         upload_id = get_param(req, 'uploadId')
         _get_upload_info(req, self.app, upload_id)
@@ -285,8 +290,6 @@ class UploadsController(Controller):
 
             :return (non_delimited_uploads, common_prefixes)
             """
-            if six.PY2:
-                (prefix, delimiter) = utf8encode(prefix, delimiter)
             non_delimited_uploads = []
             common_prefixes = set()
             for upload in uploads:
@@ -332,7 +335,8 @@ class UploadsController(Controller):
                         'last_modified': object_info['last_modified']}
             return obj_dict
 
-        is_part = re.compile('/[0-9]+$')
+        is_segment = re.compile('.*/[0-9]+$')
+
         while len(uploads) < maxuploads:
             try:
                 resp = req.get_response(self.app, container=container,
@@ -344,8 +348,8 @@ class UploadsController(Controller):
             if not objects:
                 break
 
-            new_uploads = [object_to_upload(obj) for obj in objects if
-                           is_part.search(obj.get('name', '')) is None]
+            new_uploads = [object_to_upload(obj) for obj in objects
+                           if not is_segment.match(obj.get('name', ''))]
             new_prefixes = []
             if 'delimiter' in req.params:
                 prefix = get_param(req, 'prefix', '')
@@ -354,10 +358,7 @@ class UploadsController(Controller):
                     new_uploads, prefix, delimiter)
             uploads.extend(new_uploads)
             prefixes.extend(new_prefixes)
-            if six.PY2:
-                query['marker'] = objects[-1]['name'].encode('utf-8')
-            else:
-                query['marker'] = objects[-1]['name']
+            query['marker'] = objects[-1]['name']
 
         truncated = len(uploads) >= maxuploads
         if len(uploads) > maxuploads:
@@ -533,10 +534,7 @@ class UploadController(Controller):
             if not new_objects:
                 break
             objects.extend(new_objects)
-            if six.PY2:
-                query['marker'] = new_objects[-1]['name'].encode('utf-8')
-            else:
-                query['marker'] = new_objects[-1]['name']
+            query['marker'] = new_objects[-1]['name']
 
         last_part = 0
 
@@ -633,10 +631,7 @@ class UploadController(Controller):
                 container = req.container_name + MULTIUPLOAD_SUFFIX
                 obj = bytes_to_wsgi(o['name'].encode('utf-8'))
                 req.get_response(self.app, container=container, obj=obj)
-            if six.PY2:
-                query['marker'] = objects[-1]['name'].encode('utf-8')
-            else:
-                query['marker'] = objects[-1]['name']
+            query['marker'] = objects[-1]['name']
             resp = req.get_response(self.app, 'GET', container, '',
                                     query=query)
             objects = json.loads(resp.body)
@@ -651,7 +646,14 @@ class UploadController(Controller):
         Handles Complete Multipart Upload.
         """
         upload_id = get_param(req, 'uploadId')
-        resp = _get_upload_info(req, self.app, upload_id)
+        resp, is_marker = _get_upload_info(req, self.app, upload_id)
+        if (is_marker and
+                resp.sw_headers.get('X-Backend-Timestamp') >= Timestamp.now()):
+            # Somehow the marker was created in the future w.r.t. this thread's
+            # clock. The manifest PUT may succeed but the subsequent marker
+            # DELETE will fail, so don't attempt either.
+            raise ServiceUnavailable
+
         headers = {'Accept': 'application/json',
                    sysmeta_header('object', 'upload-id'): upload_id}
         for key, val in resp.headers.items():
@@ -707,11 +709,15 @@ class UploadController(Controller):
                 previous_number = part_number
 
                 etag = normalize_etag(part_elem.find('./ETag').text)
+                if etag is None:
+                    raise InvalidPart(upload_id=upload_id,
+                                      part_number=part_number,
+                                      e_tag=etag)
                 if len(etag) != 32 or any(c not in '0123456789abcdef'
                                           for c in etag):
                     raise InvalidPart(upload_id=upload_id,
-                                      part_number=part_number)
-
+                                      part_number=part_number,
+                                      e_tag=etag)
                 manifest.append({
                     'path': '/%s/%s/%s/%d' % (
                         wsgi_to_str(container), wsgi_to_str(req.object_name),
@@ -800,8 +806,8 @@ class UploadController(Controller):
                                 status=body['Response Status'],
                                 msg='\n'.join(': '.join(err)
                                               for err in body['Errors']))
-                except BadSwiftRequest as e:
-                    msg = str(e)
+                except InvalidRequest as err_resp:
+                    msg = err_resp._msg
                     if too_small_message in msg:
                         raise EntityTooSmall(msg)
                     elif ', Etag Mismatch' in msg:
