@@ -14,10 +14,7 @@
 # limitations under the License.
 
 import re
-try:
-    from collections.abc import MutableMapping
-except ImportError:
-    from collections import MutableMapping  # py2
+from collections.abc import MutableMapping
 from functools import partial
 
 from swift.common import header_key_dict
@@ -398,7 +395,7 @@ class IllegalVersioningConfigurationException(ErrorResponse):
 class IncompleteBody(ErrorResponse):
     _status = '400 Bad Request'
     _msg = 'You did not provide the number of bytes specified by the ' \
-           'Content-Length HTTP header.'
+           'Content-Length HTTP header'
 
 
 class IncorrectNumberOfFilesInPostRequest(ErrorResponse):
@@ -447,6 +444,11 @@ class InvalidBucketState(ErrorResponse):
     _msg = 'The request is not valid with the current state of the bucket.'
 
 
+class InvalidChunkSizeError(ErrorResponse):
+    _status = '403 Forbidden'
+    _msg = 'Only the last chunk is allowed to have a size less than 8192 bytes'
+
+
 class InvalidDigest(ErrorResponse):
     _status = '400 Bad Request'
     _msg = 'The Content-MD5 you specified was invalid.'
@@ -568,6 +570,12 @@ class MalformedPOSTRequest(ErrorResponse):
            'multipart/form-data.'
 
 
+class MalformedTrailerError(ErrorResponse):
+    _status = '400 Bad Request'
+    _msg = 'The request contained trailing data that was not well-formed ' \
+           'or did not conform to our published schema.'
+
+
 class MalformedXML(ErrorResponse):
     _status = '400 Bad Request'
     _msg = 'The XML you provided was not well-formed or did not validate ' \
