@@ -61,8 +61,7 @@ from keystoneclient.v3 import client as keystone_client
 from keystoneauth1 import session as keystone_session
 from keystoneauth1 import loading as keystone_loading
 import requests
-import six
-from six.moves import urllib
+import urllib
 
 from swift.common.swob import Request, HTTPBadRequest, HTTPUnauthorized, \
     HTTPException, str_to_wsgi
@@ -178,49 +177,34 @@ class S3Token(object):
         else:
             self._verify = None
 
-        self._secret_cache_duration = int(
-            conf.get('secret_cache_duration', 60))
+        self._secret_cache_duration = int(conf.get('secret_cache_duration', 0))
         if self._secret_cache_duration < 0:
             raise ValueError('secret_cache_duration must be non-negative')
+        if self._secret_cache_duration:
+            try:
+                auth_plugin = keystone_loading.get_plugin_loader(
+                    conf.get('auth_type', 'password'))
+                available_auth_options = auth_plugin.get_options()
+                auth_options = {}
+                for option in available_auth_options:
+                    name = option.name.replace('-', '_')
+                    value = conf.get(name)
+                    if value:
+                        auth_options[name] = value
 
-        # Service authentication for s3tokens API calls
-        self.keystoneclient = None
-        try:
-            auth_plugin = keystone_loading.get_plugin_loader(
-                conf.get('auth_type', 'password'))
-            available_auth_options = auth_plugin.get_options()
-            auth_options = {}
-            for option in available_auth_options:
-                name = option.name.replace('-', '_')
-                value = conf.get(name)
-                if value:
-                    auth_options[name] = value
-
-            if not auth_options:
-                self._logger.warning(
-                    "No service auth configuration. "
-                    "s3tokens API calls will be unauthenticated. "
-                    "New versions of keystone require service auth.")
-            else:
                 auth = auth_plugin.load_from_options(**auth_options)
                 session = keystone_session.Session(auth=auth)
                 self.keystoneclient = keystone_client.Client(
                     session=session,
                     region_name=conf.get('region_name'))
-                self._logger.info(
-                    "Service authentication configured for s3tokens API")
-        except Exception:
-            self._logger.warning(
-                "Unable to load service auth configuration. "
-                "s3tokens API calls will be unauthenticated "
-                "and secret caching will be unavailable.",
-                exc_info=True)
-
-        if self._secret_cache_duration and self.keystoneclient:
-            self._logger.info("Caching s3tokens for %s seconds",
-                              self._secret_cache_duration)
-        else:
-            self._secret_cache_duration = 0
+                self._logger.info("Caching s3tokens for %s seconds",
+                                  self._secret_cache_duration)
+            except Exception:
+                self._logger.warning("Unable to load keystone auth_plugin. "
+                                     "Secret caching will be unavailable.",
+                                     exc_info=True)
+                self.keystoneclient = None
+                self._secret_cache_duration = 0
 
     def _deny_request(self, code):
         error_cls, message = {
@@ -232,24 +216,12 @@ class S3Token(object):
         error_msg = ('<?xml version="1.0" encoding="UTF-8"?>\r\n'
                      '<Error>\r\n  <Code>%s</Code>\r\n  '
                      '<Message>%s</Message>\r\n</Error>\r\n' %
-                     (code, message))
-        if six.PY3:
-            error_msg = error_msg.encode()
+                     (code, message)).encode()
         resp.body = error_msg
         return resp
 
     def _json_request(self, creds_json):
         headers = {'Content-Type': 'application/json'}
-
-        # Add service authentication headers if configured
-        if self.keystoneclient:
-            try:
-                headers.update(
-                    self.keystoneclient.session.get_auth_headers())
-            except Exception:
-                self._logger.warning("Failed to get service token",
-                                     exc_info=True)
-
         try:
             response = requests.post(self._request_uri,
                                      headers=headers, data=creds_json,
@@ -291,18 +263,18 @@ class S3Token(object):
             return self._app(environ, start_response)
 
         access = s3_auth_details['access_key']
-        if isinstance(access, six.binary_type):
+        if isinstance(access, bytes):
             access = access.decode('utf-8')
 
         signature = s3_auth_details['signature']
-        if isinstance(signature, six.binary_type):
+        if isinstance(signature, bytes):
             signature = signature.decode('utf-8')
 
         string_to_sign = s3_auth_details['string_to_sign']
-        if isinstance(string_to_sign, six.text_type):
+        if isinstance(string_to_sign, str):
             string_to_sign = string_to_sign.encode('utf-8')
         token = base64.urlsafe_b64encode(string_to_sign)
-        if isinstance(token, six.binary_type):
+        if isinstance(token, bytes):
             token = token.decode('ascii')
 
         # NOTE(chmou): This is to handle the special case with nova
@@ -424,8 +396,6 @@ class S3Token(object):
 
         req.headers.update(headers)
         tenant_to_connect = force_tenant or tenant['id']
-        if six.PY2 and isinstance(tenant_to_connect, six.text_type):
-            tenant_to_connect = tenant_to_connect.encode('utf-8')
         self._logger.debug('Connecting with tenant: %s', tenant_to_connect)
         new_tenant_name = '%s%s' % (self._reseller_prefix, tenant_to_connect)
         environ['PATH_INFO'] = environ['PATH_INFO'].replace(
