@@ -173,7 +173,6 @@ To generate a curl command line from the above::
     '
 """
 
-from __future__ import print_function
 
 import json
 from time import time
@@ -182,11 +181,12 @@ from uuid import uuid4
 import base64
 
 from eventlet import Timeout
-import six
 from swift.common.memcached import MemcacheConnectionError
-from swift.common.swob import Response, Request, wsgi_to_str
-from swift.common.swob import HTTPBadRequest, HTTPForbidden, HTTPNotFound, \
-    HTTPUnauthorized, HTTPMethodNotAllowed, HTTPServiceUnavailable
+from swift.common.swob import (
+    Response, Request, wsgi_to_str, str_to_wsgi, wsgi_unquote,
+    HTTPBadRequest, HTTPForbidden, HTTPNotFound,
+    HTTPUnauthorized, HTTPMethodNotAllowed, HTTPServiceUnavailable,
+)
 
 from swift.common.request_helpers import get_sys_meta_prefix
 from swift.common.middleware.acl import (
@@ -244,12 +244,9 @@ class TempAuth(object):
                     # Because trailing equal signs would screw up config file
                     # parsing, we auto-pad with '=' chars.
                     account += '=' * (len(account) % 4)
-                    account = base64.b64decode(account)
+                    account = base64.b64decode(account).decode('utf8')
                     username += '=' * (len(username) % 4)
-                    username = base64.b64decode(username)
-                    if not six.PY2:
-                        account = account.decode('utf8')
-                        username = username.decode('utf8')
+                    username = base64.b64decode(username).decode('utf8')
                 values = conf[conf_key].split()
                 if not values:
                     raise ValueError('%s has no key set' % conf_key)
@@ -449,8 +446,6 @@ class TempAuth(object):
             expires, groups = cached_auth_data
             if expires < time():
                 groups = None
-            elif six.PY2:
-                groups = groups.encode('utf8')
 
         s3_auth_details = env.get('s3api.auth_details') or\
             env.get('swift3.auth_details')
@@ -469,7 +464,7 @@ class TempAuth(object):
             if not s3_auth_details['check_signature'](user['key']):
                 return None
             env['PATH_INFO'] = env['PATH_INFO'].replace(
-                account_user, account_id, 1)
+                str_to_wsgi(account_user), wsgi_unquote(account_id), 1)
             groups = self._get_user_groups(account, account_user, account_id)
 
         return groups
@@ -528,7 +523,7 @@ class TempAuth(object):
             if not isinstance(result[key], list):
                 return "Value for key %s must be a list" % json.dumps(key)
             for grantee in result[key]:
-                if not isinstance(grantee, six.string_types):
+                if not isinstance(grantee, str):
                     return "Elements of %s list must be strings" % json.dumps(
                         key)
 
@@ -804,24 +799,23 @@ class TempAuth(object):
                 key = req.headers.get('x-storage-pass')
         else:
             return HTTPBadRequest(request=req)
+        unauthed_headers = {
+            'Www-Authenticate': 'Swift realm="%s"' % (account or 'unknown'),
+        }
         if not all((account, user, key)):
             self.logger.increment('token_denied')
-            realm = account or 'unknown'
-            return HTTPUnauthorized(request=req, headers={'Www-Authenticate':
-                                                          'Swift realm="%s"' %
-                                                          realm})
+            return HTTPUnauthorized(request=req, headers=unauthed_headers)
         # Authenticate user
+        account = wsgi_to_str(account)
+        user = wsgi_to_str(user)
+        key = wsgi_to_str(key)
         account_user = account + ':' + user
         if account_user not in self.users:
             self.logger.increment('token_denied')
-            auth = 'Swift realm="%s"' % account
-            return HTTPUnauthorized(request=req,
-                                    headers={'Www-Authenticate': auth})
+            return HTTPUnauthorized(request=req, headers=unauthed_headers)
         if self.users[account_user]['key'] != key:
             self.logger.increment('token_denied')
-            auth = 'Swift realm="unknown"'
-            return HTTPUnauthorized(request=req,
-                                    headers={'Www-Authenticate': auth})
+            return HTTPUnauthorized(request=req, headers=unauthed_headers)
         account_id = self.users[account_user]['url'].rsplit('/', 1)[-1]
         # Get memcache client
         memcache_client = cache_from_env(req.environ)
@@ -837,8 +831,7 @@ class TempAuth(object):
             cached_auth_data = memcache_client.get(memcache_token_key)
             if cached_auth_data:
                 expires, old_groups = cached_auth_data
-                old_groups = [group.encode('utf8') if six.PY2 else group
-                              for group in old_groups.split(',')]
+                old_groups = [group for group in old_groups.split(',')]
                 new_groups = self._get_user_groups(account, account_user,
                                                    account_id)
 
