@@ -145,11 +145,9 @@ the option ``allow_object_versioning`` to ``True``.
 import calendar
 import itertools
 import json
-import six
 import time
 
-from cgi import parse_header
-from six.moves.urllib.parse import unquote
+from urllib.parse import unquote
 
 from swift.common.constraints import MAX_FILE_SIZE, valid_api_version, \
     ACCOUNT_LISTING_LIMIT, CONTAINER_LISTING_LIMIT
@@ -158,6 +156,7 @@ from swift.common.http import is_success, is_client_error, HTTP_NOT_FOUND, \
 from swift.common.request_helpers import get_sys_meta_prefix, \
     copy_header_subset, get_reserved_name, split_reserved_name, \
     constrain_req_limit
+from swift.common.middleware import app_property
 from swift.common.middleware.symlink import TGT_OBJ_SYMLINK_HDR, \
     TGT_ETAG_SYSMETA_SYMLINK_HDR, SYMLOOP_EXTEND, ALLOW_RESERVED_NAMES, \
     TGT_BYTES_SYSMETA_SYMLINK_HDR, TGT_ACCT_SYMLINK_HDR
@@ -165,11 +164,11 @@ from swift.common.swob import HTTPPreconditionFailed, HTTPServiceUnavailable, \
     HTTPBadRequest, str_to_wsgi, bytes_to_wsgi, wsgi_quote, \
     wsgi_to_str, wsgi_unquote, Request, HTTPNotFound, HTTPException, \
     HTTPRequestEntityTooLarge, HTTPInternalServerError, HTTPNotAcceptable, \
-    HTTPConflict
+    HTTPConflict, HTTPLengthRequired
 from swift.common.storage_policy import POLICIES
 from swift.common.utils import get_logger, Timestamp, drain_and_close, \
     config_true_value, close_if_possible, closing_if_possible, \
-    FileLikeIter, split_path, parse_content_type, RESERVED_STR
+    FileLikeIter, split_path, parse_content_type, parse_header, RESERVED_STR
 from swift.common.wsgi import WSGIContext, make_pre_authed_request
 from swift.proxy.controllers.base import get_container_info
 
@@ -672,8 +671,10 @@ class ObjectContext(ObjectVersioningContext):
         Handle a PUT?version-id request and create/update the is_latest link to
         point to the specific version. Expects a valid 'version' id.
         """
-        if req.content_length is None:
+        if req.is_chunked:
             has_body = (req.body_file.read(1) != b'')
+        elif req.content_length is None:
+            raise HTTPLengthRequired(request=req)
         else:
             has_body = (req.content_length != 0)
         if has_body:
@@ -1165,12 +1166,22 @@ class ContainerContext(ObjectVersioningContext):
             params['prefix'] = get_reserved_name(params['prefix'])
 
         # NB: no end_marker support (yet)
-        versions_req.params = {
-            k: params.get(k, '')
-            for k in ('prefix', 'marker', 'limit', 'delimiter', 'reverse')}
-        versions_resp = versions_req.get_response(self.app)
+        if get_container_info(versions_req.environ, self.app,
+                              swift_source='OV')['status'] == 404:
+            # we don't usually like to LBYL like this, but 404s tend to be
+            # expensive (since we check all primaries and a bunch of handoffs)
+            # and we expect this to be a reasonably common way to listing
+            # objects since it's more complete from the user's perspective
+            # (see also: s3api and that client ecosystem)
+            versions_resp = None
+        else:
+            versions_req.params = {
+                k: params.get(k, '') for k in (
+                    'prefix', 'marker', 'limit', 'delimiter', 'reverse')}
+            versions_resp = versions_req.get_response(self.app)
 
-        if versions_resp.status_int == HTTP_NOT_FOUND:
+        if versions_resp is None \
+                or versions_resp.status_int == HTTP_NOT_FOUND:
             subdir_listing = [{'subdir': s} for s in subdir_set]
             broken_listing = []
             for item in current_versions.values():
@@ -1179,7 +1190,6 @@ class ContainerContext(ObjectVersioningContext):
                 name, ts = self._split_version_from_name(linked_name)
                 if ts is None:
                     continue
-                name = name.decode('utf8') if six.PY2 else name
                 is_latest = False
                 if name not in is_latest_set:
                     is_latest_set.add(name)
@@ -1220,8 +1230,7 @@ class ContainerContext(ObjectVersioningContext):
                         path = '/v1/%s/%s/%s' % (
                             wsgi_to_str(account),
                             wsgi_to_str(location),
-                            item['name'].encode('utf8')
-                            if six.PY2 else item['name'])
+                            item['name'])
 
                         if path in current_versions:
                             item['is_latest'] = True
@@ -1249,7 +1258,7 @@ class ContainerContext(ObjectVersioningContext):
                     if ts is None:
                         continue
                     broken_listing.append({
-                        'name': name.decode('utf8') if six.PY2 else name,
+                        'name': name,
                         'is_latest': True,
                         'version_id': ts.internal,
                         'content_type': item['content_type'],
@@ -1379,6 +1388,12 @@ class ObjectVersioningMiddleware(object):
         self.conf = conf
         self.logger = get_logger(conf, log_route='object_versioning')
 
+    # Pass these along so get_container_info will have the configured
+    # odds to skip cache
+    _pipeline_final_app = app_property('_pipeline_final_app')
+    _pipeline_request_logging_app = app_property(
+        '_pipeline_request_logging_app')
+
     def account_request(self, req, api_version, account, start_response):
         account_ctx = AccountContext(self.app, self.logger)
         if req.method == 'GET':
