@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
 
 import errno
 import hashlib
@@ -24,18 +23,14 @@ import socket
 import stat
 import string
 import sys
-import functools
 import time
-import warnings
 import fcntl
 import eventlet
-import six
 import datetime
 
 from swift.common.utils.base import md5, quote, split_path
 from swift.common.utils.timestamp import UTC
 from swift.common.utils.config import config_true_value
-from swift.common import statsd_client
 # common.utils imports a fully qualified common.exceptions so that
 # common.exceptions can import common.utils with out a circular import error
 # (if we only make reference to attributes of a module w/i our function/method
@@ -44,11 +39,8 @@ from swift.common import statsd_client
 # we do the same here
 import swift.common.exceptions
 
-if six.PY2:
-    from eventlet.green import httplib as green_http_client
-else:
-    from eventlet.green.http import client as green_http_client
-from six.moves import http_client
+from eventlet.green.http import client as green_http_client
+import http.client
 from eventlet.green import threading
 
 
@@ -175,13 +167,13 @@ class PipeMutex(object):
 
     def __del__(self):
         # We need this so we don't leak file descriptors. Otherwise, if you
-        # call get_logger() and don't explicitly dispose of it by calling
+        # call get_swift_logger() and don't explicitly dispose of it by calling
         # logger.logger.handlers[0].lock.close() [1], the pipe file
         # descriptors are leaked.
         #
         # This only really comes up in tests. Swift processes tend to call
-        # get_logger() once and then hang on to it until they exit, but the
-        # test suite calls get_logger() a lot.
+        # get_swift_logger() once and then hang on to it until they exit,
+        # but the test suite calls get_swift_logger() a lot.
         #
         # [1] and that's a completely ridiculous thing to expect callers to
         # do, so nobody does it and that's okay.
@@ -243,51 +235,27 @@ class ThreadSafeSysLogHandler(SysLogHandler):
 
 
 # double inheritance to support property with setter
-class LogAdapter(logging.LoggerAdapter, object):
+class SwiftLogAdapter(logging.LoggerAdapter, object):
     """
-    A Logger like object which performs some reformatting on calls to
-    :meth:`exception`.  Can be used to store a threadlocal transaction id and
-    client ip.
+    A LogAdapter that modifies the adapted ``Logger`` instance
+    in the following ways:
+
+    * Performs some reformatting on calls to :meth:`exception`.
+    * Provides threadlocal txn_id and client_ip attributes.
+    * Adds the txn_id, client_ip and server attributes to the ``extras`` dict
+      when a message is processed.
+    * Adds the given prefix to the start of each log message.
+    * Provides a notice method for logging at NOTICE level.
     """
 
     _cls_thread_local = threading.local()
 
-    def __init__(self, logger, server):
+    def __init__(self, logger, server, prefix=''):
         logging.LoggerAdapter.__init__(self, logger, {})
+        self.prefix = prefix
         self.server = server
         self.warn = self.warning
 
-    # There are a few properties needed for py35; see
-    # - https://bugs.python.org/issue31457
-    # - https://github.com/python/cpython/commit/1bbd482
-    # - https://github.com/python/cpython/commit/0b6a118
-    # - https://github.com/python/cpython/commit/ce9e625
-    def _log(self, level, msg, args, exc_info=None, extra=None,
-             stack_info=False):
-        """
-        Low-level log implementation, proxied to allow nested logger adapters.
-        """
-        return self.logger._log(
-            level,
-            msg,
-            args,
-            exc_info=exc_info,
-            extra=extra,
-            stack_info=stack_info,
-        )
-
-    @property
-    def manager(self):
-        return self.logger.manager
-
-    @manager.setter
-    def manager(self, value):
-        self.logger.manager = value
-
-    @property
-    def name(self):
-        return self.logger.name
-
     @property
     def txn_id(self):
         if hasattr(self._cls_thread_local, 'txn_id'):
@@ -314,15 +282,13 @@ class LogAdapter(logging.LoggerAdapter, object):
     def thread_locals(self, value):
         self.txn_id, self.client_ip = value
 
-    def getEffectiveLevel(self):
-        return self.logger.getEffectiveLevel()
-
     def process(self, msg, kwargs):
         """
         Add extra info to message
         """
         kwargs['extra'] = {'server': self.server, 'txn_id': self.txn_id,
                            'client_ip': self.client_ip}
+        msg = '%s%s' % (self.prefix, msg)
         return msg, kwargs
 
     def notice(self, msg, *args, **kwargs):
@@ -335,13 +301,15 @@ class LogAdapter(logging.LoggerAdapter, object):
         self.log(NOTICE, msg, *args, **kwargs)
 
     def _exception(self, msg, *args, **kwargs):
+        # We up-call to exception() where stdlib uses error() so we can get
+        # some of the traceback suppression from LogAdapter, below
         logging.LoggerAdapter.exception(self, msg, *args, **kwargs)
 
     def exception(self, msg, *args, **kwargs):
         _junk, exc, _junk = sys.exc_info()
         call = self.error
         emsg = ''
-        if isinstance(exc, (http_client.BadStatusLine,
+        if isinstance(exc, (http.client.BadStatusLine,
                             green_http_client.BadStatusLine)):
             # Use error(); not really exceptional
             emsg = repr(exc)
@@ -378,51 +346,6 @@ class LogAdapter(logging.LoggerAdapter, object):
             call = self._exception
         call('%s: %s' % (msg, emsg), *args, **kwargs)
 
-    def set_statsd_prefix(self, prefix):
-        """
-        This method is deprecated. Callers should use the
-        ``statsd_tail_prefix`` argument of ``get_logger`` when instantiating a
-        logger.
-
-        The StatsD client prefix defaults to the "name" of the logger.  This
-        method may override that default with a specific value.  Currently used
-        in the proxy-server to differentiate the Account, Container, and Object
-        controllers.
-        """
-        warnings.warn(
-            'set_statsd_prefix() is deprecated; use the '
-            '``statsd_tail_prefix`` argument to ``get_logger`` instead.',
-            DeprecationWarning, stacklevel=2
-        )
-        if self.logger.statsd_client:
-            self.logger.statsd_client._set_prefix(prefix)
-
-    def statsd_delegate(statsd_func_name):
-        """
-        Factory to create methods which delegate to methods on
-        self.logger.statsd_client (an instance of StatsdClient).  The
-        created methods conditionally delegate to a method whose name is given
-        in 'statsd_func_name'.  The created delegate methods are a no-op when
-        StatsD logging is not configured.
-
-        :param statsd_func_name: the name of a method on StatsdClient.
-        """
-        func = getattr(statsd_client.StatsdClient, statsd_func_name)
-
-        @functools.wraps(func)
-        def wrapped(self, *a, **kw):
-            if getattr(self.logger, 'statsd_client'):
-                func = getattr(self.logger.statsd_client, statsd_func_name)
-                return func(*a, **kw)
-        return wrapped
-
-    update_stats = statsd_delegate('update_stats')
-    increment = statsd_delegate('increment')
-    decrement = statsd_delegate('decrement')
-    timing = statsd_delegate('timing')
-    timing_since = statsd_delegate('timing_since')
-    transfer_rate = statsd_delegate('transfer_rate')
-
 
 class SwiftLogFormatter(logging.Formatter):
     """
@@ -539,9 +462,8 @@ class LoggerFileObject(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         raise IOError(errno.EBADF, 'Bad file descriptor')
-    __next__ = next
 
     def read(self, size=-1):
         raise IOError(errno.EBADF, 'Bad file descriptor')
@@ -556,73 +478,6 @@ class LoggerFileObject(object):
         return self
 
 
-class SwiftLoggerAdapter(logging.LoggerAdapter):
-    """
-    A logging.LoggerAdapter subclass that also passes through StatsD method
-    calls.
-
-    Like logging.LoggerAdapter, you have to subclass this and override the
-    process() method to accomplish anything useful.
-    """
-
-    @property
-    def name(self):
-        # py3 does this for us already; add it for py2
-        return self.logger.name
-
-    def update_stats(self, *a, **kw):
-        return self.logger.update_stats(*a, **kw)
-
-    def increment(self, *a, **kw):
-        return self.logger.increment(*a, **kw)
-
-    def decrement(self, *a, **kw):
-        return self.logger.decrement(*a, **kw)
-
-    def timing(self, *a, **kw):
-        return self.logger.timing(*a, **kw)
-
-    def timing_since(self, *a, **kw):
-        return self.logger.timing_since(*a, **kw)
-
-    def transfer_rate(self, *a, **kw):
-        return self.logger.transfer_rate(*a, **kw)
-
-    @property
-    def thread_locals(self):
-        return self.logger.thread_locals
-
-    @thread_locals.setter
-    def thread_locals(self, thread_locals):
-        self.logger.thread_locals = thread_locals
-
-    def exception(self, msg, *a, **kw):
-        # We up-call to exception() where stdlib uses error() so we can get
-        # some of the traceback suppression from LogAdapter, below
-        self.logger.exception(msg, *a, **kw)
-
-
-class PrefixLoggerAdapter(SwiftLoggerAdapter):
-    """
-    Adds an optional prefix to all its log messages. When the prefix has not
-    been set, messages are unchanged.
-    """
-
-    def set_prefix(self, prefix):
-        self.extra['prefix'] = prefix
-
-    def exception(self, msg, *a, **kw):
-        if 'prefix' in self.extra:
-            msg = self.extra['prefix'] + msg
-        super(PrefixLoggerAdapter, self).exception(msg, *a, **kw)
-
-    def process(self, msg, kwargs):
-        msg, kwargs = super(PrefixLoggerAdapter, self).process(msg, kwargs)
-        if 'prefix' in self.extra:
-            msg = self.extra['prefix'] + msg
-        return (msg, kwargs)
-
-
 class LogLevelFilter(object):
     """
     Drop messages for the logger based on level.
@@ -643,8 +498,8 @@ class LogLevelFilter(object):
         return 1
 
 
-def get_logger(conf, name=None, log_to_console=False, log_route=None,
-               fmt="%(server)s: %(message)s", statsd_tail_prefix=None):
+def get_swift_logger(conf, name=None, log_to_console=False, log_route=None,
+                     fmt="%(server)s: %(message)s"):
     """
     Get the current system logger using config settings.
 
@@ -659,10 +514,10 @@ def get_logger(conf, name=None, log_to_console=False, log_route=None,
         log_address = /dev/log
 
     :param conf: Configuration dict to read settings from
-    :param name: This value is used to populate the ``server`` field in the log
-                 format, as the prefix for statsd messages, and as the default
-                 value for ``log_route``; defaults to the ``log_name`` value in
-                 ``conf``, if it exists, or to 'swift'.
+    :param name: This value is used to populate the ``server`` field in
+                 the log format, as the default value for ``log_route``;
+                 defaults to the ``log_name`` value in ``conf``, if it exists,
+                 or to 'swift'.
     :param log_to_console: Add handler which writes to console on stderr
     :param log_route: Route for the logging, not emitted to the log, just used
                       to separate logging configurations; defaults to the value
@@ -670,13 +525,11 @@ def get_logger(conf, name=None, log_to_console=False, log_route=None,
                       is used as the name attribute of the
                       ``logging.LogAdapter`` that is returned.
     :param fmt: Override log format
-    :param statsd_tail_prefix: tail prefix to pass to statsd client; if None
-        then the tail prefix defaults to the value of ``name``.
-    :return: an instance of ``LogAdapter``
+    :return: an instance of ``SwiftLogAdapter``
     """
     # note: log_name is typically specified in conf (i.e. defined by
     # operators), whereas log_route is typically hard-coded in callers of
-    # get_logger (i.e. defined by developers)
+    # get_swift_logger (i.e. defined by developers)
     if not conf:
         conf = {}
     if name is None:
@@ -689,11 +542,11 @@ def get_logger(conf, name=None, log_to_console=False, log_route=None,
     formatter = SwiftLogFormatter(
         fmt=fmt, max_line_length=int(conf.get('log_max_line_length', 0)))
 
-    # get_logger will only ever add one SysLog Handler to a logger
-    if not hasattr(get_logger, 'handler4logger'):
-        get_logger.handler4logger = {}
-    if logger in get_logger.handler4logger:
-        logger.removeHandler(get_logger.handler4logger[logger])
+    # get_swift_logger will only ever add one SysLog Handler to a logger
+    if not hasattr(get_swift_logger, 'handler4logger'):
+        get_swift_logger.handler4logger = {}
+    if logger in get_swift_logger.handler4logger:
+        logger.removeHandler(get_swift_logger.handler4logger[logger])
 
     # facility for this logger will be set by last call wins
     facility = getattr(SysLogHandler, conf.get('log_facility', 'LOG_LOCAL0'),
@@ -722,32 +575,27 @@ def get_logger(conf, name=None, log_to_console=False, log_route=None,
             handler = ThreadSafeSysLogHandler(facility=facility)
     handler.setFormatter(formatter)
     logger.addHandler(handler)
-    get_logger.handler4logger[logger] = handler
+    get_swift_logger.handler4logger[logger] = handler
 
     # setup console logging
-    if log_to_console or hasattr(get_logger, 'console_handler4logger'):
+    if log_to_console or hasattr(get_swift_logger, 'console_handler4logger'):
         # remove pre-existing console handler for this logger
-        if not hasattr(get_logger, 'console_handler4logger'):
-            get_logger.console_handler4logger = {}
-        if logger in get_logger.console_handler4logger:
-            logger.removeHandler(get_logger.console_handler4logger[logger])
+        if not hasattr(get_swift_logger, 'console_handler4logger'):
+            get_swift_logger.console_handler4logger = {}
+        if logger in get_swift_logger.console_handler4logger:
+            logger.removeHandler(
+                get_swift_logger.console_handler4logger[logger])
 
         console_handler = logging.StreamHandler(sys.__stderr__)
         console_handler.setFormatter(formatter)
         logger.addHandler(console_handler)
-        get_logger.console_handler4logger[logger] = console_handler
+        get_swift_logger.console_handler4logger[logger] = console_handler
 
     # set the level for the logger
     logger.setLevel(
         getattr(logging, conf.get('log_level', 'INFO').upper(), logging.INFO))
 
-    # Setup logger with a StatsD client if so configured
-    if statsd_tail_prefix is None:
-        statsd_tail_prefix = name
-    logger.statsd_client = statsd_client.get_statsd_client(
-        conf, statsd_tail_prefix, logger)
-
-    adapted_logger = LogAdapter(logger, name)
+    adapted_logger = SwiftLogAdapter(logger, name)
     other_handlers = conf.get('log_custom_handlers', None)
     if other_handlers:
         log_custom_handlers = [s.strip() for s in other_handlers.split(',')
@@ -768,6 +616,19 @@ def get_logger(conf, name=None, log_to_console=False, log_route=None,
     return adapted_logger
 
 
+def get_prefixed_swift_logger(swift_logger, prefix):
+    """
+    Return a clone of the given ``swift_logger`` with a new prefix string
+    that replaces the prefix string of the given ``swift_logger``.
+
+    :param swift_logger: an instance of ``SwiftLogAdapter``.
+    :param prefix: a string prefix.
+    :returns: a new instance of ``SwiftLogAdapter``.
+    """
+    return SwiftLogAdapter(
+        swift_logger.logger, swift_logger.server, prefix=prefix)
+
+
 class NullLogger(object):
     """A no-op logger for eventlet wsgi."""
 
@@ -810,7 +671,7 @@ def capture_stdio(logger, **kwargs):
     # collect stdio file desc not in use for logging
     stdio_files = [sys.stdin, sys.stdout, sys.stderr]
     console_fds = [h.stream.fileno() for _junk, h in getattr(
-        get_logger, 'console_handler4logger', {}).items()]
+        get_swift_logger, 'console_handler4logger', {}).items()]
     stdio_files = [f for f in stdio_files if f.fileno() not in console_fds]
 
     with open(os.devnull, 'r+b') as nullfile:
@@ -841,8 +702,7 @@ class StrAnonymizer(str):
 
     def __new__(cls, data, method, salt):
         method = method.lower()
-        if method not in (hashlib.algorithms if six.PY2 else
-                          hashlib.algorithms_guaranteed):
+        if method not in hashlib.algorithms_guaranteed:
             raise ValueError('Unsupported hashing method: %r' % method)
         s = str.__new__(cls, data or '')
         s.method = method
@@ -859,8 +719,8 @@ class StrAnonymizer(str):
             else:
                 h = getattr(hashlib, self.method)()
             if self.salt:
-                h.update(six.b(self.salt))
-            h.update(six.b(self))
+                h.update(self.salt.encode('latin1'))
+            h.update(self.encode('latin1'))
             return '{%s%s}%s' % ('S' if self.salt else '', self.method.upper(),
                                  h.hexdigest())
 
@@ -977,7 +837,7 @@ def get_policy_index(req_headers, res_headers):
     """
     header = 'X-Backend-Storage-Policy-Index'
     policy_index = res_headers.get(header, req_headers.get(header))
-    if isinstance(policy_index, six.binary_type) and not six.PY2:
+    if isinstance(policy_index, bytes):
         policy_index = policy_index.decode('ascii')
     return str(policy_index) if policy_index is not None else None
 
