@@ -15,26 +15,23 @@
 
 """WSGI tools for use with swift."""
 
-from __future__ import print_function
 
 import errno
-import fcntl
+import json
 import os
 import signal
-from swift import gettext_ as _
+import struct
 import sys
 from textwrap import dedent
 import time
+import warnings
 
 import eventlet
 import eventlet.debug
 from eventlet import greenio, GreenPool, sleep, wsgi, listen, Timeout
 from paste.deploy import loadwsgi
 from eventlet.green import socket, ssl, os as green_os
-from io import BytesIO
-
-import six
-from six import StringIO
+from io import BytesIO, StringIO
 
 from swift.common import utils, constraints
 from swift.common.http_protocol import SwiftHttpProtocol, \
@@ -49,6 +46,7 @@ from swift.common.utils import capture_stdio, disable_fallocate, \
 SIGNUM_TO_NAME = {getattr(signal, n): n for n in dir(signal)
                   if n.startswith('SIG') and '_' not in n}
 NOTIFY_FD_ENV_KEY = '__SWIFT_SERVER_NOTIFY_FD'
+CHILD_STATE_FD_ENV_KEY = '__SWIFT_SERVER_CHILD_STATE_FD'
 
 # Set maximum line size of message headers to be accepted.
 wsgi.MAX_HEADER_LINE = constraints.MAX_HEADER_SIZE
@@ -67,8 +65,7 @@ class NamedConfigLoader(loadwsgi.ConfigLoader):
     """
 
     def get_context(self, object_type, name=None, global_conf=None):
-        if not six.PY2:
-            self.parser._interpolation = NicerInterpolation()
+        self.parser._interpolation = NicerInterpolation()
         context = super(NamedConfigLoader, self).get_context(
             object_type, name=name, global_conf=global_conf)
         context.name = name
@@ -128,7 +125,7 @@ class ConfigString(NamedConfigLoader):
         self.parser.optionxform = str  # Don't lower-case keys
         # Defaults don't need interpolation (crazy PasteDeploy...)
         self.parser.defaults = lambda: dict(self.parser._defaults, **defaults)
-        self.parser.readfp(self.contents)
+        self.parser.read_file(self.contents)
 
     def readline(self, *args, **kwargs):
         return self.contents.readline(*args, **kwargs)
@@ -195,27 +192,29 @@ def get_socket(conf):
             sock = listen(bind_addr, backlog=int(conf.get('backlog', 4096)),
                           family=address_family)
             if 'cert_file' in conf:
+                context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
+                context.verify_mode = ssl.CERT_NONE
+                context.load_cert_chain(conf['cert_file'], conf['key_file'])
                 warn_ssl = True
-                sock = ssl.wrap_socket(sock, certfile=conf['cert_file'],
-                                       keyfile=conf['key_file'])
+                sock = context.wrap_socket(sock, server_side=True)
         except socket.error as err:
             if err.args[0] != errno.EADDRINUSE:
                 raise
             sleep(0.1)
     if not sock:
-        raise Exception(_('Could not bind to %(addr)s:%(port)s '
-                          'after trying for %(timeout)s seconds') % {
-                              'addr': bind_addr[0], 'port': bind_addr[1],
-                              'timeout': bind_timeout})
+        raise Exception('Could not bind to %(addr)s:%(port)s '
+                        'after trying for %(timeout)s seconds' % {
+                            'addr': bind_addr[0], 'port': bind_addr[1],
+                            'timeout': bind_timeout})
     # in my experience, sockets can hang around forever without keepalive
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
     sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
     if hasattr(socket, 'TCP_KEEPIDLE'):
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, keepidle)
     if warn_ssl:
-        ssl_warning_message = _('WARNING: SSL should only be enabled for '
-                                'testing purposes. Use external SSL '
-                                'termination for a production deployment.')
+        ssl_warning_message = ('WARNING: SSL should only be enabled for '
+                               'testing purposes. Use external SSL '
+                               'termination for a production deployment.')
         get_logger(conf).warning(ssl_warning_message)
         print(ssl_warning_message)
     return sock
@@ -357,12 +356,30 @@ def loadapp(conf_file, global_conf=None, allow_modify_pipeline=True):
         if func and allow_modify_pipeline:
             func(PipelineWrapper(ctx))
         filters = [c.create() for c in reversed(ctx.filter_contexts)]
-        app = ultimate_app
-        app._pipeline_final_app = ultimate_app
+        pipeline = [ultimate_app]
+        request_logging_app = app = ultimate_app
         for filter_app in filters:
-            app = filter_app(app)
+            app = filter_app(pipeline[0])
+            pipeline.insert(0, app)
+            if request_logging_app is ultimate_app and \
+                    app.__class__.__name__ == 'ProxyLoggingMiddleware':
+                request_logging_app = filter_app(ultimate_app)
+                # Set some separate-pipeline attrs
+                request_logging_app._pipeline = [
+                    request_logging_app, ultimate_app]
+                request_logging_app._pipeline_request_logging_app = \
+                    request_logging_app
+                request_logging_app._pipeline_final_app = ultimate_app
+
+        for app in pipeline:
+            app._pipeline = pipeline
+            # For things like making (logged) backend requests for
+            # get_account_info and get_container_info
+            app._pipeline_request_logging_app = request_logging_app
+            # For getting proxy-server options like *_existence_skip_cache_pct
             app._pipeline_final_app = ultimate_app
-        return app
+
+        return pipeline[0]
     return ctx.create()
 
 
@@ -426,14 +443,23 @@ def run_server(conf, logger, sock, global_conf=None, ready_callback=None,
         # header; "Etag" just won't do).
         'capitalize_response_headers': False,
     }
+    if conf.get('keepalive_timeout'):
+        server_kwargs['keepalive'] = float(conf['keepalive_timeout']) or False
+
     if ready_callback:
         ready_callback()
+    # Yes, eventlet, we know -- we have to support bad clients, though
+    warnings.filterwarnings(
+        'ignore', message='capitalize_response_headers is disabled')
     try:
         wsgi.server(sock, app, wsgi_logger, **server_kwargs)
     except socket.error as err:
         if err.errno != errno.EINVAL:
             raise
-    pool.waitall()
+    finally:
+        pool.waitall()
+        if hasattr(app._pipeline_final_app, 'watchdog'):
+            app._pipeline_final_app.watchdog.kill()
 
 
 class StrategyBase(object):
@@ -450,6 +476,13 @@ class StrategyBase(object):
         # children to easily drop refs to sibling sockets in post_fork_hook().
         self.tracking_data = {}
 
+        # When doing a seamless reload, we inherit a bunch of child processes
+        # that should all clean themselves up fairly quickly; track them here
+        self.reload_pids = dict()
+        # If they don't cleanup quickly, we'll start killing them after this
+        self.stale_worker_timeout = utils.non_negative_float(
+            conf.get('stale_worker_timeout', 86400))
+
     def post_fork_hook(self):
         """
         Called in each forked-off child process, prior to starting the actual
@@ -460,6 +493,8 @@ class StrategyBase(object):
             capture_stdio(self.logger)
         drop_privileges(self.conf.get('user', 'swift'))
         del self.tracking_data  # children don't need to track siblings
+        # only MAINPID should be sending systemd notifications
+        os.environ.pop('NOTIFY_SOCKET', None)
 
     def shutdown_sockets(self):
         """
@@ -476,13 +511,10 @@ class StrategyBase(object):
         """
 
         for sock in self.iter_sockets():
-            if six.PY2:
-                fcntl.fcntl(sock.fileno(), fcntl.F_SETFD, fcntl.FD_CLOEXEC)
-            else:
-                # Python 3.4 and later default to sockets having close-on-exec
-                # set (what PEP 0446 calls "non-inheritable").  This new method
-                # on socket objects is provided to toggle it.
-                sock.set_inheritable(False)
+            # Python 3.4 and later default to sockets having close-on-exec
+            # set (what PEP 0446 calls "non-inheritable").  This new method
+            # on socket objects is provided to toggle it.
+            sock.set_inheritable(False)
 
     def signal_ready(self):
         """
@@ -500,9 +532,21 @@ class StrategyBase(object):
         # connections. This is used for seamless reloading using SIGUSR1.
         reexec_signal_fd = os.getenv(NOTIFY_FD_ENV_KEY)
         if reexec_signal_fd:
+            if ',' in reexec_signal_fd:
+                reexec_signal_fd, worker_state_fd = reexec_signal_fd.split(',')
             reexec_signal_fd = int(reexec_signal_fd)
             os.write(reexec_signal_fd, str(os.getpid()).encode('utf8'))
             os.close(reexec_signal_fd)
+        worker_state_fd = os.getenv(CHILD_STATE_FD_ENV_KEY)
+        try:
+            self.read_state_from_old_manager(worker_state_fd)
+        except Exception as e:
+            # This was all opportunistic anyway; old swift wouldn't even
+            # *try* to send us any state -- we don't want *new* code to
+            # fail just because *old* code didn't live up to its promise
+            self.logger.warning(
+                'Failed to read state from the old manager: %r', e,
+                exc_info=True)
 
         # Finally, signal systemd (if appropriate) that process started
         # properly.
@@ -510,6 +554,110 @@ class StrategyBase(object):
 
         self.signaled_ready = True
 
+    def read_state_from_old_manager(self, worker_state_fd):
+        """
+        Read worker state from the old manager's socket-closer.
+
+        The socket-closing process is the last thing to still have the worker
+        PIDs in its head, so it sends us a JSON dict (prefixed by its length)
+        of the form::
+
+           {
+             "old_pids": {
+               "<old worker>": "<first reload time>",
+               ...
+             }
+           }
+
+        More data may be added in the future.
+
+        :param worker_state_fd: The file descriptor that should have the
+                                old worker state. Should be passed to us
+                                via the ``__SWIFT_SERVER_CHILD_STATE_FD``
+                                environment variable.
+        """
+        if not worker_state_fd:
+            return
+        worker_state_fd = int(worker_state_fd)
+        try:
+            # The temporary manager may have up and died while trying to send
+            # state; hopefully its logs will have more about what went wrong
+            # -- let's just log at warning here
+            data_len = os.read(worker_state_fd, 4)
+            if len(data_len) != 4:
+                self.logger.warning(
+                    'Invalid worker state received; expected 4 bytes '
+                    'followed by a payload but only received %d bytes',
+                    len(data_len))
+                return
+
+            data_len = struct.unpack('!I', data_len)[0]
+            data = b''
+            while len(data) < data_len:
+                chunk = os.read(worker_state_fd, data_len - len(data))
+                if not chunk:
+                    break
+                data += chunk
+            if len(data) != data_len:
+                self.logger.warning(
+                    'Incomplete worker state received; expected %d '
+                    'bytes but only received %d', data_len, len(data))
+                return
+
+            # OK, the temporary manager was able to tell us how much it wanted
+            # to send and send it; from here on, error seems appropriate.
+            try:
+                old_state = json.loads(data)
+            except ValueError:
+                self.logger.error(
+                    'Invalid worker state received; '
+                    'invalid JSON: %r', data)
+                return
+
+            try:
+                old_pids = {
+                    int(pid): float(reloaded)
+                    for pid, reloaded in old_state["old_pids"].items()}
+            except (KeyError, TypeError) as err:
+                self.logger.error(
+                    'Invalid worker state received; '
+                    'error reading old pids: %s', err)
+            self.logger.debug('Received old worker pids: %s', old_pids)
+            self.reload_pids.update(old_pids)
+
+            def smother(old_pids=old_pids, timeout=self.stale_worker_timeout):
+                own_pid = os.getpid()
+                kill_times = sorted(((reloaded + timeout, pid)
+                                     for pid, reloaded in old_pids.items()),
+                                    reverse=True)
+                while kill_times:
+                    kill_time, pid = kill_times.pop()
+                    now = time.time()
+                    if kill_time > now:
+                        sleep(kill_time - now)
+                    try:
+                        ppid = utils.get_ppid(pid)
+                    except OSError as e:
+                        if e.errno != errno.ESRCH:
+                            self.logger.error("Could not determine parent "
+                                              "for stale pid %d: %s", pid, e)
+                        continue
+                    if ppid == own_pid:
+                        self.logger.notice("Killing long-running stale worker "
+                                           "%d after %ds", pid, int(timeout))
+                        try:
+                            os.kill(pid, signal.SIGKILL)
+                        except OSError as e:
+                            if e.errno != errno.ESRCH:
+                                self.logger.error(
+                                    "Could not kill stale pid %d: %s", pid, e)
+                    # else, pid got re-used?
+
+            eventlet.spawn_n(smother)
+
+        finally:
+            os.close(worker_state_fd)
+
 
 class WorkersStrategy(StrategyBase):
     """
@@ -599,11 +747,17 @@ class WorkersStrategy(StrategyBase):
         :param int pid: The PID of the worker that exited.
         """
 
+        if self.reload_pids.pop(pid, None):
+            self.logger.notice('Removing stale child %d from parent %d',
+                               pid, os.getpid())
+            return
+
         sock = self.tracking_data.pop(pid, None)
         if sock is None:
-            self.logger.info('Ignoring wait() result from unknown PID %s', pid)
+            self.logger.warning('Ignoring wait() result from unknown PID %d',
+                                pid)
         else:
-            self.logger.error('Removing dead child %s from parent %s',
+            self.logger.error('Removing dead child %d from parent %d',
                               pid, os.getpid())
             greenio.shutdown_safe(sock)
             sock.close()
@@ -616,6 +770,9 @@ class WorkersStrategy(StrategyBase):
         for sock in self.tracking_data.values():
             yield sock
 
+    def get_worker_pids(self):
+        return list(self.tracking_data.keys())
+
 
 class ServersPerPortStrategy(StrategyBase):
     """
@@ -744,7 +901,7 @@ class ServersPerPortStrategy(StrategyBase):
         Called when a new worker is started.
 
         :param socket sock: The listen socket for the worker just started.
-        :param server_idx: The socket's server_idx as yielded by
+        :param tuple data: The socket's (port, server_idx) as yielded by
                            :py:meth:`new_worker_socks`.
         :param int pid: The new worker process' PID
         """
@@ -763,14 +920,23 @@ class ServersPerPortStrategy(StrategyBase):
         :param int pid: The PID of the worker that exited.
         """
 
+        if self.reload_pids.pop(pid, None):
+            self.logger.notice('Removing stale child %d from parent %d',
+                               pid, os.getpid())
+            return
+
         for port_data in self.tracking_data.values():
             for idx, (child_pid, sock) in enumerate(port_data):
                 if child_pid == pid:
+                    self.logger.error('Removing dead child %d from parent %d',
+                                      pid, os.getpid())
                     port_data[idx] = (None, None)
                     greenio.shutdown_safe(sock)
                     sock.close()
                     return
 
+        self.logger.warning('Ignoring wait() result from unknown PID %d', pid)
+
     def iter_sockets(self):
         """
         Yields all known listen sockets.
@@ -780,28 +946,17 @@ class ServersPerPortStrategy(StrategyBase):
             for _pid, sock in port_data:
                 yield sock
 
+    def get_worker_pids(self):
+        return [
+            pid
+            for port_data in self.tracking_data.values()
+            for pid, _sock in port_data]
 
-def run_wsgi(conf_path, app_section, *args, **kwargs):
-    """
-    Runs the server according to some strategy.  The default strategy runs a
-    specified number of workers in pre-fork model.  The object-server (only)
-    may use a servers-per-port strategy if its config has a servers_per_port
-    setting with a value greater than zero.
 
-    :param conf_path: Path to paste.deploy style configuration file/directory
-    :param app_section: App name from conf file to load config from
-    :param allow_modify_pipeline: Boolean for whether the server should have
-                                  an opportunity to change its own pipeline.
-                                  Defaults to True
-    :returns: 0 if successful, nonzero otherwise
-    """
+def check_config(conf_path, app_section, *args, **kwargs):
     # Load configuration, Set logger and Load request processor
-    try:
-        (conf, logger, log_name) = \
-            _initrp(conf_path, app_section, *args, **kwargs)
-    except ConfigFileError as e:
-        print(e)
-        return 1
+    (conf, logger, log_name) = \
+        _initrp(conf_path, app_section, *args, **kwargs)
 
     # optional nice/ionice priority scheduling
     utils.modify_priority(conf, logger)
@@ -818,16 +973,16 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
         strategy = WorkersStrategy(conf, logger)
         try:
             # Quick sanity check
-            int(conf['bind_port'])
+            if not (1 <= int(conf['bind_port']) <= 2 ** 16 - 1):
+                raise ValueError
         except (ValueError, KeyError, TypeError):
             error_msg = 'bind_port wasn\'t properly set in the config file. ' \
-                'It must be explicitly set to a valid port number.'
+                        'It must be explicitly set to a valid port number.'
             logger.error(error_msg)
-            print(error_msg)
-            return 1
+            raise ConfigFileError(error_msg)
 
     # patch event before loadapp
-    utils.eventlet_monkey_patch()
+    utils.monkey_patch()
 
     # Ensure the configuration and application can be loaded before proceeding.
     global_conf = {'log_name': log_name}
@@ -835,21 +990,50 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
     if 'global_conf_callback' in kwargs:
         kwargs['global_conf_callback'](conf, global_conf)
 
-    allow_modify_pipeline = kwargs.get('allow_modify_pipeline', True)
-
     # set utils.FALLOCATE_RESERVE if desired
     utils.FALLOCATE_RESERVE, utils.FALLOCATE_IS_PERCENT = \
         utils.config_fallocate_value(conf.get('fallocate_reserve', '1%'))
+    return conf, logger, global_conf, strategy
+
+
+def run_wsgi(conf_path, app_section, *args, **kwargs):
+    """
+    Runs the server according to some strategy.  The default strategy runs a
+    specified number of workers in pre-fork model.  The object-server (only)
+    may use a servers-per-port strategy if its config has a servers_per_port
+    setting with a value greater than zero.
+
+    :param conf_path: Path to paste.deploy style configuration file/directory
+    :param app_section: App name from conf file to load config from
+    :param allow_modify_pipeline: Boolean for whether the server should have
+                                  an opportunity to change its own pipeline.
+                                  Defaults to True
+    :param test_config: if False (the default) then load and validate the
+        config and if successful then continue to run the server; if True then
+        load and validate the config but do not run the server.
+    :returns: 0 if successful, nonzero otherwise
+    """
+    try:
+        conf, logger, global_conf, strategy = check_config(
+            conf_path, app_section, *args, **kwargs)
+    except ConfigFileError as err:
+        print(err)
+        return 1
+
+    if kwargs.get('test_config'):
+        return 0
 
     # Do some daemonization process hygene before we fork any children or run a
     # server without forking.
     clean_up_daemon_hygiene()
 
+    allow_modify_pipeline = kwargs.get('allow_modify_pipeline', True)
     no_fork_sock = strategy.no_fork_sock()
     if no_fork_sock:
         run_server(conf, logger, no_fork_sock, global_conf=global_conf,
                    ready_callback=strategy.signal_ready,
                    allow_modify_pipeline=allow_modify_pipeline)
+        systemd_notify(logger, "STOPPING=1")
         return 0
 
     def stop_with_signal(signum, *args):
@@ -864,6 +1048,7 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
     signal.signal(signal.SIGUSR1, stop_with_signal)
 
     while running_context[0]:
+        new_workers = {}  # pid -> status pipe
         for sock, sock_info in strategy.new_worker_socks():
             read_fd, write_fd = os.pipe()
             pid = os.fork()
@@ -888,16 +1073,16 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
                 return 0
             else:
                 os.close(write_fd)
-                worker_status = os.read(read_fd, 30)
-                os.close(read_fd)
-                # TODO: delay this status checking until after we've tried
-                # to start all workers. But, we currently use the register
-                # event to know when we've got enough workers :-/
-                if worker_status == b'ready':
-                    strategy.register_worker_start(sock, sock_info, pid)
-                else:
-                    raise Exception(
-                        'worker did not start normally: %r' % worker_status)
+                new_workers[pid] = read_fd
+                strategy.register_worker_start(sock, sock_info, pid)
+
+        for pid, read_fd in new_workers.items():
+            worker_status = os.read(read_fd, 30)
+            os.close(read_fd)
+            if worker_status != b'ready':
+                raise Exception(
+                    'worker %d did not start normally: %r' %
+                    (pid, worker_status))
 
         # TODO: signal_ready() as soon as we have at least one new worker for
         # each port, instead of waiting for all of them
@@ -942,8 +1127,10 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
         else:
             logger.notice('%s received (%s)', signame, os.getpid())
     if running_context[1] == signal.SIGTERM:
+        systemd_notify(logger, "STOPPING=1")
         os.killpg(0, signal.SIGTERM)
     elif running_context[1] == signal.SIGUSR1:
+        systemd_notify(logger, "RELOADING=1")
         # set up a pipe, fork off a child to handle cleanup later,
         # and rexec ourselves with an environment variable set which will
         # indicate which fd (one of the pipe ends) to write a byte to
@@ -962,24 +1149,29 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
         # then the old server can't actually ever exit.
         strategy.set_close_on_exec_on_listen_sockets()
         read_fd, write_fd = os.pipe()
+        state_rfd, state_wfd = os.pipe()
         orig_server_pid = os.getpid()
         child_pid = os.fork()
         if child_pid:
             # parent; set env var for fds and reexec ourselves
             os.close(read_fd)
+            os.close(state_wfd)
             os.putenv(NOTIFY_FD_ENV_KEY, str(write_fd))
+            os.putenv(CHILD_STATE_FD_ENV_KEY, str(state_rfd))
             myself = os.path.realpath(sys.argv[0])
             logger.info("Old server PID=%d re'execing as: %r",
                         orig_server_pid, [myself] + list(sys.argv))
             if hasattr(os, 'set_inheritable'):
                 # See https://www.python.org/dev/peps/pep-0446/
                 os.set_inheritable(write_fd, True)
-            os.execv(myself, sys.argv)
+                os.set_inheritable(state_rfd, True)
+            os.execv(myself, sys.argv)  # nosec B606
             logger.error('Somehow lived past os.execv()?!')
             exit('Somehow lived past os.execv()?!')
         elif child_pid == 0:
             # child
             os.close(write_fd)
+            os.close(state_rfd)
             logger.info('Old server temporary child PID=%d waiting for '
                         "re-exec'ed PID=%d to signal readiness...",
                         os.getpid(), orig_server_pid)
@@ -994,6 +1186,16 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
                     logger.info('Old server temporary child PID=%d notified '
                                 'to shutdown old listen sockets by PID=%s',
                                 os.getpid(), got_pid)
+                    # Ensure new process knows about old children
+                    stale_pids = dict(strategy.reload_pids)
+                    stale_pids[os.getpid()] = now = time.time()
+                    stale_pids.update({
+                        pid: now for pid in strategy.get_worker_pids()})
+                    data = json.dumps({
+                        "old_pids": stale_pids,
+                    }).encode('ascii')
+                    os.write(state_wfd, struct.pack('!I', len(data)) + data)
+                    os.close(state_wfd)
                 else:
                     logger.warning('Old server temporary child PID=%d *NOT* '
                                    'notified to shutdown old listen sockets; '
@@ -1002,6 +1204,9 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
                 os.close(read_fd)
             except Exception:
                 pass
+    else:
+        # SIGHUP or, less likely, run in "once" mode
+        systemd_notify(logger, "STOPPING=1")
 
     strategy.shutdown_sockets()
     signal.signal(signal.SIGTERM, signal.SIG_IGN)
