@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
 import logging
 import os
 import signal
@@ -32,8 +31,8 @@ from swift.common.exceptions import ConnectionTimeout, LockTimeout
 from swift.common.ring import Ring
 from swift.common.utils import get_logger, config_true_value, \
     dump_recon_cache, majority_size, Timestamp, EventletRateLimiter, \
-    eventlet_monkey_patch
-from swift.common.daemon import Daemon
+    eventlet_monkey_patch, node_to_string, parse_options
+from swift.common.daemon import Daemon, run_daemon
 from swift.common.http import is_success, HTTP_INTERNAL_SERVER_ERROR
 from swift.common.recon import RECON_CONTAINER_FILE, DEFAULT_RECON_CACHE_PATH
 
@@ -158,6 +157,7 @@ class ContainerUpdater(Daemon):
                     pid2filename[pid] = tmpfilename
                 else:
                     signal.signal(signal.SIGTERM, signal.SIG_DFL)
+                    os.environ.pop('NOTIFY_SOCKET', None)
                     eventlet_monkey_patch()
                     self.no_changes = 0
                     self.successes = 0
@@ -340,9 +340,8 @@ class ContainerUpdater(Daemon):
                     node['device'], part, 'PUT', container, headers=headers)
             except (Exception, Timeout):
                 self.logger.exception(
-                    'ERROR account update failed with '
-                    '%(replication_ip)s:%(replication_port)s/%(device)s '
-                    '(will retry later): ', node)
+                    'ERROR account update failed with %s (will retry later):',
+                    node_to_string(node, replication=True))
                 return HTTP_INTERNAL_SERVER_ERROR
         with Timeout(self.node_timeout):
             try:
@@ -352,9 +351,17 @@ class ContainerUpdater(Daemon):
             except (Exception, Timeout):
                 if self.logger.getEffectiveLevel() <= logging.DEBUG:
                     self.logger.exception(
-                        'Exception with '
-                        '%(replication_ip)s:%(replication_port)s/%(device)s',
-                        node)
+                        'Exception with %s',
+                        node_to_string(node, replication=True))
                 return HTTP_INTERNAL_SERVER_ERROR
             finally:
                 conn.close()
+
+
+def main():
+    conf_file, options = parse_options(once=True)
+    run_daemon(ContainerUpdater, conf_file, **options)
+
+
+if __name__ == '__main__':
+    main()
