@@ -14,13 +14,13 @@
 # limitations under the License.
 
 from eventlet import sleep
-import six
-from six.moves import urllib
+import urllib
 
 from swift.common import bufferedhttp
 from swift.common import exceptions
 from swift.common import http
-from swift.common.utils import config_true_value
+from swift.common import utils
+from swift.common.swob import wsgi_to_bytes
 
 
 def encode_missing(object_hash, ts_data, ts_meta=None, ts_ctype=None,
@@ -42,9 +42,13 @@ def encode_missing(object_hash, ts_data, ts_meta=None, ts_ctype=None,
     if ts_meta and ts_meta != ts_data:
         delta = ts_meta.raw - ts_data.raw
         extra_parts.append('m:%x' % delta)
+        if ts_meta.offset:
+            extra_parts[-1] += '__%x' % ts_meta.offset
         if ts_ctype and ts_ctype != ts_data:
             delta = ts_ctype.raw - ts_data.raw
             extra_parts.append('t:%x' % delta)
+            if ts_ctype.offset:
+                extra_parts[-1] += '__%x' % ts_ctype.offset
     if 'durable' in kwargs and kwargs['durable'] is False:
         # only send durable in the less common case that it is False
         extra_parts.append('durable:%s' % kwargs['durable'])
@@ -208,20 +212,18 @@ class Sender(object):
                 return True, can_delete_obj
             except (exceptions.MessageTimeout,
                     exceptions.ReplicationException) as err:
-                self.daemon.logger.error(
-                    '%s:%s/%s/%s %s', self.node.get('replication_ip'),
-                    self.node.get('replication_port'), self.node.get('device'),
-                    self.job.get('partition'), err)
+                node_str = utils.node_to_string(self.node, replication=True)
+                self.daemon.logger.error('%s/%s %s', node_str,
+                                         self.job['partition'], err)
             except Exception:
                 # We don't want any exceptions to escape our code and possibly
                 # mess up the original replicator code that called us since it
                 # was originally written to shell out to rsync which would do
                 # no such thing.
+                node_str = utils.node_to_string(self.node, replication=True)
                 self.daemon.logger.exception(
-                    '%s:%s/%s/%s EXCEPTION in ssync.Sender',
-                    self.node.get('replication_ip'),
-                    self.node.get('replication_port'),
-                    self.node.get('device'), self.job.get('partition'))
+                    '%s/%s EXCEPTION in ssync.Sender',
+                    node_str, self.job['partition'])
             finally:
                 self.disconnect(connection)
         except Exception:
@@ -264,11 +266,11 @@ class Sender(object):
                 self.daemon.node_timeout, 'connect receive'):
             response = connection.getresponse()
             if response.status != http.HTTP_OK:
-                err_msg = response.read()[:1024]
+                err_msg = utils.cap_length(response.read(), 1024)
                 raise exceptions.ReplicationException(
                     'Expected status %s; got %s (%s)' %
                     (http.HTTP_OK, response.status, err_msg))
-            if self.include_non_durable and not config_true_value(
+            if self.include_non_durable and not utils.config_true_value(
                     response.getheader('x-backend-accept-no-commit', False)):
                 # fall back to legacy behaviour if receiver does not understand
                 # X-Backend-Commit
@@ -303,7 +305,7 @@ class Sender(object):
             frag_index=self.job.get('frag_index'),
             frag_prefs=frag_prefs)
         if self.remote_check_objs is not None:
-            hash_gen = six.moves.filter(
+            hash_gen = filter(
                 lambda objhash_timestamps:
                 objhash_timestamps[0] in
                 self.remote_check_objs, hash_gen)
@@ -350,13 +352,12 @@ class Sender(object):
             if line == b':MISSING_CHECK: START':
                 break
             elif line:
-                if not six.PY2:
-                    try:
-                        line = line.decode('ascii')
-                    except UnicodeDecodeError:
-                        pass
+                try:
+                    line = line.decode('ascii')
+                except UnicodeDecodeError:
+                    pass
                 raise exceptions.ReplicationException(
-                    'Unexpected response: %r' % line[:1024])
+                    'Unexpected response: %r' % utils.cap_length(line, 1024))
         while True:
             with exceptions.MessageTimeout(
                     self.daemon.http_timeout, 'missing_check line wait'):
@@ -439,13 +440,12 @@ class Sender(object):
             if line == b':UPDATES: START':
                 break
             elif line:
-                if not six.PY2:
-                    try:
-                        line = line.decode('ascii')
-                    except UnicodeDecodeError:
-                        pass
+                try:
+                    line = line.decode('ascii')
+                except UnicodeDecodeError:
+                    pass
                 raise exceptions.ReplicationException(
-                    'Unexpected response: %r' % line[:1024])
+                    'Unexpected response: %r' % utils.cap_length(line, 1024))
         while True:
             with exceptions.MessageTimeout(
                     self.daemon.http_timeout, 'updates line wait'):
@@ -456,23 +456,17 @@ class Sender(object):
             if line == b':UPDATES: END':
                 break
             elif line:
-                if not six.PY2:
-                    try:
-                        line = line.decode('ascii')
-                    except UnicodeDecodeError:
-                        pass
+                try:
+                    line = line.decode('ascii')
+                except UnicodeDecodeError:
+                    pass
                 raise exceptions.ReplicationException(
-                    'Unexpected response: %r' % line[:1024])
+                    'Unexpected response: %r' % utils.cap_length(line, 1024))
 
     def send_subrequest(self, connection, method, url_path, headers, df):
         msg = [b'%s %s' % (method.encode('ascii'), url_path.encode('utf8'))]
         for key, value in sorted(headers.items()):
-            if six.PY2:
-                msg.append(b'%s: %s' % (key, value))
-            else:
-                msg.append(b'%s: %s' % (
-                    key.encode('utf8', 'surrogateescape'),
-                    str(value).encode('utf8', 'surrogateescape')))
+            msg.append(wsgi_to_bytes('%s: %s' % (key, value)))
         msg = b'\r\n'.join(msg) + b'\r\n\r\n'
         with exceptions.MessageTimeout(self.daemon.node_timeout,
                                        'send_%s' % method.lower()):
