@@ -12,14 +12,15 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import collections
 import contextlib
 import logging
-import mock
+from unittest import mock
 import sys
 
 from collections import defaultdict
 
-from swift.common import utils
+from swift.common import utils, statsd_client
 from swift.common.utils import NOTICE
 
 
@@ -29,12 +30,23 @@ class WARN_DEPRECATED(Exception):
         print(self.msg)
 
 
-class FakeStatsdClient(utils.StatsdClient):
-    def __init__(self, host, port, base_prefix='', tail_prefix='',
-                 default_sample_rate=1, sample_rate_factor=1, logger=None):
-        super(FakeStatsdClient, self).__init__(
-            host, port, base_prefix, tail_prefix, default_sample_rate,
-            sample_rate_factor, logger)
+SendtoCall = collections.namedtuple('SendToCall', ['payload', 'address'])
+
+
+class RecordingSocket(object):
+    def __init__(self):
+        self.sendto_calls = []
+
+    def sendto(self, payload, address):
+        self.sendto_calls.append(SendtoCall(payload, address))
+
+    def close(self):
+        pass
+
+
+class FakeStatsdClient(statsd_client.StatsdClient):
+    def __init__(self, *args, **kwargs):
+        super(FakeStatsdClient, self).__init__(*args, **kwargs)
         self.clear()
 
         # Capture then call parent pubic stat functions
@@ -53,18 +65,11 @@ class FakeStatsdClient(utils.StatsdClient):
             return func(*args, **kwargs)
         return wrapper
 
-    def _determine_sock_family(self, host, port):
-        return None, None
+    def _set_sock_family_and_target(self, host, port):
+        self._target = (host, port)
 
     def _open_socket(self):
-        return self
-
-    # sendto and close are mimicing the socket calls.
-    def sendto(self, msg, target):
-        self.sendto_calls.append((msg, target))
-
-    def close(self):
-        pass
+        return self.recording_socket
 
     def _send(self, *args, **kwargs):
         self.send_calls.append((args, kwargs))
@@ -73,7 +78,11 @@ class FakeStatsdClient(utils.StatsdClient):
     def clear(self):
         self.send_calls = []
         self.calls = defaultdict(list)
-        self.sendto_calls = []
+        self.recording_socket = RecordingSocket()
+
+    @property
+    def sendto_calls(self):
+        return self.recording_socket.sendto_calls
 
     def get_increments(self):
         return [call[0][0] for call in self.calls['increment']]
@@ -85,7 +94,8 @@ class FakeStatsdClient(utils.StatsdClient):
         counts = defaultdict(int)
         for metric in self.get_increments():
             counts[metric] += 1
-        return counts
+        # convert to normal dict for better failure messages
+        return dict(counts)
 
     def get_update_stats(self):
         return [call[0][:2] for call in self.calls['update_stats']]
@@ -94,7 +104,8 @@ class FakeStatsdClient(utils.StatsdClient):
         counts = defaultdict(int)
         for metric, step in self.get_update_stats():
             counts[metric] += step
-        return counts
+        # convert to normal dict for better failure messages
+        return dict(counts)
 
 
 class CaptureLog(object):
@@ -102,6 +113,7 @@ class CaptureLog(object):
     Captures log records passed to the ``handle`` method and provides accessor
     functions to the captured logs.
     """
+
     def __init__(self):
         self.clear()
 
@@ -147,9 +159,12 @@ class FakeLogger(logging.Logger, CaptureLog):
         self.level = logging.NOTSET
         if 'facility' in kwargs:
             self.facility = kwargs['facility']
-        self.statsd_client = FakeStatsdClient("host", 8125)
+        self.statsd_client = FakeStatsdClient('host', 8125)
         self.thread_locals = None
         self.parent = None
+        # ensure the NOTICE level has been named, in case it has not already
+        # been set
+        logging.addLevelName(NOTICE, 'NOTICE')
 
     store_in = {
         logging.ERROR: 'error',
@@ -243,7 +258,10 @@ class DebugLogger(FakeLogger):
         self.records[record.levelname].append(formatted)
 
 
-class DebugLogAdapter(utils.LogAdapter):
+class DebugLogAdapter(utils.logs.SwiftLogAdapter):
+    def __init__(self, *args, **kwargs):
+        super(DebugLogAdapter, self).__init__(*args, **kwargs)
+        self.txn_id = None
 
     def __getattribute__(self, name):
         try:
@@ -254,7 +272,9 @@ class DebugLogAdapter(utils.LogAdapter):
 
 def debug_logger(name='test'):
     """get a named adapted debug logger"""
-    return DebugLogAdapter(DebugLogger(), name)
+    adapted_logger = DebugLogAdapter(DebugLogger(), name)
+    utils._patch_statsd_methods(adapted_logger, adapted_logger.logger)
+    return adapted_logger
 
 
 class ForwardingLogHandler(logging.NullHandler):
@@ -263,6 +283,7 @@ class ForwardingLogHandler(logging.NullHandler):
     to a given handler function. This can be useful to forward records to a
     handler without the handler itself needing to subclass LogHandler.
     """
+
     def __init__(self, handler_fn):
         super(ForwardingLogHandler, self).__init__()
         self.handler_fn = handler_fn
@@ -271,11 +292,12 @@ class ForwardingLogHandler(logging.NullHandler):
         return self.handler_fn(record)
 
 
-class CaptureLogAdapter(utils.LogAdapter, CaptureLog):
+class CaptureLogAdapter(utils.logs.SwiftLogAdapter, CaptureLog):
     """
     A LogAdapter that is capable of capturing logs for inspection via accessor
     methods.
     """
+
     def __init__(self, logger, name):
         super(CaptureLogAdapter, self).__init__(logger, name)
         self.clear()
@@ -305,7 +327,8 @@ def capture_logger(conf, *args, **kwargs):
     accessor methods (e.g. get_lines_for_level) directly on the logger
     instance.
     """
-    with mock.patch('swift.common.utils.LogAdapter', CaptureLogAdapter):
+    with mock.patch('swift.common.utils.logs.SwiftLogAdapter',
+                    CaptureLogAdapter):
         log_adapter = utils.get_logger(conf, *args, **kwargs)
     log_adapter.start_capture()
     try:
