@@ -13,11 +13,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
-import mock
+
+import configparser
+import contextlib
+
+from unittest import mock
 import os
-import six
-from six.moves.urllib.parse import urlparse, urlsplit, urlunsplit
+from urllib.parse import urlparse, urlsplit, urlunsplit
 import sys
 import pickle
 import socket
@@ -33,10 +35,11 @@ from contextlib import closing
 from gzip import GzipFile
 from shutil import rmtree
 from tempfile import mkdtemp
+from unittest import SkipTest
 
-from six.moves.configparser import ConfigParser, NoSectionError
-from six.moves import http_client
-from six.moves.http_client import HTTPException
+from configparser import ConfigParser
+import http.client
+from http.client import HTTPException
 
 from swift.common.middleware.memcache import MemcacheMiddleware
 from swift.common.storage_policy import parse_storage_policies, PolicyError
@@ -47,10 +50,8 @@ from test import get_config, listen_zero
 from test.debug_logger import debug_logger
 from test.unit import FakeMemcache
 # importing skip_if_no_xattrs so that functional tests can grab it from the
-# test.functional namespace. Importing SkipTest so this works under both
-# nose and testr test runners.
+# test.functional namespace.
 from test.unit import skip_if_no_xattrs as real_skip_if_no_xattrs
-from test.unit import SkipTest
 
 from swift.common import constraints, utils, ring, storage_policy
 from swift.common.ring import Ring
@@ -62,7 +63,7 @@ from swift.container import server as container_server
 from swift.obj import server as object_server, mem_server as mem_object_server
 import swift.proxy.controllers.obj
 
-http_client._MAXHEADERS = constraints.MAX_HEADER_COUNT
+http.client._MAXHEADERS = constraints.MAX_HEADER_COUNT
 DEBUG = True
 
 # In order to get the proper blocking behavior of sockets without using
@@ -81,19 +82,6 @@ from test.functional.swift_test_client import Account, Connection, Container, \
 
 from swiftclient import get_auth, http_connection
 
-has_insecure = False
-try:
-    from swiftclient import __version__ as client_version
-    # Prevent a ValueError in StrictVersion with '2.0.3.68.ga99c2ff'
-    client_version = '.'.join(client_version.split('.')[:3])
-except ImportError:
-    # Pre-PBR we had version, not __version__. Anyhow...
-    client_version = '1.2'
-from distutils.version import StrictVersion
-if StrictVersion(client_version) >= StrictVersion('2.0'):
-    has_insecure = True
-
-
 config = {}
 web_front_end = None
 normalized_urls = None
@@ -145,24 +133,60 @@ def _debug(msg):
         _info('DEBUG: ' + msg)
 
 
+@contextlib.contextmanager
+def _modify_conf_file(src_conf_file, dest_conf_file):
+    conf = ConfigParser()
+    conf.read(src_conf_file)
+    try:
+        yield conf
+    except configparser.Error as err:
+        msg = 'Error modifying conf file %s: %s' % (src_conf_file, err)
+        raise InProcessException(msg)
+    else:
+        with open(dest_conf_file, 'w') as fp:
+            conf.write(fp)
+
+
+def _cleanup_pipeline(pipeline_str):
+    # return single-whitespace delimited and padded string
+    # e.g. "x   y  " -> " x y "
+    return ' %s ' % ' '.join([x for x in pipeline_str.split(' ') if x])
+
+
+def _modify_pipeline_substring(conf, old_substr, new_substr):
+    section = 'pipeline:main'
+    old_pipeline = _cleanup_pipeline(conf.get(section, 'pipeline'))
+    old_substr = _cleanup_pipeline(old_substr)
+    new_substr = _cleanup_pipeline(new_substr)
+    if old_substr not in old_pipeline:
+        raise InProcessException(
+            'Failed to replace pipeline substring: old="%s", new="%s", '
+            'pipeline="%s"' % (old_substr, new_substr, old_pipeline))
+
+    if new_substr in old_pipeline:
+        _info('WARNING: "%s" is already in pipeline, not replacing'
+              % new_substr)
+        new_pipeline = old_pipeline
+    else:
+        new_pipeline = old_pipeline.replace(old_substr, new_substr)
+        _debug('Replaced pipeline substring "%s" with "%s"'
+               % (old_substr, new_substr))
+    # remove padding
+    new_pipeline = new_pipeline.strip(' ')
+    conf.set(section, 'pipeline', new_pipeline)
+    _debug('Proxy pipeline is now "%s"' % new_pipeline)
+
+
 def _in_process_setup_swift_conf(swift_conf_src, testdir):
     # override swift.conf contents for in-process functional test runs
-    conf = ConfigParser()
-    conf.read(swift_conf_src)
-    try:
+    test_conf_file = os.path.join(testdir, 'swift.conf')
+    with _modify_conf_file(swift_conf_src, test_conf_file) as conf:
         section = 'swift-hash'
         conf.set(section, 'swift_hash_path_suffix', 'inprocfunctests')
         conf.set(section, 'swift_hash_path_prefix', 'inprocfunctests')
         section = 'swift-constraints'
         max_file_size = (8 * 1024 * 1024) + 2  # 8 MB + 2
         conf.set(section, 'max_file_size', str(max_file_size))
-    except NoSectionError:
-        msg = 'Conf file %s is missing section %s' % (swift_conf_src, section)
-        raise InProcessException(msg)
-
-    test_conf_file = os.path.join(testdir, 'swift.conf')
-    with open(test_conf_file, 'w') as fp:
-        conf.write(fp)
 
     return test_conf_file
 
@@ -209,39 +233,36 @@ def _in_process_setup_ring(swift_conf, conf_src_dir, testdir):
       in process testing, and renaming it to suit policy-0
     Otherwise, create a default ring file.
     """
-    conf = ConfigParser()
-    conf.read(swift_conf)
-    sp_prefix = 'storage-policy:'
+    with _modify_conf_file(swift_conf, swift_conf) as conf:
+        sp_prefix = 'storage-policy:'
 
-    try:
-        # policy index 0 will be created if no policy exists in conf
-        policies = parse_storage_policies(conf)
-    except PolicyError as e:
-        raise InProcessException(e)
+        try:
+            # policy index 0 will be created if no policy exists in conf
+            policies = parse_storage_policies(conf)
+        except PolicyError as e:
+            raise InProcessException(e)
 
-    # clear all policies from test swift.conf before adding test policy back
-    for policy in policies:
-        conf.remove_section(sp_prefix + str(policy.idx))
+        # clear all policies from test swift.conf before adding test policy
+        # back
+        for policy in policies:
+            conf.remove_section(sp_prefix + str(policy.idx))
 
-    if policy_specified:
-        policy_to_test = policies.get_by_name(policy_specified)
-        if policy_to_test is None:
-            raise InProcessException('Failed to find policy name "%s"'
-                                     % policy_specified)
-        _info('Using specified policy %s' % policy_to_test.name)
-    else:
-        policy_to_test = policies.default
-        _info('Defaulting to policy %s' % policy_to_test.name)
+        if policy_specified:
+            policy_to_test = policies.get_by_name(policy_specified)
+            if policy_to_test is None:
+                raise InProcessException('Failed to find policy name "%s"'
+                                         % policy_specified)
+            _info('Using specified policy %s' % policy_to_test.name)
+        else:
+            policy_to_test = policies.default
+            _info('Defaulting to policy %s' % policy_to_test.name)
 
-    # make policy_to_test be policy index 0 and default for the test config
-    sp_zero_section = sp_prefix + '0'
-    conf.add_section(sp_zero_section)
-    for (k, v) in policy_to_test.get_info(config=True).items():
-        conf.set(sp_zero_section, k, str(v))
-    conf.set(sp_zero_section, 'default', 'True')
-
-    with open(swift_conf, 'w') as fp:
-        conf.write(fp)
+        # make policy_to_test be policy index 0 and default for the test config
+        sp_zero_section = sp_prefix + '0'
+        conf.add_section(sp_zero_section)
+        for (k, v) in policy_to_test.get_info(config=True).items():
+            conf.set(sp_zero_section, k, str(v))
+        conf.set(sp_zero_section, 'default', 'True')
 
     # look for a source ring file
     ring_file_src = ring_file_test = 'object.ring.gz'
@@ -298,6 +319,18 @@ def _in_process_setup_ring(swift_conf, conf_src_dir, testdir):
     return obj_sockets
 
 
+def _load_etag_quoter(proxy_conf_file, swift_conf_file, **kwargs):
+    _debug('Ensuring etag-quoter is in proxy pipeline')
+    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
+    with _modify_conf_file(proxy_conf_file, test_conf_file) as conf:
+        _modify_pipeline_substring(
+            conf,
+            "cache",
+            "cache etag-quoter")
+
+    return test_conf_file, swift_conf_file
+
+
 def _load_encryption(proxy_conf_file, swift_conf_file, **kwargs):
     """
     Load encryption configuration and override proxy-server.conf contents.
@@ -316,34 +349,18 @@ def _load_encryption(proxy_conf_file, swift_conf_file, **kwargs):
     # conf during wsgi load_app.
     # Therefore we must modify the [pipeline:main] section.
 
-    conf = ConfigParser()
-    conf.read(proxy_conf_file)
-    try:
-        section = 'pipeline:main'
-        pipeline = conf.get(section, 'pipeline')
-        pipeline = pipeline.replace(
+    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
+    with _modify_conf_file(proxy_conf_file, test_conf_file) as conf:
+        _modify_pipeline_substring(
+            conf,
             "proxy-logging proxy-server",
             "keymaster encryption proxy-logging proxy-server")
-        pipeline = pipeline.replace(
-            "cache listing_formats",
-            "cache etag-quoter listing_formats")
-        conf.set(section, 'pipeline', pipeline)
-        root_secret = base64.b64encode(os.urandom(32))
-        if not six.PY2:
-            root_secret = root_secret.decode('ascii')
+        root_secret = base64.b64encode(os.urandom(32)).decode('ascii')
         conf.set('filter:keymaster', 'encryption_root_secret', root_secret)
         conf.set('filter:versioned_writes', 'allow_object_versioning', 'true')
         conf.set('filter:etag-quoter', 'enable_by_default', 'true')
-    except NoSectionError as err:
-        msg = 'Error problem with proxy conf file %s: %s' % \
-              (proxy_conf_file, err)
-        raise InProcessException(msg)
 
-    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
-    with open(test_conf_file, 'w') as fp:
-        conf.write(fp)
-
-    return test_conf_file, swift_conf_file
+    return _load_etag_quoter(test_conf_file, swift_conf_file)
 
 
 def _load_ec_as_default_policy(proxy_conf_file, swift_conf_file, **kwargs):
@@ -356,29 +373,26 @@ def _load_ec_as_default_policy(proxy_conf_file, swift_conf_file, **kwargs):
     """
     _debug('Setting configuration for default EC policy')
 
-    conf = ConfigParser()
-    conf.read(swift_conf_file)
-    # remove existing policy sections that came with swift.conf-sample
-    for section in list(conf.sections()):
-        if section.startswith('storage-policy'):
-            conf.remove_section(section)
-    # add new policy 0 section for an EC policy
-    conf.add_section('storage-policy:0')
-    ec_policy_spec = {
-        'name': 'ec-test',
-        'policy_type': 'erasure_coding',
-        'ec_type': 'liberasurecode_rs_vand',
-        'ec_num_data_fragments': 2,
-        'ec_num_parity_fragments': 1,
-        'ec_object_segment_size': 1048576,
-        'default': True
-    }
+    with _modify_conf_file(swift_conf_file, swift_conf_file) as conf:
+        # remove existing policy sections that came with swift.conf-sample
+        for section in list(conf.sections()):
+            if section.startswith('storage-policy'):
+                conf.remove_section(section)
+        # add new policy 0 section for an EC policy
+        conf.add_section('storage-policy:0')
+        ec_policy_spec = {
+            'name': 'ec-test',
+            'policy_type': 'erasure_coding',
+            'ec_type': 'liberasurecode_rs_vand',
+            'ec_num_data_fragments': 2,
+            'ec_num_parity_fragments': 1,
+            'ec_object_segment_size': 1048576,
+            'default': True
+        }
 
-    for k, v in ec_policy_spec.items():
-        conf.set('storage-policy:0', k, str(v))
+        for k, v in ec_policy_spec.items():
+            conf.set('storage-policy:0', k, str(v))
 
-    with open(swift_conf_file, 'w') as fp:
-        conf.write(fp)
     return proxy_conf_file, swift_conf_file
 
 
@@ -404,33 +418,19 @@ def _load_domain_remap_staticweb(proxy_conf_file, swift_conf_file, **kwargs):
     # DEFAULTS options) then it prevents pipeline being loaded into the local
     # conf during wsgi load_app.
     # Therefore we must modify the [pipeline:main] section.
-    conf = ConfigParser()
-    conf.read(proxy_conf_file)
-    try:
-        section = 'pipeline:main'
-        old_pipeline = conf.get(section, 'pipeline')
-        pipeline = old_pipeline.replace(
-            " tempauth ",
-            " tempauth staticweb ")
-        pipeline = pipeline.replace(
-            " listing_formats ",
-            " domain_remap listing_formats ")
-        if pipeline == old_pipeline:
-            raise InProcessException(
-                "Failed to insert domain_remap and staticweb into pipeline: %s"
-                % old_pipeline)
-        conf.set(section, 'pipeline', pipeline)
+    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
+    with _modify_conf_file(proxy_conf_file, test_conf_file) as conf:
+        _modify_pipeline_substring(
+            conf,
+            "tempauth",
+            "tempauth staticweb")
+        _modify_pipeline_substring(
+            conf,
+            "listing_formats",
+            "domain_remap listing_formats")
         # set storage_domain in domain_remap middleware to match test config
         section = 'filter:domain_remap'
         conf.set(section, 'storage_domain', storage_domain)
-    except NoSectionError as err:
-        msg = 'Error problem with proxy conf file %s: %s' % \
-              (proxy_conf_file, err)
-        raise InProcessException(msg)
-
-    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
-    with open(test_conf_file, 'w') as fp:
-        conf.write(fp)
 
     return test_conf_file, swift_conf_file
 
@@ -453,26 +453,15 @@ def _load_s3api(proxy_conf_file, swift_conf_file, **kwargs):
     # conf during wsgi load_app.
     # Therefore we must modify the [pipeline:main] section.
 
-    conf = ConfigParser()
-    conf.read(proxy_conf_file)
-    try:
-        section = 'pipeline:main'
-        pipeline = conf.get(section, 'pipeline')
-        pipeline = pipeline.replace(
+    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
+    with _modify_conf_file(proxy_conf_file, test_conf_file) as conf:
+        _modify_pipeline_substring(
+            conf,
             "tempauth",
             "s3api tempauth")
-        conf.set(section, 'pipeline', pipeline)
         conf.set('filter:s3api', 's3_acl', 'true')
 
         conf.set('filter:versioned_writes', 'allow_object_versioning', 'true')
-    except NoSectionError as err:
-        msg = 'Error problem with proxy conf file %s: %s' % \
-              (proxy_conf_file, err)
-        raise InProcessException(msg)
-
-    test_conf_file = os.path.join(_testdir, 'proxy-server.conf')
-    with open(test_conf_file, 'w') as fp:
-        conf.write(fp)
 
     return test_conf_file, swift_conf_file
 
@@ -586,6 +575,7 @@ def in_process_setup(the_object_server=object_server):
         'container_update_timeout': '3',
         'allow_account_management': 'true',
         'account_autocreate': 'true',
+        'allow_open_expired': 'true',
         'allow_versions': 'True',
         'allow_versioned_writes': 'True',
         # Below are values used by the functional test framework, as well as
@@ -621,9 +611,9 @@ def in_process_setup(the_object_server=object_server):
         'reseller_prefix': 'AUTH, SERVICE',
         'SERVICE_require_group': 'service',
         # Reseller admin user (needs reseller_admin_role)
-        'account6': 'test6',
-        'username6': 'tester6',
-        'password6': 'testing6'
+        'account6': 'admin',
+        'username6': 'admin',
+        'password6': 'admin'
     })
 
     acc1lis = listen_zero()
@@ -833,7 +823,8 @@ def setup_package():
     global web_front_end
     web_front_end = config.get('web_front_end', 'integral')
     global normalized_urls
-    normalized_urls = config.get('normalized_urls', False)
+    normalized_urls = utils.config_true_value(
+        config.get('normalized_urls', False))
 
     global orig_collate
     orig_collate = locale.setlocale(locale.LC_COLLATE)
@@ -926,6 +917,13 @@ def setup_package():
                 swift_test_tenant[4] = config['account5']
             except KeyError:
                 pass  # no service token tests can be run
+            try:
+                swift_test_user[5] = '%s%s' % (
+                    '%s:' % config['account6'], config['username6'])
+                swift_test_key[5] = config['password6']
+                swift_test_tenant[5] = config['account6']
+            except KeyError:
+                pass  # no reseller admin tests can be run
 
             for _ in range(3):
                 swift_test_perm[_] = swift_test_user[_]
@@ -1081,38 +1079,35 @@ class InternalServerError(Exception):
     pass
 
 
-url = [None, None, None, None, None]
-token = [None, None, None, None, None]
-service_token = [None, None, None, None, None]
-parsed = [None, None, None, None, None]
-conn = [None, None, None, None, None]
+url = [None, None, None, None, None, None]
+token = [None, None, None, None, None, None]
+service_token = [None, None, None, None, None, None]
+parsed = [None, None, None, None, None, None]
+conn = [None, None, None, None, None, None]
 
 
 def reset_globals():
     global url, token, service_token, parsed, conn, config
-    url = [None, None, None, None, None]
-    token = [None, None, None, None, None]
-    service_token = [None, None, None, None, None]
-    parsed = [None, None, None, None, None]
-    conn = [None, None, None, None, None]
+    url = [None, None, None, None, None, None]
+    token = [None, None, None, None, None, None]
+    service_token = [None, None, None, None, None, None]
+    parsed = [None, None, None, None, None, None]
+    conn = [None, None, None, None, None, None]
     if config:
         config = {}
 
 
 def connection(url):
-    if has_insecure:
-        parsed_url, http_conn = http_connection(url, insecure=insecure)
-    else:
-        parsed_url, http_conn = http_connection(url)
+    parsed_url, http_conn = http_connection(url, insecure=insecure)
 
     orig_request = http_conn.request
 
     # Add the policy header if policy_specified is set
     def request_with_policy(method, url, body=None, headers={}):
-        version, account, container, obj = split_path(url, 1, 4, True)
-        if policy_specified and method == 'PUT' and container and not obj \
-                and 'X-Storage-Policy' not in headers:
-            headers['X-Storage-Policy'] = policy_specified
+        if policy_specified and method == 'PUT':
+            version, account, container, obj = split_path(url, 1, 4, True)
+            if container and not obj and 'X-Storage-Policy' not in headers:
+                headers['X-Storage-Policy'] = policy_specified
 
         return orig_request(method, url, body, headers)
 
@@ -1131,10 +1126,6 @@ def get_url_token(user_index, os_options):
                           swift_test_user[user_index],
                           swift_test_key[user_index],
                           **authargs)
-    if six.PY2 and not isinstance(url, bytes):
-        url = url.encode('utf-8')
-    if six.PY2 and not isinstance(token, bytes):
-        token = token.encode('utf-8')
     return url, token
 
 
