@@ -17,6 +17,7 @@ import botocore
 import datetime
 import unittest
 import os
+from unittest import SkipTest
 
 import test.functional as tf
 from swift.common.utils import config_true_value
@@ -39,7 +40,7 @@ class TestS3ApiBucket(S3ApiBaseBoto3):
         self.assertEqual(len(resp_objects), len(req_objects))
         for i, obj in enumerate(resp_objects):
             self.assertEqual(obj['Key'], req_objects[i])
-            self.assertEqual(type(obj['LastModified']), datetime.datetime)
+            self.assertIsInstance(obj['LastModified'], datetime.datetime)
             self.assertIn('ETag', obj)
             self.assertIn('Size', obj)
             self.assertEqual(obj['StorageClass'], 'STANDARD')
@@ -123,7 +124,7 @@ class TestS3ApiBucket(S3ApiBaseBoto3):
 
     def test_bucket_listing_with_staticweb(self):
         if 'staticweb' not in tf.cluster_info:
-            raise tf.SkipTest('Staticweb not enabled')
+            raise SkipTest('Staticweb not enabled')
         bucket = 'bucket'
 
         resp = self.conn.create_bucket(Bucket=bucket)
@@ -176,7 +177,7 @@ class TestS3ApiBucket(S3ApiBaseBoto3):
         if config_true_value(tf.cluster_info['s3api'].get('s3_acl')):
             if 's3_access_key2' not in tf.config or \
                     's3_secret_key2' not in tf.config:
-                raise tf.SkipTest(
+                raise SkipTest(
                     'Cannot test for BucketAlreadyExists with second user; '
                     'need s3_access_key2 and s3_secret_key2 configured')
 
@@ -196,8 +197,8 @@ class TestS3ApiBucket(S3ApiBaseBoto3):
     def test_put_bucket_error_key3(self):
         if 's3_access_key3' not in tf.config or \
                 's3_secret_key3' not in tf.config:
-            raise tf.SkipTest('Cannot test for AccessDenied; need '
-                              's3_access_key3 and s3_secret_key3 configured')
+            raise SkipTest('Cannot test for AccessDenied; need '
+                           's3_access_key3 and s3_secret_key3 configured')
 
         self.conn.create_bucket(Bucket='bucket')
         # If the user can't create buckets, they shouldn't even know
@@ -560,12 +561,70 @@ class TestS3ApiBucket(S3ApiBaseBoto3):
                                 _mangle_req_method)
         event_system.register('request-created.s3.CreateBucket',
                               _mangle_req_controller_method)
+
+        try:
+            import awscrt  # noqa: F401
+        except ImportError:
+            raise SkipTest('lower-case request methods require awscrt for '
+                           'proper signing (try `pip install awscrt`)')
         # the method exists in the controller but deny as MethodNotAllowed
         with self.assertRaises(botocore.exceptions.ClientError) as ctx:
             self.conn.create_bucket(Bucket='bucket')
         self.assertEqual(
             ctx.exception.response['Error']['Code'], 'MethodNotAllowed')
 
+    def test_bucket_get_object_lock_configuration(self):
+        bucket = 'bucket'
+
+        # PUT Bucket
+        resp = self.conn.create_bucket(Bucket=bucket)
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        headers = resp['ResponseMetadata']['HTTPHeaders']
+        self.assertCommonResponseHeaders(headers)
+
+        # now attempt to get object_lock_configuration from new bucket.
+        with self.assertRaises(botocore.exceptions.ClientError) as ce:
+            self.conn.get_object_lock_configuration(
+                Bucket=bucket)
+        self.assertEqual(
+            ce.exception.response['ResponseMetadata']['HTTPStatusCode'],
+            404)
+        self.assertEqual(
+            ce.exception.response['Error']['Code'],
+            'ObjectLockConfigurationNotFoundError')
+
+        self.assertEqual(
+            str(ce.exception),
+            'An error occurred (ObjectLockConfigurationNotFoundError) when '
+            'calling the GetObjectLockConfiguration operation: Object Lock '
+            'configuration does not exist for this bucket')
+
+    def test_bucket_put_object_lock_configuration(self):
+        bucket = 'bucket'
+
+        # PUT Bucket
+        resp = self.conn.create_bucket(Bucket=bucket)
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        headers = resp['ResponseMetadata']['HTTPHeaders']
+        self.assertCommonResponseHeaders(headers)
+
+        # now attempt to get object_lock_configuration from new bucket.
+        with self.assertRaises(botocore.exceptions.ClientError) as ce:
+            self.conn.put_object_lock_configuration(
+                Bucket=bucket, ObjectLockConfiguration={})
+
+        self.assertEqual(
+            ce.exception.response['ResponseMetadata']['HTTPStatusCode'],
+            501)
+        self.assertEqual(
+            ce.exception.response['Error']['Code'],
+            'NotImplemented')
+
+        self.assertEqual(str(ce.exception),
+                         'An error occurred (NotImplemented) when calling '
+                         'the PutObjectLockConfiguration operation: The '
+                         'requested resource is not implemented')
+
 
 class TestS3ApiBucketSigV4(TestS3ApiBucket):
     @classmethod
