@@ -13,30 +13,25 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 import unittest
-import os
-import boto
-
-# For an issue with venv and distutils, disable pylint message here
-# pylint: disable-msg=E0611,F0401
-from distutils.version import StrictVersion
 
 import calendar
 import email.parser
 from email.utils import formatdate, parsedate
 from time import mktime
-import six
 
 import test.functional as tf
+from swift.common import utils
 
 from swift.common.middleware.s3api.etree import fromstring
 from swift.common.middleware.s3api.utils import S3Timestamp
 from swift.common.utils import md5, quote
 
-from test.functional.s3api import S3ApiBase
+from test.functional.s3api import S3ApiBase, SigV4Mixin, \
+    skip_boto2_sort_header_bug, S3ApiBaseBoto3, get_boto3_conn
 from test.functional.s3api.s3_test_client import Connection
-from test.functional.s3api.utils import get_error_code, calculate_md5
+from test.functional.s3api.utils import get_error_code, calculate_md5, \
+    get_error_msg
 
 DAY = 86400.0  # 60 * 60 * 24 (sec)
 
@@ -49,6 +44,43 @@ def tearDownModule():
     tf.teardown_package()
 
 
+class TestS3ApiObjectBoto3(S3ApiBaseBoto3):
+    def setUp(self):
+        super().setUp()
+        self.conn = get_boto3_conn(tf.config['s3_access_key'],
+                                   tf.config['s3_secret_key'])
+        self.bucket = 'test-bucket'
+        resp = self.conn.create_bucket(Bucket=self.bucket)
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+
+    def test_put(self):
+        body = b'abcd' * 8192
+        resp = self.conn.put_object(Bucket=self.bucket, Key='obj', Body=body)
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        resp = self.conn.get_object(Bucket=self.bucket, Key='obj')
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        self.assertEqual(body, resp['Body'].read())
+
+    def test_put_chunked(self):
+        body = b'abcd' * 8192
+        resp = self.conn.put_object(Bucket=self.bucket, Key='obj', Body=body,
+                                    ContentEncoding='aws-chunked')
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        resp = self.conn.get_object(Bucket=self.bucket, Key='obj')
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        self.assertEqual(body, resp['Body'].read())
+
+    def test_put_chunked_sha256(self):
+        body = b'abcd' * 8192
+        resp = self.conn.put_object(Bucket=self.bucket, Key='obj', Body=body,
+                                    ContentEncoding='aws-chunked',
+                                    ChecksumAlgorithm='SHA256')
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        resp = self.conn.get_object(Bucket=self.bucket, Key='obj')
+        self.assertEqual(200, resp['ResponseMetadata']['HTTPStatusCode'])
+        self.assertEqual(body, resp['Body'].read())
+
+
 class TestS3ApiObject(S3ApiBase):
     def setUp(self):
         super(TestS3ApiObject, self).setUp()
@@ -136,6 +168,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertTrue(headers['last-modified'] is not None)
         self.assertTrue(headers['content-type'] is not None)
         self.assertEqual(headers['content-length'], str(len(content)))
+        self.assertEqual(headers['accept-ranges'], 'bytes')
 
         # HEAD Object
         status, headers, body = \
@@ -146,6 +179,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertTrue(headers['last-modified'] is not None)
         self.assertTrue('content-type' in headers)
         self.assertEqual(headers['content-length'], str(len(content)))
+        self.assertEqual(headers['accept-ranges'], 'bytes')
 
         # DELETE Object
         status, headers, body = \
@@ -366,6 +400,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_metadata(self):
         self._test_put_object_headers({
             'X-Amz-Meta-Bar': 'foo',
@@ -417,6 +452,53 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    def test_put_object_valid_delete_headers(self):
+        obj = 'object'
+        content = b'abcdefghij'
+        ts = utils.Timestamp.now()
+        delete_at = {'X-Delete-At': str(int(ts) + 70)}
+        delete_after = {'X-Delete-After': str(int(ts) + 130)}
+        status, delete_at, body = \
+            self.conn.make_request('PUT', self.bucket, obj, delete_at, content)
+        self.assertEqual(status, 200)
+        status, delete_after, body = \
+            self.conn.make_request('PUT', self.bucket, obj, delete_after,
+                                   content)
+        self.assertEqual(status, 200)
+
+    def test_put_object_invalid_x_delete_at(self):
+        obj = 'object'
+        content = b'abcdefghij'
+        ts = utils.Timestamp.now()
+        headers = {'X-Delete-At': str(int(ts) - 140)}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'X-Delete-At in past')
+        headers = {'X-Delete-At': 'test'}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'Non-integer X-Delete-At')
+
+    def test_put_object_invalid_x_delete_after(self):
+        obj = 'object'
+        content = b'abcdefghij'
+        headers = {'X-Delete-After': 'test'}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'Non-integer X-Delete-After')
+        headers = {'X-Delete-After': '-140'}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'X-Delete-After in past')
+
     def test_put_object_copy_source_params(self):
         obj = 'object'
         src_headers = {'X-Amz-Meta-Test': 'src'}
@@ -537,6 +619,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('PUT', dst_bucket, dst_obj, headers)
         self.assertEqual(status, 400)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_modified_since(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -557,6 +640,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_unmodified_since(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -577,6 +661,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_match(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -596,6 +681,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_none_match(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -622,6 +708,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-type'], 'text/plain')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_response_content_language(self):
         obj = 'object'
@@ -633,6 +720,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-language'], 'en')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_response_cache_control(self):
         obj = 'object'
@@ -643,6 +731,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, query=query)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertEqual(headers['cache-control'], 'private')
 
     def test_get_object_response_content_disposition(self):
@@ -655,6 +744,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-disposition'], 'inline')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_response_content_encoding(self):
         obj = 'object'
@@ -666,6 +756,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-encoding'], 'gzip')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_range(self):
         obj = 'object'
@@ -681,6 +772,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
         self.assertTrue('content-length' in headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertTrue('x-amz-meta-test' in headers)
         self.assertEqual('swift', headers['x-amz-meta-test'])
@@ -693,6 +785,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
         self.assertTrue('content-length' in headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertTrue('x-amz-meta-test' in headers)
         self.assertEqual('swift', headers['x-amz-meta-test'])
@@ -703,6 +796,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertTrue('content-length' in headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertTrue('x-amz-meta-test' in headers)
@@ -716,6 +810,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertIn('content-length', headers)
 
         self.assertIn('content-type', headers)  # sanity
@@ -727,10 +822,7 @@ class TestS3ApiObject(S3ApiBase):
 
         # TODO: Using swift.common.utils.multipart_byteranges_to_document_iters
         #       could be easy enough.
-        if six.PY2:
-            parser = email.parser.FeedParser()
-        else:
-            parser = email.parser.BytesFeedParser()
+        parser = email.parser.BytesFeedParser()
         parser.feed(
             b"Content-Type: multipart/byterange; boundary=%s\r\n\r\n" %
             boundary_str.encode('ascii'))
@@ -767,6 +859,7 @@ class TestS3ApiObject(S3ApiBase):
         status, headers, body = \
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
+        self.assertTrue('accept-ranges' in headers)
         self.assertCommonResponseHeaders(headers)
 
     def test_get_object_if_unmodified_since(self):
@@ -782,7 +875,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
-
+        self.assertTrue('accept-ranges' in headers)
         # check we can use the last modified time from the listing...
         status, headers, body = \
             self.conn.make_request('GET', self.bucket)
@@ -798,11 +891,13 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
         headers = {'If-Modified-Since': header_datetime}
         status, headers, body = \
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 304)
+        self.assertTrue('accept-ranges' in headers)
         self.assertCommonResponseHeaders(headers)
 
     def test_get_object_if_match(self):
@@ -818,6 +913,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_if_none_match(self):
         obj = 'object'
@@ -827,6 +923,7 @@ class TestS3ApiObject(S3ApiBase):
         status, headers, body = \
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
+        self.assertTrue('accept-ranges' in headers)
         self.assertCommonResponseHeaders(headers)
 
     def test_head_object_range(self):
@@ -839,18 +936,21 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
         headers = {'Range': 'bytes=5-'}
         status, headers, body = \
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
         headers = {'Range': 'bytes=-5'}
         status, headers, body = \
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_modified_since(self):
         obj = 'object'
@@ -865,6 +965,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_unmodified_since(self):
         obj = 'object'
@@ -879,6 +980,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_match(self):
         obj = 'object'
@@ -893,6 +995,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_none_match(self):
         obj = 'object'
@@ -903,48 +1006,11 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
 
-class TestS3ApiObjectSigV4(TestS3ApiObject):
-    @classmethod
-    def setUpClass(cls):
-        os.environ['S3_USE_SIGV4'] = "True"
-
-    @classmethod
-    def tearDownClass(cls):
-        del os.environ['S3_USE_SIGV4']
-
-    def setUp(self):
-        super(TestS3ApiObjectSigV4, self).setUp()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_metadata(self):
-        super(TestS3ApiObjectSigV4, self).test_put_object_metadata()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_modified_since(self):
-        super(TestS3ApiObjectSigV4, self).\
-            test_put_object_copy_source_if_modified_since()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_unmodified_since(self):
-        super(TestS3ApiObjectSigV4, self).\
-            test_put_object_copy_source_if_unmodified_since()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_match(self):
-        super(TestS3ApiObjectSigV4,
-              self).test_put_object_copy_source_if_match()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_none_match(self):
-        super(TestS3ApiObjectSigV4,
-              self).test_put_object_copy_source_if_none_match()
+class TestS3ApiObjectSigV4(TestS3ApiObject, SigV4Mixin):
+    pass
 
 
 if __name__ == '__main__':
