@@ -13,31 +13,30 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import hashlib
+import functools
 import io
 import json
 import os
 import random
-import sys
+import ssl
 import socket
 import time
 
 from unittest import SkipTest
 from xml.dom import minidom
 
-import six
-from six.moves import http_client
-from six.moves import urllib
+import http.client
+import urllib.parse
 from swiftclient import get_auth
 
 from swift.common import constraints
 from swift.common.http import is_success
 from swift.common.swob import str_to_wsgi, wsgi_to_str
-from swift.common.utils import config_true_value
+from swift.common.utils import config_true_value, md5
 
 from test import safe_repr
 
-http_client._MAXHEADERS = constraints.MAX_HEADER_COUNT
+http.client._MAXHEADERS = constraints.MAX_HEADER_COUNT
 
 
 class AuthenticationFailed(Exception):
@@ -138,10 +137,10 @@ def putrequest(self, method, url, skip_host=False, skip_accept_encoding=False):
             and self._HTTPConnection__response.isclosed():
         self._HTTPConnection__response = None
 
-    if self._HTTPConnection__state == http_client._CS_IDLE:
-        self._HTTPConnection__state = http_client._CS_REQ_STARTED
+    if self._HTTPConnection__state == http.client._CS_IDLE:
+        self._HTTPConnection__state = http.client._CS_REQ_STARTED
     else:
-        raise http_client.CannotSendRequest(self._HTTPConnection__state)
+        raise http.client.CannotSendRequest(self._HTTPConnection__state)
 
     self._method = method
     if not url:
@@ -225,15 +224,12 @@ class Connection(object):
 
     @storage_url.setter
     def storage_url(self, value):
-        if six.PY2 and not isinstance(value, bytes):
-            value = value.encode('utf-8')
-
         url = urllib.parse.urlparse(value)
 
         if url.scheme == 'http':
-            self.conn_class = http_client.HTTPConnection
+            self.conn_class = http.client.HTTPConnection
         elif url.scheme == 'https':
-            self.conn_class = http_client.HTTPSConnection
+            self.conn_class = http.client.HTTPSConnection
         else:
             raise ValueError('unexpected protocol %s' % (url.scheme))
 
@@ -250,7 +246,7 @@ class Connection(object):
     def storage_scheme(self):
         if self.conn_class is None:
             return None
-        if issubclass(self.conn_class, http_client.HTTPSConnection):
+        if issubclass(self.conn_class, http.client.HTTPSConnection):
             return 'https'
         return 'http'
 
@@ -312,9 +308,7 @@ class Connection(object):
         return json.loads(self.response.read())
 
     def http_connect(self):
-        if self.storage_scheme == 'https' and \
-                self.insecure and sys.version_info >= (2, 7, 9):
-            import ssl
+        if self.storage_scheme == 'https' and self.insecure:
             self.connection = self.conn_class(
                 self.storage_netloc,
                 context=ssl._create_unverified_context())
@@ -411,7 +405,7 @@ class Connection(object):
             except socket.timeout as e:
                 fail_messages.append(safe_repr(e))
                 continue
-            except http_client.HTTPException as e:
+            except http.client.HTTPException as e:
                 fail_messages.append(safe_repr(e))
                 continue
 
@@ -500,7 +494,7 @@ class Base(object):
                 'x-container-bytes-used',
             )
 
-        # NB: on py2, headers are always lower; on py3, they match the bytes
+        # NB: on py2, headers were always lower; on py3, they match the bytes
         # on the wire
         headers = dict((wsgi_to_str(h).lower(), wsgi_to_str(v))
                        for h, v in self.conn.response.getheaders())
@@ -568,22 +562,18 @@ class Account(Base):
         if status == 200:
             if format_type == 'json':
                 conts = json.loads(self.conn.response.read())
-                if six.PY2:
-                    for cont in conts:
-                        cont['name'] = cont['name'].encode('utf-8')
                 return conts
             elif format_type == 'xml':
                 conts = []
                 tree = minidom.parseString(self.conn.response.read())
                 for x in tree.getElementsByTagName('container'):
                     cont = {}
-                    for key in ['name', 'count', 'bytes', 'last_modified']:
+                    for key in ['name', 'count', 'bytes', 'last_modified',
+                                'storage_policy']:
                         cont[key] = x.getElementsByTagName(key)[0].\
                             childNodes[0].nodeValue
                     conts.append(cont)
                 for cont in conts:
-                    if six.PY2:
-                        cont['name'] = cont['name'].encode('utf-8')
                     for key in ('count', 'bytes'):
                         cont[key] = int(cont[key])
                 return conts
@@ -591,8 +581,6 @@ class Account(Base):
                 lines = self.conn.response.read().split(b'\n')
                 if lines and not lines[-1]:
                     lines = lines[:-1]
-                if six.PY2:
-                    return lines
                 return [line.decode('utf-8') for line in lines]
         elif status == 204:
             return []
@@ -672,29 +660,34 @@ class Container(Base):
         raise ResponseError(self.conn.response, 'POST',
                             self.conn.make_path(self.path))
 
-    def delete(self, hdrs=None, parms=None):
+    def delete(self, hdrs=None, parms=None, tolerate_missing=False):
         if hdrs is None:
             hdrs = {}
         if parms is None:
             parms = {}
+        allowed_codes = (204, 404) if tolerate_missing else (204, )
         return self.conn.make_request('DELETE', self.path, hdrs=hdrs,
-                                      parms=parms) == 204
+                                      parms=parms) in allowed_codes
 
-    def delete_files(self):
-        for f in listing_items(self.files):
+    def delete_files(self, tolerate_missing=False):
+        partialed_files = functools.partial(
+            self.files, tolerate_missing=tolerate_missing)
+
+        for f in listing_items(partialed_files):
             file_item = self.file(f)
             if not file_item.delete(tolerate_missing=True):
                 return False
 
-        return listing_empty(self.files)
+        return listing_empty(partialed_files)
 
     def delete_recursive(self):
-        return self.delete_files() and self.delete()
+        return self.delete_files(tolerate_missing=True) and \
+            self.delete(tolerate_missing=True)
 
     def file(self, file_name):
         return File(self.conn, self.account, self.name, file_name)
 
-    def files(self, hdrs=None, parms=None, cfg=None):
+    def files(self, hdrs=None, parms=None, cfg=None, tolerate_missing=False):
         if hdrs is None:
             hdrs = {}
         if parms is None:
@@ -711,15 +704,7 @@ class Container(Base):
                                         parms=parms, cfg=cfg)
         if status == 200:
             if format_type == 'json' or 'versions' in parms:
-                files = json.loads(self.conn.response.read())
-
-                if six.PY2:
-                    for file_item in files:
-                        for key in ('name', 'subdir', 'content_type',
-                                    'version_id'):
-                            if key in file_item:
-                                file_item[key] = file_item[key].encode('utf-8')
-                return files
+                return json.loads(self.conn.response.read())
             elif format_type == 'xml':
                 files = []
                 tree = minidom.parseString(self.conn.response.read())
@@ -740,15 +725,7 @@ class Container(Base):
                     files.append(file_item)
 
                 for file_item in files:
-                    if 'subdir' in file_item:
-                        if six.PY2:
-                            file_item['subdir'] = \
-                                file_item['subdir'].encode('utf-8')
-                    else:
-                        if six.PY2:
-                            file_item.update({
-                                k: file_item[k].encode('utf-8')
-                                for k in ('name', 'content_type')})
+                    if 'bytes' in file_item:
                         file_item['bytes'] = int(file_item['bytes'])
                 return files
             else:
@@ -757,12 +734,10 @@ class Container(Base):
                     lines = content.split(b'\n')
                     if lines and not lines[-1]:
                         lines = lines[:-1]
-                    if six.PY2:
-                        return lines
                     return [line.decode('utf-8') for line in lines]
                 else:
                     return []
-        elif status == 204:
+        elif status == 204 or (status == 404 and tolerate_missing):
             return []
 
         raise ResponseError(self.conn.response, 'GET',
@@ -851,7 +826,7 @@ class File(Base):
         if isinstance(data, bytes):
             data = io.BytesIO(data)
 
-        checksum = hashlib.md5()
+        checksum = md5(usedforsecurity=False)
         buff = data.read(block_size)
         while buff:
             checksum.update(buff)
@@ -861,6 +836,10 @@ class File(Base):
 
     def copy(self, dest_cont, dest_file, hdrs=None, parms=None, cfg=None,
              return_resp=False):
+        """
+        Make a copy of this object using a COPY request with a Destination
+        header.
+        """
         if hdrs is None:
             hdrs = {}
         if parms is None:
@@ -886,6 +865,29 @@ class File(Base):
             return self.conn.response
         return True
 
+    def copy_using_x_copy_from(self, dest_cont, dest_file, hdrs=None,
+                               parms=None, cfg=None, return_resp=False):
+        """
+        Make a copy of this object using a PUT request with an X-Copy-From
+        header.
+        """
+        if hdrs is None:
+            hdrs = {}
+        if parms is None:
+            parms = {}
+        if cfg is None:
+            cfg = {}
+        headers = {'X-Copy-From': '/'.join(self.path)}
+        headers.update(hdrs)
+        path = [dest_cont, dest_file]
+        if self.conn.make_request('PUT', path, hdrs=headers,
+                                  cfg=cfg, parms=parms) != 201:
+            raise ResponseError(self.conn.response, 'PUT',
+                                self.conn.make_path(path))
+        if return_resp:
+            return self.conn.response
+        return True
+
     def copy_account(self, dest_account, dest_cont, dest_file,
                      hdrs=None, parms=None, cfg=None):
         if hdrs is None:
@@ -933,7 +935,7 @@ class File(Base):
 
         return True
 
-    def info(self, hdrs=None, parms=None, cfg=None):
+    def info(self, hdrs=None, parms=None, cfg=None, exp_status=200):
         if hdrs is None:
             hdrs = {}
         if parms is None:
@@ -941,7 +943,7 @@ class File(Base):
         if cfg is None:
             cfg = {}
         if self.conn.make_request('HEAD', self.path, hdrs=hdrs,
-                                  parms=parms, cfg=cfg) != 200:
+                                  parms=parms, cfg=cfg) != exp_status:
 
             raise ResponseError(self.conn.response, 'HEAD',
                                 self.conn.make_path(self.path))
@@ -1058,7 +1060,7 @@ class File(Base):
             raise ResponseError(self.conn.response, 'GET',
                                 self.conn.make_path(self.path))
 
-        checksum = hashlib.md5()
+        checksum = md5(usedforsecurity=False)
 
         scratch = self.conn.response.read(8192)
         while len(scratch) > 0:
