@@ -20,15 +20,14 @@ import json
 import unittest
 from uuid import uuid4
 import time
+from unittest import SkipTest
 from xml.dom import minidom
 
-import six
-from six.moves import range
-
+from swift.common.header_key_dict import HeaderKeyDict
 from test.functional import check_response, retry, requires_acls, \
-    requires_policies, SkipTest, requires_bulk
+    requires_policies, requires_bulk
 import test.functional as tf
-from swift.common.utils import md5
+from swift.common.utils import md5, config_true_value
 
 
 def setUpModule():
@@ -107,11 +106,7 @@ class TestObject(unittest.TestCase):
 
         # delete an object
         def delete(url, token, parsed, conn, container, obj):
-            if six.PY2:
-                obj_name = obj['name'].encode('utf8')
-            else:
-                obj_name = obj['name']
-            path = '/'.join([parsed.path, container, obj_name])
+            path = '/'.join([parsed.path, container, obj['name']])
             conn.request('DELETE', path, '', {'X-Auth-Token': token})
             return check_response(conn)
 
@@ -464,6 +459,274 @@ class TestObject(unittest.TestCase):
         resp.read()
         self.assertEqual(resp.status, 201)
 
+    def test_open_expired_enabled(self):
+        allow_open_expired = config_true_value(tf.cluster_info['swift'].get(
+            'allow_open_expired', 'false'))
+
+        if not allow_open_expired:
+            raise SkipTest('allow_open_expired is disabled')
+
+        def put(url, token, parsed, conn):
+            dt = datetime.datetime.now()
+            epoch = time.mktime(dt.timetuple())
+            delete_time = str(int(epoch) + 2)
+            conn.request(
+                'PUT',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                {'X-Auth-Token': token,
+                 'Content-Length': '0',
+                 'X-Delete-At': delete_time})
+            return check_response(conn)
+        resp = retry(put)
+        resp.read()
+        self.assertEqual(resp.status, 201)
+
+        def get(url, token, parsed, conn, extra_headers=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'GET',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                headers)
+            return check_response(conn)
+
+        def head(url, token, parsed, conn, extra_headers=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'HEAD',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                headers)
+            return check_response(conn)
+
+        def post(url, token, parsed, conn, extra_headers=None):
+            dt = datetime.datetime.now()
+            epoch = time.mktime(dt.timetuple())
+            delete_time = str(int(epoch) + 2)
+            headers = {'X-Auth-Token': token,
+                       'Content-Length': '0',
+                       'X-Delete-At': delete_time
+                       }
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'POST',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                headers)
+            return check_response(conn)
+
+        resp = retry(get)
+        resp.read()
+        count = 0
+        while resp.status == 200 and count < 10:
+            resp = retry(get)
+            resp.read()
+            count += 1
+            time.sleep(1)
+
+        # check to see object has expired
+        self.assertEqual(resp.status, 404)
+
+        dt = datetime.datetime.now()
+        now = str(int(time.mktime(dt.timetuple())))
+        resp = retry(get, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        headers = HeaderKeyDict(resp.getheaders())
+        # read the expired object with magic x-open-expired header
+        self.assertEqual(resp.status, 200)
+        self.assertTrue(now > headers['X-Delete-At'])
+
+        resp = retry(head, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        # head expired object with magic x-open-expired header
+        self.assertEqual(resp.status, 200)
+
+        resp = retry(get)
+        resp.read()
+        # verify object is still expired
+        self.assertEqual(resp.status, 404)
+
+        # verify object is still expired if x-open-expire is False
+        resp = retry(get, extra_headers={'X-Open-Expired': False})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(get, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        self.assertEqual(resp.status, 200)
+        headers = HeaderKeyDict(resp.getheaders())
+        self.assertTrue(now > headers['X-Delete-At'])
+
+        resp = retry(head, extra_headers={'X-Open-Expired': False})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(head, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        self.assertEqual(resp.status, 200)
+        headers = HeaderKeyDict(resp.getheaders())
+        self.assertTrue(now > headers['X-Delete-At'])
+
+        resp = retry(post, extra_headers={'X-Open-Expired': False})
+        resp.read()
+        # verify object is not updated and remains deleted
+        self.assertEqual(resp.status, 404)
+
+        # object got restored with magic x-open-expired header
+        resp = retry(post, extra_headers={'X-Open-Expired': True,
+                                          'X-Object-Meta-Test': 'restored!'})
+        resp.read()
+        self.assertEqual(resp.status, 202)
+
+        # verify object could be restored and you can do normal GET
+        resp = retry(get)
+        resp.read()
+        self.assertEqual(resp.status, 200)
+        self.assertIn('X-Object-Meta-Test', resp.headers)
+        self.assertEqual(resp.headers['x-object-meta-test'], 'restored!')
+
+        # verify object is restored and you can do normal HEAD
+        resp = retry(head)
+        resp.read()
+        self.assertEqual(resp.status, 200)
+        # verify object is updated with advanced delete time
+        self.assertIn('X-Delete-At', resp.headers)
+
+        # To avoid an error when the object deletion in tearDown(),
+        # the object is added again.
+        resp = retry(put)
+        resp.read()
+        self.assertEqual(resp.status, 201)
+
+    def test_allow_open_expired_disabled(self):
+        allow_open_expired = config_true_value(tf.cluster_info['swift'].get(
+            'allow_open_expired', 'false'))
+
+        if allow_open_expired:
+            raise SkipTest('allow_open_expired is enabled')
+
+        def put(url, token, parsed, conn):
+            dt = datetime.datetime.now()
+            epoch = time.mktime(dt.timetuple())
+            delete_time = str(int(epoch) + 2)
+            conn.request(
+                'PUT',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                {'X-Auth-Token': token,
+                 'Content-Length': '0',
+                 'X-Delete-At': delete_time})
+            return check_response(conn)
+        resp = retry(put)
+        resp.read()
+        self.assertEqual(resp.status, 201)
+
+        def get(url, token, parsed, conn, extra_headers=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'GET',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                headers)
+            return check_response(conn)
+
+        def head(url, token, parsed, conn, extra_headers=None):
+            headers = {'X-Auth-Token': token}
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'HEAD',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                headers)
+            return check_response(conn)
+
+        def post(url, token, parsed, conn, extra_headers=None):
+            dt = datetime.datetime.now()
+            epoch = time.mktime(dt.timetuple())
+            delete_time = str(int(epoch) + 2)
+            headers = {'X-Auth-Token': token,
+                       'Content-Length': '0',
+                       'X-Delete-At': delete_time
+                       }
+            if extra_headers:
+                headers.update(extra_headers)
+            conn.request(
+                'POST',
+                '%s/%s/%s' % (parsed.path, self.container, 'x_delete_at'),
+                '',
+                headers)
+            return check_response(conn)
+
+        resp = retry(get)
+        resp.read()
+        count = 0
+        while resp.status == 200 and count < 10:
+            resp = retry(get)
+            resp.read()
+            count += 1
+            time.sleep(1)
+
+        # check to see object has expired
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(get, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        # read the expired object with magic x-open-expired header
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(head, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        # head expired object with magic x-open-expired header
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(get)
+        resp.read()
+        # verify object is still expired
+        self.assertEqual(resp.status, 404)
+
+        # verify object is still expired if x-open-expire is False
+        resp = retry(get, extra_headers={'X-Open-Expired': False})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(get, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(head, extra_headers={'X-Open-Expired': False})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(head, extra_headers={'X-Open-Expired': True})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        resp = retry(post, extra_headers={'X-Open-Expired': False})
+        resp.read()
+        # verify object is not updated and remains deleted
+        self.assertEqual(resp.status, 404)
+
+        # object cannot be restored with magic x-open-expired header
+        resp = retry(post, extra_headers={'X-Open-Expired': True,
+                                          'X-Object-Meta-Test': 'restored!'})
+        resp.read()
+        self.assertEqual(resp.status, 404)
+
+        # To avoid an error when the object deletion in tearDown(),
+        # the object is added again.
+        resp = retry(put)
+        resp.read()
+        self.assertEqual(resp.status, 201)
+
     def test_non_integer_x_delete_after(self):
         def put(url, token, parsed, conn):
             conn.request('PUT', '%s/%s/%s' % (parsed.path, self.container,
@@ -941,7 +1204,7 @@ class TestObject(unittest.TestCase):
     @requires_acls
     def test_read_only(self):
         if tf.skip3:
-            raise tf.SkipTest
+            raise SkipTest
 
         def get_listing(url, token, parsed, conn):
             conn.request('GET', '%s/%s' % (parsed.path, self.container), '',
@@ -991,9 +1254,7 @@ class TestObject(unittest.TestCase):
 
         # can list objects
         resp = retry(get_listing, use_account=3)
-        listing = resp.read()
-        if not six.PY2:
-            listing = listing.decode('utf8')
+        listing = resp.read().decode('utf8')
         self.assertEqual(resp.status, 200)
         self.assertIn(self.obj, listing.split('\n'))
 
@@ -1016,9 +1277,7 @@ class TestObject(unittest.TestCase):
 
         # sanity with account1
         resp = retry(get_listing, use_account=3)
-        listing = resp.read()
-        if not six.PY2:
-            listing = listing.decode('utf8')
+        listing = resp.read().decode('utf8')
         self.assertEqual(resp.status, 200)
         self.assertNotIn(obj_name, listing.split('\n'))
         self.assertIn(self.obj, listing.split('\n'))
@@ -1076,9 +1335,7 @@ class TestObject(unittest.TestCase):
 
         # can list objects
         resp = retry(get_listing, use_account=3)
-        listing = resp.read()
-        if not six.PY2:
-            listing = listing.decode('utf8')
+        listing = resp.read().decode('utf8')
         self.assertEqual(resp.status, 200)
         self.assertIn(self.obj, listing.split('\n'))
 
@@ -1101,9 +1358,7 @@ class TestObject(unittest.TestCase):
 
         # sanity with account1
         resp = retry(get_listing, use_account=3)
-        listing = resp.read()
-        if not six.PY2:
-            listing = listing.decode('utf8')
+        listing = resp.read().decode('utf8')
         self.assertEqual(resp.status, 200)
         self.assertIn(obj_name, listing.split('\n'))
         self.assertNotIn(self.obj, listing.split('\n'))
@@ -1161,9 +1416,7 @@ class TestObject(unittest.TestCase):
 
         # can list objects
         resp = retry(get_listing, use_account=3)
-        listing = resp.read()
-        if not six.PY2:
-            listing = listing.decode('utf8')
+        listing = resp.read().decode('utf8')
         self.assertEqual(resp.status, 200)
         self.assertIn(self.obj, listing.split('\n'))
 
@@ -1186,9 +1439,7 @@ class TestObject(unittest.TestCase):
 
         # sanity with account1
         resp = retry(get_listing, use_account=3)
-        listing = resp.read()
-        if not six.PY2:
-            listing = listing.decode('utf8')
+        listing = resp.read().decode('utf8')
         self.assertEqual(resp.status, 200)
         self.assertIn(obj_name, listing.split('\n'))
         self.assertNotIn(self.obj, listing)
@@ -1542,7 +1793,7 @@ class TestObject(unittest.TestCase):
         def put_obj(url, token, parsed, conn, obj):
             conn.request(
                 'PUT', '%s/%s/%s' % (parsed.path, self.container, obj),
-                'test', {'X-Auth-Token': token})
+                'test', {'X-Auth-Token': token, 'X-Object-Meta-Color': 'red'})
             return check_response(conn)
 
         def check_cors(url, token, parsed, conn,
@@ -1576,6 +1827,8 @@ class TestObject(unittest.TestCase):
         headers = dict((k.lower(), v) for k, v in resp.getheaders())
         self.assertEqual(headers.get('access-control-allow-origin'),
                          '*')
+        # Just a pre-flight; this doesn't show up yet
+        self.assertNotIn('access-control-expose-headers', headers)
 
         resp = retry(check_cors,
                      'GET', 'cat', {'Origin': 'http://m.com'})
@@ -1583,6 +1836,8 @@ class TestObject(unittest.TestCase):
         headers = dict((k.lower(), v) for k, v in resp.getheaders())
         self.assertEqual(headers.get('access-control-allow-origin'),
                          '*')
+        self.assertIn('x-object-meta-color', headers.get(
+            'access-control-expose-headers').split(', '))
 
         resp = retry(check_cors,
                      'GET', 'cat', {'Origin': 'http://m.com',
@@ -1591,6 +1846,8 @@ class TestObject(unittest.TestCase):
         headers = dict((k.lower(), v) for k, v in resp.getheaders())
         self.assertEqual(headers.get('access-control-allow-origin'),
                          '*')
+        self.assertIn('x-object-meta-color', headers.get(
+            'access-control-expose-headers').split(', '))
 
         ####################
 
@@ -1685,9 +1942,7 @@ class TestObject(unittest.TestCase):
         for c, o, body in validate_requests:
             resp = retry(get_obj, c, o)
             self.assertEqual(resp.status, 200)
-            if not six.PY2:
-                body = body.encode('utf8')
-            self.assertEqual(body, resp.read())
+            self.assertEqual(body.encode('utf8'), resp.read())
 
     @requires_bulk
     def test_bulk_delete(self):
