@@ -19,10 +19,12 @@ import unittest
 import uuid
 import random
 import time
-import six
 
 from swift.common.direct_client import DirectClientException
+from swift.common.header_key_dict import HeaderKeyDict
+from swift.common.internal_client import UnexpectedResponse
 from swift.common.manager import Manager
+from swift.common.swob import wsgi_to_str, str_to_wsgi
 from swift.common.utils import md5
 from swift.obj.reconstructor import ObjectReconstructor
 from test.probe.common import ECProbeTest
@@ -54,14 +56,12 @@ class Body(object):
         self.hasher.update(self.chunk)
         return self.chunk
 
-    # for py2 compat
-    next = __next__
-
 
 class TestReconstructorRebuild(ECProbeTest):
 
     def setUp(self):
         super(TestReconstructorRebuild, self).setUp()
+        self.int_client = self.make_internal_client()
         # create EC container
         headers = {'X-Storage-Policy': self.policy.name}
         client.put_container(self.url, self.token, self.container_name,
@@ -86,6 +86,19 @@ class TestReconstructorRebuild(ECProbeTest):
                 'X-Backend-Durable-Timestamp', hdrs,
                 'Missing durable timestamp in %r' % self.frag_headers)
 
+    def proxy_get(self):
+        # Use internal-client instead of python-swiftclient, since we can't
+        # handle UTF-8 headers properly w/ swiftclient.
+        # Still a proxy-server tho!
+        status, headers, body = self.int_client.get_object(
+            self.account,
+            self.container_name.decode('utf-8'),
+            self.object_name.decode('utf-8'))
+        resp_checksum = md5(usedforsecurity=False)
+        for chunk in body:
+            resp_checksum.update(chunk)
+        return HeaderKeyDict(headers), resp_checksum.hexdigest()
+
     def _format_node(self, node):
         return '%s#%s' % (node['device'], node['index'])
 
@@ -134,8 +147,12 @@ class TestReconstructorRebuild(ECProbeTest):
             headers, etag = self.proxy_get()
             self.assertEqual(self.etag, etag)
             for key in self.headers_post:
-                self.assertIn(key, headers)
-                self.assertEqual(self.headers_post[key], headers[key])
+                # Since we use internal_client for the GET, headers come back
+                # as WSGI strings
+                wsgi_key = str_to_wsgi(key)
+                self.assertIn(wsgi_key, headers)
+                self.assertEqual(self.headers_post[key],
+                                 wsgi_to_str(headers[wsgi_key]))
 
         # fire up reconstructor
         for i in range(reconstructor_cycles):
@@ -146,8 +163,10 @@ class TestReconstructorRebuild(ECProbeTest):
             headers, etag = self.proxy_get()
             self.assertEqual(self.etag, etag)
             for key in self.headers_post:
-                self.assertIn(key, headers)
-                self.assertEqual(self.headers_post[key], headers[key])
+                wsgi_key = str_to_wsgi(key)
+                self.assertIn(wsgi_key, headers)
+                self.assertEqual(self.headers_post[key],
+                                 wsgi_to_str(headers[wsgi_key]))
         # check all frags are intact, durable and have expected metadata
         with self._annotate_failure_with_scenario(failed, non_durable):
             frag_headers, frag_etags = self._assert_all_nodes_have_frag()
@@ -272,8 +291,8 @@ class TestReconstructorRebuild(ECProbeTest):
         while time.time() < timeout:
             try:
                 self.proxy_get()
-            except ClientException as e:
-                if e.http_status == 404:
+            except UnexpectedResponse as e:
+                if e.resp.status_int == 404:
                     break
                 else:
                     raise
@@ -412,12 +431,19 @@ class TestReconstructorRebuild(ECProbeTest):
             client.get_object(self.url, self.token, self.container_name,
                               self.object_name)
         self.assertEqual(503, cm.exception.http_status)
-        # ... but client GET succeeds
-        headers = client.head_object(self.url, self.token, self.container_name,
-                                     self.object_name)
+        # ... but client HEAD succeeds
+
+        path = self.int_client.make_path(
+            self.account,
+            self.container_name.decode('utf-8'),
+            self.object_name.decode('utf-8'))
+        resp = self.int_client.make_request(
+            'HEAD', path, {}, acceptable_statuses=(2,))
         for key in self.headers_post:
-            self.assertIn(key, headers)
-            self.assertEqual(self.headers_post[key], headers[key])
+            wsgi_key = str_to_wsgi(key)
+            self.assertIn(wsgi_key, resp.headers)
+            self.assertEqual(self.headers_post[key],
+                             wsgi_to_str(resp.headers[wsgi_key]))
 
         # run the reconstructor without quarantine_threshold set
         error_lines = []
@@ -426,7 +452,7 @@ class TestReconstructorRebuild(ECProbeTest):
             reconstructor = self.run_custom_daemon(
                 ObjectReconstructor, 'object-reconstructor', conf_index,
                 {'quarantine_age': '0'})
-            logger = reconstructor.logger.logger
+            logger = reconstructor.logger
             error_lines.append(logger.get_lines_for_level('error'))
             warning_lines.append(logger.get_lines_for_level('warning'))
 
@@ -463,7 +489,7 @@ class TestReconstructorRebuild(ECProbeTest):
             reconstructor = self.run_custom_daemon(
                 ObjectReconstructor, 'object-reconstructor', conf_index,
                 {'quarantine_age': '0', 'quarantine_threshold': '1'})
-            logger = reconstructor.logger.logger
+            logger = reconstructor.logger
             error_lines.append(logger.get_lines_for_level('error'))
             warning_lines.append(logger.get_lines_for_level('warning'))
 
@@ -516,7 +542,7 @@ class TestReconstructorRebuild(ECProbeTest):
             reconstructor = self.run_custom_daemon(
                 ObjectReconstructor, 'object-reconstructor', conf_index,
                 {'quarantine_age': '0', 'quarantine_threshold': '1'})
-            logger = reconstructor.logger.logger
+            logger = reconstructor.logger
             error_lines.append(logger.get_lines_for_level('error'))
             warning_lines.append(logger.get_lines_for_level('warning'))
 
@@ -526,15 +552,11 @@ class TestReconstructorRebuild(ECProbeTest):
             self.assertEqual([], lines)
 
 
-if six.PY2:
-    # The non-ASCII chars in metadata cause test hangs in
-    # _assert_all_nodes_have_frag because of https://bugs.python.org/issue37093
+class TestReconstructorRebuildUTF8(TestReconstructorRebuild):
 
-    class TestReconstructorRebuildUTF8(TestReconstructorRebuild):
-
-        def _make_name(self, prefix):
-            return b'%s\xc3\xa8-%s' % (
-                prefix.encode(), str(uuid.uuid4()).encode())
+    def _make_name(self, prefix):
+        return b'%s\xc3\xa8-%s' % (
+            prefix.encode(), str(uuid.uuid4()).encode())
 
 
 if __name__ == "__main__":
