@@ -21,7 +21,7 @@ import time
 
 import boto3
 from botocore.exceptions import ClientError
-from six.moves import urllib
+import urllib.parse
 
 from swift.common.utils import config_true_value, readconf
 
@@ -50,8 +50,8 @@ def load_aws_config(conf_file):
     conf = readconf(conf_file, 'default')
     global _CONFIG
     _CONFIG = {
-        'endpoint': 'https://s3.amazonaws.com',
-        'region': 'us-east-1',
+        'endpoint': conf.get('endpoint', 'https://s3.amazonaws.com'),
+        'region': conf.get('region', 'us-east-1'),
         'access_key1': conf.get('aws_access_key_id'),
         'secret_key1': conf.get('aws_secret_access_key'),
         'session_token1': conf.get('aws_session_token')
@@ -144,6 +144,18 @@ def get_s3_client(user=1, signature_version='s3v4', addressing_style='path'):
     )
 
 
+def etag_from_resp(response):
+    return response['ETag']
+
+
+def code_from_error(error):
+    return error.response['Error']['Code']
+
+
+def status_from_error(error):
+    return error.response['ResponseMetadata']['HTTPStatusCode']
+
+
 TEST_PREFIX = 's3api-test-'
 
 
@@ -249,7 +261,7 @@ class BaseS3TestCase(BaseS3Mixin, unittest.TestCase):
 class BaseS3TestCaseWithBucket(BaseS3Mixin, unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        cls.bucket_name = cls.create_name('test-bucket')
+        cls.bucket_name = cls.create_name('bucket')
         client = cls.get_s3_client(1)
         client.create_bucket(Bucket=cls.bucket_name)
 
