@@ -15,7 +15,6 @@
 
 """ Swift tests """
 
-from __future__ import print_function
 import os
 import copy
 import logging
@@ -23,10 +22,7 @@ import logging.handlers
 import sys
 from contextlib import contextmanager, closing
 from collections import defaultdict
-try:
-    from collections.abc import Iterable
-except ImportError:
-    from collections import Iterable  # py2
+from collections.abc import Iterable
 import itertools
 from numbers import Number
 from tempfile import NamedTemporaryFile
@@ -42,17 +38,15 @@ import random
 import errno
 import xattr
 from io import BytesIO
+from uuid import uuid4
+import pickle
+from http.client import HTTPException
 
-import six
-import six.moves.cPickle as pickle
-from six.moves import range
-from six.moves.http_client import HTTPException
-
-from swift.common import storage_policy, swob, utils
+from swift.common import storage_policy, swob, utils, exceptions
 from swift.common.memcached import MemcacheConnectionError
 from swift.common.storage_policy import (StoragePolicy, ECStoragePolicy,
                                          VALID_EC_TYPES)
-from swift.common.utils import Timestamp, md5
+from swift.common.utils import Timestamp, md5, close_if_possible, checksum
 from test import get_config
 from test.debug_logger import FakeLogger
 from swift.common.header_key_dict import HeaderKeyDict
@@ -61,17 +55,11 @@ from swift.obj import server
 
 import functools
 from gzip import GzipFile
-import mock as mocklib
+from unittest import mock as mocklib
 import inspect
-import unittest
+from unittest import SkipTest
 
 
-class SkipTest(unittest.SkipTest):
-    pass
-
-
-EMPTY_ETAG = md5(usedforsecurity=False).hexdigest()
-
 # try not to import this module from swift
 if not os.path.basename(sys.argv[0]).startswith('swift'):
     # never patch HASH_PATH_SUFFIX AGAIN!
@@ -413,6 +401,7 @@ class FakeMemcache(object):
 
     def __init__(self, error_on_set=None, error_on_get=None):
         self.store = {}
+        self.times = {}
         self.calls = []
         self.error_on_incr = False
         self.error_on_get = error_on_get or []
@@ -443,6 +432,7 @@ class FakeMemcache(object):
         else:
             assert isinstance(value, (str, bytes))
         self.store[key] = value
+        self.times[key] = time
         return True
 
     @track
@@ -462,20 +452,18 @@ class FakeMemcache(object):
     def decr(self, key, delta=1, time=0):
         return self.incr(key, delta=-delta, time=time)
 
-    @contextmanager
-    def soft_lock(self, key, timeout=0, retries=5):
-        yield True
-
     @track
     def delete(self, key):
         try:
             del self.store[key]
+            del self.times[key]
         except Exception:
             pass
         return True
 
     def delete_all(self):
         self.store.clear()
+        self.times.clear()
 
 
 # This decorator only makes sense in the context of FakeMemcache;
@@ -496,8 +484,6 @@ class FakeIterable(object):
         self.next_call_count += 1
         return next(self.values)
 
-    next = __next__  # py2
-
     def close(self):
         self.close_call_count += 1
 
@@ -519,6 +505,17 @@ def readuntil2crlfs(fd):
     return rv
 
 
+def readlength(fd, size, timeout=1.0):
+    buf = b''
+    with eventlet.Timeout(timeout):
+        while len(buf) < size:
+            chunk = fd.read(min(64, size - len(buf)))
+            buf += chunk
+            if len(buf) >= size:
+                break
+    return buf
+
+
 def connect_tcp(hostport):
     rv = socket.socket()
     rv.connect(hostport)
@@ -800,7 +797,7 @@ def fake_http_connect(*code_iter, **kwargs):
             self.received = 0
             self.etag = etag
             self.body = body
-            self.headers = headers or {}
+            self._headers = headers or {}
             self.expect_headers = expect_headers or {}
             if timestamp == -1:
                 # -1 is reserved to mean "magic default"
@@ -885,7 +882,7 @@ def fake_http_connect(*code_iter, **kwargs):
                     headers['x-container-timestamp'] = '1'
             except StopIteration:
                 pass
-            headers.update(self.headers)
+            headers.update(self._headers)
             return headers.items()
 
         def get_slow(self):
@@ -977,14 +974,9 @@ def fake_http_connect(*code_iter, **kwargs):
         if 'give_connect' in kwargs:
             give_conn_fn = kwargs['give_connect']
 
-            if six.PY2:
-                argspec = inspect.getargspec(give_conn_fn)
-                if argspec.keywords or 'connection_id' in argspec.args:
-                    ckwargs['connection_id'] = i
-            else:
-                argspec = inspect.getfullargspec(give_conn_fn)
-                if argspec.varkw or 'connection_id' in argspec.args:
-                    ckwargs['connection_id'] = i
+            argspec = inspect.getfullargspec(give_conn_fn)
+            if argspec.varkw or 'connection_id' in argspec.args:
+                ckwargs['connection_id'] = i
             give_conn_fn(*args, **ckwargs)
         etag = next(etag_iter)
         headers = next(headers_iter)
@@ -1017,8 +1009,6 @@ def mocked_http_conn(*args, **kwargs):
     responses = []
 
     def capture_requests(ip, port, method, path, headers, qs, ssl):
-        if six.PY2 and not isinstance(ip, bytes):
-            ip = ip.encode('ascii')
         req = {
             'ip': ip,
             'port': port,
@@ -1041,8 +1031,11 @@ def mocked_http_conn(*args, **kwargs):
         if left_over_status:
             raise AssertionError('left over status %r' % left_over_status)
         if fake_conn.unexpected_requests:
-            raise AssertionError('unexpected requests:\n%s' % '\n  '.join(
-                '%r' % (req,) for req in fake_conn.unexpected_requests))
+            raise AssertionError(
+                '%d unexpected requests:\n%s' %
+                (len(fake_conn.unexpected_requests),
+                 '\n  '.join('%r' % (req,)
+                             for req in fake_conn.unexpected_requests)))
 
 
 def make_timestamp_iter(offset=0):
@@ -1092,10 +1085,32 @@ def requires_o_tmpfile_support_in_tmp(func):
     return wrapper
 
 
+def requires_crc32c(func):
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        try:
+            checksum.crc32c()
+        except NotImplementedError as e:
+            raise SkipTest(str(e))
+        return func(*args, **kwargs)
+    return wrapper
+
+
+def requires_crc64nvme(func):
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        try:
+            checksum.crc64nvme()
+        except NotImplementedError as e:
+            raise SkipTest(str(e))
+        return func(*args, **kwargs)
+    return wrapper
+
+
 class StubResponse(object):
 
     def __init__(self, status, body=b'', headers=None, frag_index=None,
-                 slowdown=None):
+                 slowdown=None, slowdown_after=0):
         self.status = status
         self.body = body
         self.readable = BytesIO(body)
@@ -1103,13 +1118,17 @@ class StubResponse(object):
             self._slowdown = iter(slowdown)
         except TypeError:
             self._slowdown = iter([slowdown])
+        self.slowdown_after = slowdown_after
         self.headers = HeaderKeyDict(headers)
         if frag_index is not None:
             self.headers['X-Object-Sysmeta-Ec-Frag-Index'] = frag_index
         fake_reason = ('Fake', 'This response is a lie.')
         self.reason = swob.RESPONSE_REASONS.get(status, fake_reason)[0]
+        self.bytes_read = 0
 
     def slowdown(self):
+        if self.bytes_read < self.slowdown_after:
+            return
         try:
             wait = next(self._slowdown)
         except StopIteration:
@@ -1131,11 +1150,15 @@ class StubResponse(object):
 
     def read(self, amt=0):
         self.slowdown()
-        return self.readable.read(amt)
+        res = self.readable.read(amt)
+        self.bytes_read += len(res)
+        return res
 
     def readline(self, size=-1):
         self.slowdown()
-        return self.readable.readline(size)
+        res = self.readable.readline(size)
+        self.bytes_read += len(res)
+        return res
 
     def __repr__(self):
         info = ['Status: %s' % self.status]
@@ -1409,3 +1432,136 @@ def group_by_byte(contents):
     return [
         (char, sum(1 for _ in grp))
         for char, grp in itertools.groupby(byte_iter)]
+
+
+def generate_db_path(tempdir, server_type):
+    return os.path.join(
+        tempdir, '%ss' % server_type, 'part', 'suffix', 'hash',
+        '%s-%s.db' % (server_type, uuid4()))
+
+
+class FakeSource(object):
+    def __init__(self, chunks, headers=None, body=b''):
+        self.chunks = list(chunks)
+        self.headers = headers or {}
+        self.status = 200
+        self.swift_conn = None
+        self.body = body
+
+    def read(self, _read_size):
+        if self.chunks:
+            chunk = self.chunks.pop(0)
+            if chunk is None:
+                raise exceptions.ChunkReadTimeout()
+            else:
+                return chunk
+        else:
+            return self.body
+
+    def getheader(self, header):
+        # content-length for the whole object is generated dynamically
+        # by summing non-None chunks
+        if header.lower() == "content-length":
+            if self.chunks:
+                return str(sum(len(c) for c in self.chunks
+                               if c is not None))
+            return len(self.read(-1))
+        return self.headers.get(header.lower())
+
+    def getheaders(self):
+        return [('content-length', self.getheader('content-length'))] + \
+               [(k, v) for k, v in self.headers.items()]
+
+
+class CaptureIterator(object):
+    """
+    Wraps an iterable, forwarding all calls to the wrapped iterable but
+    capturing the calls via a callback.
+
+    This class may be used to observe garbage collection, so tests should not
+    have to hold a reference to instances of this class because that would
+    prevent them being garbage collected. Calls are therefore captured via a
+    callback rather than being stashed locally.
+
+    :param wrapped: an iterable to wrap.
+    :param call_capture_callback: a function that will be called to capture
+        calls to this iterator.
+    """
+    def __init__(self, wrapped, call_capture_callback):
+        self.call_capture_callback = call_capture_callback
+        self.wrapped_iter = wrapped
+
+    def _capture_call(self):
+        # call home to capture the call
+        self.call_capture_callback(inspect.stack()[1][3])
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        self._capture_call()
+        return next(self.wrapped_iter)
+
+    def __del__(self):
+        self._capture_call()
+
+    def close(self):
+        self._capture_call()
+        close_if_possible(self.wrapped_iter)
+
+
+class CaptureIteratorFactory(object):
+    """
+    Create instances of ``CaptureIterator`` to wrap a given iterable, and
+    provides a callback function for the ``CaptureIterator`` to capture its
+    calls.
+
+    :param wrapped: an iterable to wrap.
+    """
+    def __init__(self, wrapped):
+        self.wrapped = wrapped
+        self.instance_count = 0
+        self.captured_calls = defaultdict(list)
+
+    def log_call(self, instance_number, call):
+        self.captured_calls[instance_number].append(call)
+
+    def __call__(self, *args, **kwargs):
+        # note: do not keep a reference to the CaptureIterator because that
+        # would prevent it being garbage collected
+        self.instance_count += 1
+        return CaptureIterator(
+            self.wrapped(*args, **kwargs),
+            functools.partial(self.log_call, self.instance_count))
+
+
+def get_node_error_stats(proxy_app, ring_node):
+    node_key = proxy_app.error_limiter.node_key(ring_node)
+    return proxy_app.error_limiter.stats.get(node_key) or {}
+
+
+def node_error_count(proxy_app, ring_node):
+    # Reach into the proxy's internals to get the error count for a
+    # particular node
+    return get_node_error_stats(proxy_app, ring_node).get('errors', 0)
+
+
+def node_error_counts(proxy_app, ring_nodes):
+    # Reach into the proxy's internals to get the error counts for a
+    # list of nodes
+    return sorted([get_node_error_stats(proxy_app, node).get('errors', 0)
+                   for node in ring_nodes], reverse=True)
+
+
+def node_last_error(proxy_app, ring_node):
+    # Reach into the proxy's internals to get the last error for a
+    # particular node
+    return get_node_error_stats(proxy_app, ring_node).get('last_error')
+
+
+def set_node_errors(proxy_app, ring_node, value, last_error):
+    # Set the node's error count to value
+    node_key = proxy_app.error_limiter.node_key(ring_node)
+    stats = {'errors': value,
+             'last_error': last_error}
+    proxy_app.error_limiter.stats[node_key] = stats
