@@ -53,14 +53,15 @@ class TestAuditorRealBrokerMigration(
         self.assertUnmigrated(self.broker)
 
         # run auditor, and validate migration
-        conf = {'devices': self.tempdir, 'mount_check': False,
-                'recon_cache_path': self.tempdir}
+        conf = {'devices': self.testdir, 'mount_check': False,
+                'recon_cache_path': self.testdir}
         test_auditor = auditor.AccountAuditor(conf, logger=debug_logger())
         test_auditor.run_once()
 
         self.restore_account_broker()
 
-        broker = auditor.AccountBroker(self.db_path)
+        broker = auditor.AccountBroker(self.db_path, account='a')
+        broker.initialize(Timestamp('1').internal, 0)
         # go after rows directly to avoid unintentional migration
         with broker.get() as conn:
             rows = conn.execute('''
@@ -76,6 +77,9 @@ class TestAuditorRealBroker(unittest.TestCase):
 
     def setUp(self):
         self.logger = debug_logger()
+        # really, this would come by way of base_prefix/tail_prefix in
+        # get_logger, ultimately tracing back to section_name in daemon...
+        self.logger.logger.statsd_client._prefix = 'account-auditor.'
 
     @with_tempdir
     def test_db_validate_fails(self, tempdir):
@@ -125,10 +129,16 @@ class TestAuditorRealBroker(unittest.TestCase):
         self.assertEqual(len(error_lines), 1)
         error_message = error_lines[0]
         self.assertIn(broker.db_file, error_message)
-        self.assertIn('container_count', error_message)
-        self.assertIn('does not match', error_message)
-        self.assertEqual(test_auditor.logger.get_increment_counts(),
-                         {'failures': 1})
+        self.assertIn(
+            'The total container_count for the account a (%d) does not match '
+            'the sum of container_count across policies (%d)'
+            % (num_containers, num_containers - 1), error_message)
+        self.assertEqual(
+            test_auditor.logger.statsd_client.get_increment_counts(),
+            {'failures': 1})
+        self.assertIn(
+            (b'account-auditor.failures:1|c', ('host', 8125)),
+            test_auditor.logger.statsd_client.sendto_calls)
 
 
 if __name__ == '__main__':
