@@ -17,23 +17,18 @@
 
 from collections import defaultdict
 import json
-import unittest
 import pickle
 import os
 from time import sleep, time
 from uuid import uuid4
-from tempfile import mkdtemp
-from shutil import rmtree
 import sqlite3
 import itertools
 from contextlib import contextmanager
 import random
-import mock
+from unittest import mock
 import base64
 import shutil
 
-import six
-
 from swift.account.backend import AccountBroker
 from swift.common.utils import Timestamp
 from test.unit import patch_policies, with_tempdir, make_timestamp_iter
@@ -46,23 +41,25 @@ from test.unit.common import test_db
 
 
 @patch_policies
-class TestAccountBroker(unittest.TestCase):
+class TestAccountBroker(test_db.TestDbBase):
     """Tests for AccountBroker"""
 
     def setUp(self):
+        super(TestAccountBroker, self).setUp()
         # tests seem to assume x-timestamp was set by the proxy before "now"
         self.ts = make_timestamp_iter(offset=-1)
 
     def test_creation(self):
         # Test AccountBroker.__init__
-        broker = AccountBroker(':memory:', account='a')
-        self.assertEqual(broker.db_file, ':memory:')
+        db_file = self.get_db_path()
+        broker = AccountBroker(db_file, account='a')
+        self.assertEqual(broker.db_file, db_file)
         try:
             with broker.get() as conn:
                 pass
         except DatabaseConnectionError as e:
             self.assertTrue(hasattr(e, 'path'))
-            self.assertEqual(e.path, ':memory:')
+            self.assertEqual(e.path, db_file)
             self.assertTrue(hasattr(e, 'msg'))
             self.assertEqual(e.msg, "DB doesn't exist")
         except Exception as e:
@@ -76,7 +73,7 @@ class TestAccountBroker(unittest.TestCase):
             self.assertEqual(curs.fetchall()[0][0], 1)
 
     def test_initialize_fail(self):
-        broker = AccountBroker(':memory:')
+        broker = AccountBroker(self.get_db_path())
         with self.assertRaises(ValueError) as cm:
             broker.initialize(Timestamp('1').internal)
         self.assertEqual(str(cm.exception), 'Attempting to create a new'
@@ -85,7 +82,7 @@ class TestAccountBroker(unittest.TestCase):
     def test_exception(self):
         # Test AccountBroker throwing a conn away after exception
         first_conn = None
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         with broker.get() as conn:
             first_conn = conn
@@ -99,7 +96,7 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_empty(self):
         # Test AccountBroker.empty
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         self.assertTrue(broker.empty())
         broker.put_container('o', Timestamp.now().internal, 0, 0, 0,
@@ -112,12 +109,12 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_is_status_deleted(self):
         # Test AccountBroker.is_status_deleted
-        broker1 = AccountBroker(':memory:', account='a')
+        broker1 = AccountBroker(self.get_db_path(), account='a')
         broker1.initialize(Timestamp.now().internal)
         self.assertFalse(broker1.is_status_deleted())
         broker1.delete_db(Timestamp.now().internal)
         self.assertTrue(broker1.is_status_deleted())
-        broker2 = AccountBroker(':memory:', account='a')
+        broker2 = AccountBroker(self.get_db_path(), account='a')
         broker2.initialize(Timestamp.now().internal)
         # Set delete_timestamp greater than put_timestamp
         broker2.merge_timestamps(
@@ -126,10 +123,12 @@ class TestAccountBroker(unittest.TestCase):
         self.assertTrue(broker2.is_status_deleted())
 
     def test_reclaim(self):
-        broker = AccountBroker(':memory:', account='test_account')
+        broker = AccountBroker(self.get_db_path(), account='test_account')
         broker.initialize(Timestamp('1').internal)
         broker.put_container('c', Timestamp.now().internal, 0, 0, 0,
                              POLICIES.default.idx)
+        # commit pending file into db
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT count(*) FROM container "
@@ -148,6 +147,7 @@ class TestAccountBroker(unittest.TestCase):
         sleep(.00001)
         broker.put_container('c', 0, Timestamp.now().internal, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT count(*) FROM container "
@@ -176,6 +176,7 @@ class TestAccountBroker(unittest.TestCase):
         broker.put_container('x', 0, 0, 0, 0, POLICIES.default.idx)
         broker.put_container('y', 0, 0, 0, 0, POLICIES.default.idx)
         broker.put_container('z', 0, 0, 0, 0, POLICIES.default.idx)
+        broker._commit_puts()
         broker.reclaim(Timestamp.now().internal, time())
         # Now delete the account
         broker.delete_db(Timestamp.now().internal)
@@ -187,13 +188,13 @@ class TestAccountBroker(unittest.TestCase):
         now = time()
         top_of_the_minute = now - (now % 60)
         c = itertools.cycle([True, False])
-        for m, is_deleted in six.moves.zip(range(num_of_containers), c):
+        for m, is_deleted in zip(range(num_of_containers), c):
             offset = top_of_the_minute - (m * 60)
             container_specs.append((Timestamp(offset), is_deleted))
         random.seed(now)
         random.shuffle(container_specs)
         policy_indexes = list(p.idx for p in POLICIES)
-        broker = AccountBroker(':memory:', account='test_account')
+        broker = AccountBroker(self.get_db_path(), account='test_account')
         broker.initialize(Timestamp('1').internal)
         for i, container_spec in enumerate(container_specs):
             # with container12 before container2 and shuffled ts.internal we
@@ -206,6 +207,9 @@ class TestAccountBroker(unittest.TestCase):
             else:
                 broker.put_container(name, ts.internal, 0, 0, 0, pidx)
 
+        # commit pending file into db
+        broker._commit_puts()
+
         def count_reclaimable(conn, reclaim_age):
             return conn.execute(
                 "SELECT count(*) FROM container "
@@ -251,7 +255,7 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_delete_db_status(self):
         start = next(self.ts)
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(start.internal)
         info = broker.get_info()
         self.assertEqual(info['put_timestamp'], start.internal)
@@ -273,10 +277,12 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_delete_container(self):
         # Test AccountBroker.delete_container
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         broker.put_container('o', Timestamp.now().internal, 0, 0, 0,
                              POLICIES.default.idx)
+        # commit pending file into db
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT count(*) FROM container "
@@ -287,6 +293,7 @@ class TestAccountBroker(unittest.TestCase):
         sleep(.00001)
         broker.put_container('o', 0, Timestamp.now().internal, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT count(*) FROM container "
@@ -297,13 +304,15 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_put_container(self):
         # Test AccountBroker.put_container
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
 
         # Create initial container
         timestamp = Timestamp.now().internal
         broker.put_container('"{<container \'&\' name>}"', timestamp, 0, 0, 0,
                              POLICIES.default.idx)
+        # commit pending file into db
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -317,6 +326,7 @@ class TestAccountBroker(unittest.TestCase):
         # Reput same event
         broker.put_container('"{<container \'&\' name>}"', timestamp, 0, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -332,6 +342,7 @@ class TestAccountBroker(unittest.TestCase):
         timestamp = Timestamp.now().internal
         broker.put_container('"{<container \'&\' name>}"', timestamp, 0, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -346,6 +357,7 @@ class TestAccountBroker(unittest.TestCase):
         otimestamp = Timestamp(float(Timestamp(timestamp)) - 1).internal
         broker.put_container('"{<container \'&\' name>}"', otimestamp, 0, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -360,6 +372,7 @@ class TestAccountBroker(unittest.TestCase):
         dtimestamp = Timestamp(float(Timestamp(timestamp)) - 1).internal
         broker.put_container('"{<container \'&\' name>}"', 0, dtimestamp, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -378,6 +391,7 @@ class TestAccountBroker(unittest.TestCase):
         timestamp = Timestamp.now().internal
         broker.put_container('"{<container \'&\' name>}"', 0, timestamp, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -393,6 +407,7 @@ class TestAccountBroker(unittest.TestCase):
         timestamp = Timestamp.now().internal
         broker.put_container('"{<container \'&\' name>}"', timestamp, 0, 0, 0,
                              POLICIES.default.idx)
+        broker._commit_puts()
         with broker.get() as conn:
             self.assertEqual(conn.execute(
                 "SELECT name FROM container").fetchone()[0],
@@ -405,7 +420,7 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_get_info(self):
         # Test AccountBroker.get_info
-        broker = AccountBroker(':memory:', account='test1')
+        broker = AccountBroker(self.get_db_path(), account='test1')
         broker.initialize(Timestamp('1').internal)
 
         info = broker.get_info()
@@ -452,7 +467,7 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_list_containers_iter(self):
         # Test AccountBroker.list_containers_iter
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         for cont1 in range(4):
             for cont2 in range(125):
@@ -582,7 +597,7 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_list_objects_iter_order_and_reverse(self):
         # Test ContainerBroker.list_objects_iter
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal, 0)
 
         broker.put_container(
@@ -617,7 +632,7 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_list_container_iter_with_reserved_name(self):
         # Test ContainerBroker.list_objects_iter
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(next(self.ts).internal, 0)
 
         broker.put_container(
@@ -754,11 +769,13 @@ class TestAccountBroker(unittest.TestCase):
         }
         failures = []
         for expected in expectations:
-            broker = AccountBroker(':memory:', account='a')
+            broker = AccountBroker(self.get_db_path(), account='a')
             broker.initialize(next(self.ts).internal, 0)
             for name in expected['containers']:
                 broker.put_container(name, next(self.ts).internal, 0, 0, 0,
                                      POLICIES.default.idx)
+            # commit pending file into db
+            broker._commit_puts()
             params = default_listing_params.copy()
             params.update(expected['params'])
             listing = list(c[0] for c in broker.list_containers_iter(**params))
@@ -773,7 +790,7 @@ class TestAccountBroker(unittest.TestCase):
     def test_double_check_trailing_delimiter(self):
         # Test AccountBroker.list_containers_iter for an
         # account that has an odd container with a trailing delimiter
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         broker.put_container('a', Timestamp.now().internal, 0, 0, 0,
                              POLICIES.default.idx)
@@ -814,7 +831,7 @@ class TestAccountBroker(unittest.TestCase):
         self.assertEqual([row[0] for row in listing], ['b-a', 'b-b'])
 
     def test_chexor(self):
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         broker.put_container('a', Timestamp(1).internal,
                              Timestamp(0).internal, 0, 0,
@@ -828,7 +845,7 @@ class TestAccountBroker(unittest.TestCase):
         text = '%s-%s' % ('b', "%s-%s-%s-%s" % (
                Timestamp(2).internal, Timestamp(0).internal, 0, 0))
         hashb = md5(text.encode('ascii'), usedforsecurity=False).digest()
-        hashc = ''.join(('%02x' % (ord(a) ^ ord(b) if six.PY2 else a ^ b)
+        hashc = ''.join(('%02x' % (a ^ b)
                          for a, b in zip(hasha, hashb)))
         self.assertEqual(broker.get_info()['hash'], hashc)
         broker.put_container('b', Timestamp(3).internal,
@@ -837,14 +854,14 @@ class TestAccountBroker(unittest.TestCase):
         text = '%s-%s' % ('b', "%s-%s-%s-%s" % (
                Timestamp(3).internal, Timestamp(0).internal, 0, 0))
         hashb = md5(text.encode('ascii'), usedforsecurity=False).digest()
-        hashc = ''.join(('%02x' % (ord(a) ^ ord(b) if six.PY2 else a ^ b)
+        hashc = ''.join(('%02x' % (a ^ b)
                          for a, b in zip(hasha, hashb)))
         self.assertEqual(broker.get_info()['hash'], hashc)
 
     def test_merge_items(self):
-        broker1 = AccountBroker(':memory:', account='a')
+        broker1 = AccountBroker(self.get_db_path(), account='a')
         broker1.initialize(Timestamp('1').internal)
-        broker2 = AccountBroker(':memory:', account='a')
+        broker2 = AccountBroker(self.get_db_path(), account='a')
         broker2.initialize(Timestamp('1').internal)
         broker1.put_container('a', Timestamp(1).internal, 0, 0, 0,
                               POLICIES.default.idx)
@@ -867,12 +884,10 @@ class TestAccountBroker(unittest.TestCase):
 
     def test_merge_items_overwrite_unicode(self):
         snowman = u'\N{SNOWMAN}'
-        if six.PY2:
-            snowman = snowman.encode('utf-8')
-        broker1 = AccountBroker(':memory:', account='a')
+        broker1 = AccountBroker(self.get_db_path(), account='a')
         broker1.initialize(Timestamp('1').internal, 0)
         id1 = broker1.get_info()['id']
-        broker2 = AccountBroker(':memory:', account='a')
+        broker2 = AccountBroker(self.get_db_path(), account='a')
         broker2.initialize(Timestamp('1').internal, 0)
         broker1.put_container(snowman, Timestamp(2).internal, 0, 1, 100,
                               POLICIES.default.idx)
@@ -974,7 +989,7 @@ class TestAccountBroker(unittest.TestCase):
                      StoragePolicy(2, 'two', False),
                      StoragePolicy(3, 'three', False)])
     def test_get_policy_stats(self):
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(next(self.ts).internal)
         # check empty policy_stats
         self.assertTrue(broker.empty())
@@ -1041,7 +1056,7 @@ class TestAccountBroker(unittest.TestCase):
     @patch_policies([StoragePolicy(0, 'zero', False),
                      StoragePolicy(1, 'one', True)])
     def test_policy_stats_tracking(self):
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(next(self.ts).internal)
 
         # policy 0
@@ -1162,6 +1177,7 @@ def premetadata_create_account_stat_table(self, conn, put_timestamp):
 class TestCommonAccountBroker(test_db.TestExampleBroker):
 
     broker_class = AccountBroker
+    server_type = 'account'
 
     def setUp(self):
         super(TestCommonAccountBroker, self).setUp()
@@ -1183,13 +1199,14 @@ class TestAccountBrokerBeforeMetadata(TestAccountBroker):
     """
 
     def setUp(self):
+        super(TestAccountBroker, self).setUp()
         # tests seem to assume x-timestamp was set by the proxy before "now"
         self.ts = make_timestamp_iter(offset=-1)
         self._imported_create_account_stat_table = \
             AccountBroker.create_account_stat_table
         AccountBroker.create_account_stat_table = \
             premetadata_create_account_stat_table
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         exc = None
         with broker.get() as conn:
@@ -1202,10 +1219,11 @@ class TestAccountBrokerBeforeMetadata(TestAccountBroker):
     def tearDown(self):
         AccountBroker.create_account_stat_table = \
             self._imported_create_account_stat_table
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         with broker.get() as conn:
             conn.execute('SELECT metadata FROM account_stat')
+        super(TestAccountBrokerBeforeMetadata, self).tearDown()
 
 
 def prespi_create_container_table(self, conn):
@@ -1270,6 +1288,7 @@ class TestAccountBrokerBeforeSPI(TestAccountBroker):
     """
 
     def setUp(self):
+        super(TestAccountBrokerBeforeSPI, self).setUp()
         # tests seem to assume x-timestamp was set by the proxy before "now"
         self.ts = make_timestamp_iter(offset=-1)
         self._imported_create_container_table = \
@@ -1278,7 +1297,7 @@ class TestAccountBrokerBeforeSPI(TestAccountBroker):
             prespi_create_container_table
         self._imported_initialize = AccountBroker._initialize
         AccountBroker._initialize = prespi_AccountBroker_initialize
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         exc = None
         with broker.get() as conn:
@@ -1299,10 +1318,11 @@ class TestAccountBrokerBeforeSPI(TestAccountBroker):
         AccountBroker.create_container_table = \
             self._imported_create_container_table
         AccountBroker._initialize = self._imported_initialize
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         with broker.get() as conn:
             conn.execute('SELECT storage_policy_index FROM container')
+        super(TestAccountBrokerBeforeSPI, self).tearDown()
 
     @with_tempdir
     def test_policy_table_migration(self, tempdir):
@@ -1428,7 +1448,7 @@ class TestAccountBrokerBeforeSPI(TestAccountBroker):
 
         # make sure we can iter containers without the migration
         for c in broker.list_containers_iter(1, None, None, None, None):
-            self.assertEqual(c, ('test_name', 1, 2, timestamp, 0))
+            self.assertEqual(c, ('test_name', 1, 2, timestamp, 0, 0))
 
         # stats table is mysteriously empty...
         stats = broker.get_policy_stats()
@@ -1497,23 +1517,37 @@ class TestAccountBrokerBeforeSPI(TestAccountBroker):
         real_get = broker.get
         called = []
 
+        class ExpectedError(Exception):
+            '''Expected error to be raised during the test'''
+
         @contextmanager
         def mock_get():
             with real_get() as conn:
 
-                def mock_executescript(script):
-                    if called:
-                        raise Exception('kaboom!')
-                    called.append(script)
+                class MockConn(object):
+                    def __init__(self, real_conn):
+                        self.real_conn = real_conn
 
-                conn.executescript = mock_executescript
-                yield conn
+                    @property
+                    def cursor(self):
+                        return self.real_conn.cursor
+
+                    @property
+                    def execute(self):
+                        return self.real_conn.execute
+
+                    def executescript(self, script):
+                        if called:
+                            raise ExpectedError('kaboom!')
+                        called.append(script)
+
+                yield MockConn(conn)
 
         broker.get = mock_get
 
         try:
             broker._commit_puts()
-        except Exception:
+        except ExpectedError:
             pass
         else:
             self.fail('mock exception was not raised')
@@ -1573,7 +1607,7 @@ class TestAccountBrokerBeforeSPI(TestAccountBroker):
         # make sure "test_name" container in new database
         self.assertEqual(new_broker.get_info()['container_count'], 1)
         for c in new_broker.list_containers_iter(1, None, None, None, None):
-            self.assertEqual(c, ('test_name', 1, 2, timestamp, 0))
+            self.assertEqual(c, ('test_name', 1, 2, timestamp, 0, 0))
 
         # full migration successful
         with new_broker.get() as conn:
@@ -1681,7 +1715,7 @@ def pre_track_containers_create_container_table(self, conn):
     """ + OLD_POLICY_STAT_TRIGGER_SCRIPT)
 
 
-class AccountBrokerPreTrackContainerCountSetup(object):
+class AccountBrokerPreTrackContainerCountSetup(test_db.TestDbBase):
     def assertUnmigrated(self, broker):
         with broker.get() as conn:
             try:
@@ -1696,6 +1730,7 @@ class AccountBrokerPreTrackContainerCountSetup(object):
                           'trying to select container_count from policy_stat!')
 
     def setUp(self):
+        super(AccountBrokerPreTrackContainerCountSetup, self).setUp()
         # use old version of policy_stat
         self._imported_create_policy_stat_table = \
             AccountBroker.create_policy_stat_table
@@ -1708,17 +1743,16 @@ class AccountBrokerPreTrackContainerCountSetup(object):
         AccountBroker.create_container_table = \
             pre_track_containers_create_container_table
 
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         self.assertUnmigrated(broker)
 
-        self.tempdir = mkdtemp()
         # tests seem to assume x-timestamp was set by the proxy before "now"
         self.ts = make_timestamp_iter(offset=-1)
 
-        self.db_path = os.path.join(self.tempdir, 'sda', 'accounts',
+        self.db_path = os.path.join(self.testdir, 'sda', 'accounts',
                                     '0', '0', '0', 'test.db')
-        self.broker = AccountBroker(self.db_path, account='a')
+        self.broker = AccountBroker(self.get_db_path(), account='a')
         self.broker.initialize(next(self.ts).internal)
 
         # Common sanity-check that our starting, pre-migration state correctly
@@ -1726,14 +1760,13 @@ class AccountBrokerPreTrackContainerCountSetup(object):
         self.assertUnmigrated(self.broker)
 
     def tearDown(self):
-        rmtree(self.tempdir, ignore_errors=True)
-
         self.restore_account_broker()
 
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.get_db_path(), account='a')
         broker.initialize(Timestamp('1').internal)
         with broker.get() as conn:
             conn.execute('SELECT container_count FROM policy_stat')
+        super(AccountBrokerPreTrackContainerCountSetup, self).tearDown()
 
     def restore_account_broker(self):
         AccountBroker.create_policy_stat_table = \
@@ -1925,7 +1958,7 @@ class TestAccountBrokerBeforePerPolicyContainerTrack(
             self.assertEqual(policy_stat['container_count'], 1)
 
     def test_migrate_add_storage_policy_index_fail(self):
-        broker = AccountBroker(':memory:', account='a')
+        broker = AccountBroker(self.db_path, account='a')
         broker.initialize(Timestamp('1').internal)
         with mock.patch.object(
                 broker, 'create_policy_stat_table',
