@@ -26,10 +26,11 @@ from swift.common.middleware.crypto import keymaster
 from swift.common.middleware.crypto.crypto_utils import (
     load_crypto_meta, Crypto)
 from swift.common.ring import Ring
-from swift.common.swob import Request
+from swift.common.swob import Request, str_to_wsgi
 from swift.obj import diskfile
 
-from test.unit import FakeLogger
+from test.debug_logger import debug_logger
+from test.unit import skip_if_no_xattrs
 from test.unit.common.middleware.crypto.crypto_helpers import (
     md5hex, encrypt, TEST_KEYMASTER_CONF)
 from test.unit.helpers import setup_servers, teardown_servers
@@ -54,19 +55,25 @@ class TestCryptoPipelineChanges(unittest.TestCase):
             cls._test_context = None
 
     def setUp(self):
-        self.plaintext = 'unencrypted body content'
+        skip_if_no_xattrs()
+        self.plaintext = b'unencrypted body content'
         self.plaintext_etag = md5hex(self.plaintext)
         self._setup_crypto_app()
 
-    def _setup_crypto_app(self, disable_encryption=False):
+    def _setup_crypto_app(self, disable_encryption=False, root_secret_id=None):
         # Set up a pipeline of crypto middleware ending in the proxy app so
         # that tests can make requests to either the proxy server directly or
         # via the crypto middleware. Make a fresh instance for each test to
         # avoid any state coupling.
         conf = {'disable_encryption': disable_encryption}
         self.encryption = crypto.filter_factory(conf)(self.proxy_app)
-        self.km = keymaster.KeyMaster(self.encryption, TEST_KEYMASTER_CONF)
+        self.encryption.logger = self.proxy_app.logger
+        km_conf = dict(TEST_KEYMASTER_CONF)
+        if root_secret_id is not None:
+            km_conf['active_root_secret_id'] = root_secret_id
+        self.km = keymaster.KeyMaster(self.encryption, km_conf)
         self.crypto_app = self.km  # for clarity
+        self.crypto_app.logger = self.encryption.logger
 
     def _create_container(self, app, policy_name='one', container_path=None):
         if not container_path:
@@ -77,27 +84,28 @@ class TestCryptoPipelineChanges(unittest.TestCase):
             self.object_path = self.container_path + '/' + self.object_name
             container_path = self.container_path
         req = Request.blank(
-            container_path, method='PUT',
+            str_to_wsgi(container_path), method='PUT',
             headers={'X-Storage-Policy': policy_name})
         resp = req.get_response(app)
         self.assertEqual('201 Created', resp.status)
         # sanity check
         req = Request.blank(
-            container_path, method='HEAD',
+            str_to_wsgi(container_path), method='HEAD',
             headers={'X-Storage-Policy': policy_name})
         resp = req.get_response(app)
         self.assertEqual(policy_name, resp.headers['X-Storage-Policy'])
 
     def _put_object(self, app, body):
-        req = Request.blank(self.object_path, method='PUT', body=body,
-                            headers={'Content-Type': 'application/test'})
+        req = Request.blank(
+            str_to_wsgi(self.object_path), method='PUT', body=body,
+            headers={'Content-Type': 'application/test'})
         resp = req.get_response(app)
         self.assertEqual('201 Created', resp.status)
         self.assertEqual(self.plaintext_etag, resp.headers['Etag'])
         return resp
 
     def _post_object(self, app):
-        req = Request.blank(self.object_path, method='POST',
+        req = Request.blank(str_to_wsgi(self.object_path), method='POST',
                             headers={'Content-Type': 'application/test',
                                      'X-Object-Meta-Fruit': 'Kiwi'})
         resp = req.get_response(app)
@@ -105,7 +113,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         return resp
 
     def _copy_object(self, app, destination):
-        req = Request.blank(self.object_path, method='COPY',
+        req = Request.blank(str_to_wsgi(self.object_path), method='COPY',
                             headers={'Destination': destination})
         resp = req.get_response(app)
         self.assertEqual('201 Created', resp.status)
@@ -113,7 +121,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         return resp
 
     def _check_GET_and_HEAD(self, app, object_path=None):
-        object_path = object_path or self.object_path
+        object_path = str_to_wsgi(object_path or self.object_path)
         req = Request.blank(object_path, method='GET')
         resp = req.get_response(app)
         self.assertEqual('200 OK', resp.status)
@@ -123,13 +131,13 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         req = Request.blank(object_path, method='HEAD')
         resp = req.get_response(app)
         self.assertEqual('200 OK', resp.status)
-        self.assertEqual('', resp.body)
+        self.assertEqual(b'', resp.body)
         self.assertEqual('Kiwi', resp.headers['X-Object-Meta-Fruit'])
 
     def _check_match_requests(self, method, app, object_path=None):
-        object_path = object_path or self.object_path
+        object_path = str_to_wsgi(object_path or self.object_path)
         # verify conditional match requests
-        expected_body = self.plaintext if method == 'GET' else ''
+        expected_body = self.plaintext if method == 'GET' else b''
 
         # If-Match matches
         req = Request.blank(object_path, method=method,
@@ -154,7 +162,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                             headers={'If-Match': '"not the etag"'})
         resp = req.get_response(app)
         self.assertEqual('412 Precondition Failed', resp.status)
-        self.assertEqual('', resp.body)
+        self.assertEqual(b'', resp.body)
         self.assertEqual(self.plaintext_etag, resp.headers['Etag'])
 
         # If-None-Match matches
@@ -163,7 +171,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
             headers={'If-None-Match': '"%s"' % self.plaintext_etag})
         resp = req.get_response(app)
         self.assertEqual('304 Not Modified', resp.status)
-        self.assertEqual('', resp.body)
+        self.assertEqual(b'', resp.body)
         self.assertEqual(self.plaintext_etag, resp.headers['Etag'])
 
         # If-None-Match wildcard
@@ -171,7 +179,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                             headers={'If-None-Match': '*'})
         resp = req.get_response(app)
         self.assertEqual('304 Not Modified', resp.status)
-        self.assertEqual('', resp.body)
+        self.assertEqual(b'', resp.body)
         self.assertEqual(self.plaintext_etag, resp.headers['Etag'])
 
         # If-None-Match does not match
@@ -184,7 +192,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         self.assertEqual('Kiwi', resp.headers['X-Object-Meta-Fruit'])
 
     def _check_listing(self, app, expect_mismatch=False, container_path=None):
-        container_path = container_path or self.container_path
+        container_path = str_to_wsgi(container_path or self.container_path)
         req = Request.blank(
             container_path, method='GET', query_string='format=json')
         resp = req.get_response(app)
@@ -261,6 +269,36 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         self._check_match_requests('HEAD', self.crypto_app)
         self._check_listing(self.crypto_app)
 
+    def test_write_with_crypto_read_with_crypto_different_root_secrets(self):
+        root_secret = self.crypto_app.root_secret
+        self._create_container(self.proxy_app, policy_name='one')
+        self._put_object(self.crypto_app, self.plaintext)
+        # change root secret
+        self._setup_crypto_app(root_secret_id='1')
+        root_secret_1 = self.crypto_app.root_secret
+        self.assertNotEqual(root_secret, root_secret_1)  # sanity check
+        self._post_object(self.crypto_app)
+        self._check_GET_and_HEAD(self.crypto_app)
+        self._check_match_requests('GET', self.crypto_app)
+        self._check_match_requests('HEAD', self.crypto_app)
+        self._check_listing(self.crypto_app)
+        # change root secret
+        self._setup_crypto_app(root_secret_id='2')
+        root_secret_2 = self.crypto_app.root_secret
+        self.assertNotEqual(root_secret_2, root_secret_1)  # sanity check
+        self.assertNotEqual(root_secret_2, root_secret)  # sanity check
+        self._check_GET_and_HEAD(self.crypto_app)
+        self._check_match_requests('GET', self.crypto_app)
+        self._check_match_requests('HEAD', self.crypto_app)
+        self._check_listing(self.crypto_app)
+        # write object again
+        self._put_object(self.crypto_app, self.plaintext)
+        self._post_object(self.crypto_app)
+        self._check_GET_and_HEAD(self.crypto_app)
+        self._check_match_requests('GET', self.crypto_app)
+        self._check_match_requests('HEAD', self.crypto_app)
+        self._check_listing(self.crypto_app)
+
     def test_write_with_crypto_read_with_crypto_ec(self):
         self._create_container(self.proxy_app, policy_name='ec')
         self._put_object(self.crypto_app, self.plaintext)
@@ -315,7 +353,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         req = Request.blank(self.object_path, method='HEAD')
         resp = req.get_response(app)
         self.assertEqual('200 OK', resp.status)
-        self.assertEqual('', resp.body)
+        self.assertEqual(b'', resp.body)
         self.assertNotEqual('Kiwi', resp.headers['X-Object-Meta-Fruit'])
 
     def test_write_with_crypto_read_without_crypto(self):
@@ -391,9 +429,9 @@ class TestCryptoPipelineChanges(unittest.TestCase):
             crypto_meta = crypto_meta_param[len('swift_meta='):]
             listing_etag_iv = load_crypto_meta(crypto_meta)['iv']
             exp_enc_listing_etag = base64.b64encode(
-                encrypt(self.plaintext_etag,
+                encrypt(self.plaintext_etag.encode('ascii'),
                         self.km.create_key('/a/%s' % self.container_name),
-                        listing_etag_iv))
+                        listing_etag_iv)).decode('ascii')
             self.assertEqual(exp_enc_listing_etag, parts[0])
 
         # Verify diskfile data and metadata is encrypted
@@ -401,7 +439,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         partition, nodes = ring_object.get_nodes('a', self.container_name, 'o')
         conf = {'devices': self._test_context["testdir"],
                 'mount_check': 'false'}
-        df_mgr = diskfile.DiskFileRouter(conf, FakeLogger())[policy]
+        df_mgr = diskfile.DiskFileRouter(conf, debug_logger())[policy]
         ondisk_data = []
         exp_enc_body = None
         for node_index, node in enumerate(nodes):
@@ -410,7 +448,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                                      policy=policy)
             with df.open():
                 meta = df.get_metadata()
-                contents = ''.join(df.reader())
+                contents = b''.join(df.reader())
                 metadata = dict((k.lower(), v) for k, v in meta.items())
                 # verify on disk data - body
                 body_iv = load_crypto_meta(
@@ -427,8 +465,8 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                     'x-object-transient-sysmeta-crypto-meta-fruit'].split(';')
                 meta = meta.strip()[len('swift_meta='):]
                 metadata_iv = load_crypto_meta(meta)['iv']
-                exp_enc_meta = base64.b64encode(encrypt('Kiwi', obj_key,
-                                                        metadata_iv))
+                exp_enc_meta = base64.b64encode(encrypt(
+                    b'Kiwi', obj_key, metadata_iv)).decode('ascii')
                 self.assertEqual(exp_enc_meta, enc_val)
                 self.assertNotIn('x-object-meta-fruit', metadata)
 
@@ -442,7 +480,7 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                     '/a/%s/%s' % (self.container_name, self.object_name),
                     meta['key_id']['path'])
                 self.assertIn('v', meta['key_id'])
-                self.assertEqual('1', meta['key_id']['v'])
+                self.assertEqual('2', meta['key_id']['v'])
                 self.assertIn('cipher', meta)
                 self.assertEqual(Crypto.cipher, meta['cipher'])
 
@@ -450,14 +488,16 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                 actual_enc_etag, _junk, actual_etag_meta = metadata[
                     'x-object-sysmeta-crypto-etag'].partition('; swift_meta=')
                 etag_iv = load_crypto_meta(actual_etag_meta)['iv']
-                exp_enc_etag = base64.b64encode(encrypt(self.plaintext_etag,
-                                                        obj_key, etag_iv))
+                exp_enc_etag = base64.b64encode(encrypt(
+                    self.plaintext_etag.encode('ascii'),
+                    obj_key, etag_iv)).decode('ascii')
                 self.assertEqual(exp_enc_etag, actual_enc_etag)
 
                 # verify etag hmac
                 exp_etag_mac = hmac.new(
-                    obj_key, self.plaintext_etag, digestmod=hashlib.sha256)
-                exp_etag_mac = base64.b64encode(exp_etag_mac.digest())
+                    obj_key, self.plaintext_etag.encode('ascii'),
+                    digestmod=hashlib.sha256).digest()
+                exp_etag_mac = base64.b64encode(exp_etag_mac).decode('ascii')
                 self.assertEqual(exp_etag_mac,
                                  metadata['x-object-sysmeta-crypto-etag-mac'])
 
@@ -469,8 +509,8 @@ class TestCryptoPipelineChanges(unittest.TestCase):
                 listing_etag_iv = load_crypto_meta(crypto_meta)['iv']
                 cont_key = self.km.create_key('/a/%s' % self.container_name)
                 exp_enc_listing_etag = base64.b64encode(
-                    encrypt(self.plaintext_etag, cont_key,
-                            listing_etag_iv))
+                    encrypt(self.plaintext_etag.encode('ascii'), cont_key,
+                            listing_etag_iv)).decode('ascii')
                 self.assertEqual(exp_enc_listing_etag, parts[0])
 
         self._check_GET_and_HEAD(self.crypto_app)
@@ -618,14 +658,5 @@ class TestCryptoPipelineChanges(unittest.TestCase):
         self._check_listing(self.crypto_app)
 
 
-class TestCryptoPipelineChangesFastPost(TestCryptoPipelineChanges):
-    @classmethod
-    def setUpClass(cls):
-        # set proxy config to use fast post
-        extra_conf = {'object_post_as_copy': 'False'}
-        cls._test_context = setup_servers(extra_conf=extra_conf)
-        cls.proxy_app = cls._test_context["test_servers"][0]
-
-
 if __name__ == '__main__':
     unittest.main()
