@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import base64
-import mock
+from unittest import mock
 import unittest
 import sys
 sys.modules['castellan'] = mock.Mock()
@@ -89,6 +89,7 @@ def mock_keystone_password_side_effect(auth_url, username, password,
                         user_domain_id, trust_id, domain_id, domain_name,
                         project_id, project_domain_id, reauthenticate)
 
+
 ERR_MESSAGE_SECRET_INCORRECTLY_SPECIFIED = 'Secret incorrectly specified.'
 ERR_MESSAGE_KEY_UUID_NOT_FOUND = 'Key not found, uuid: '
 
@@ -129,7 +130,11 @@ class MockBarbicanKeyManager(object):
             raise ValueError(ERR_MESSAGE_SECRET_INCORRECTLY_SPECIFIED)
         elif key_id == TEST_KMS_NONE_KEY_ID:
             return None
-        return MockBarbicanKey(b'x' * 32, key_id)
+        if 'unicode' in key_id:
+            key_str = key_id[0] * 32
+        else:
+            key_str = (str(key_id[0]) * 32).encode('utf8')
+        return MockBarbicanKey(key_str, key_id)
 
 
 class MockBarbicanKey(object):
@@ -324,7 +329,7 @@ class TestKmsKeymaster(unittest.TestCase):
                 self.swift, TEST_PROXYSERVER_CONF_EXTERNAL_KEYMASTER_CONF)
             raise Exception('Success even though key id invalid')
         except ValueError as e:
-            self.assertEqual(e.message,
+            self.assertEqual(e.args[0],
                              ERR_MESSAGE_SECRET_INCORRECTLY_SPECIFIED)
         except Exception:
             print("Unexpected error: %s" % sys.exc_info()[0])
@@ -369,7 +374,7 @@ class TestKmsKeymaster(unittest.TestCase):
         except Exception as e:
             expected_message = ('Key not found, uuid: ' +
                                 TEST_KMS_NONEXISTENT_KEY_ID)
-            self.assertEqual(e.message, expected_message)
+            self.assertEqual(e.args[0], expected_message)
 
     @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
                 'keystone_password.KeystonePassword')
@@ -453,7 +458,7 @@ class TestKmsKeymaster(unittest.TestCase):
         except Exception as e:
             expected_message = ('keymaster_config_path is set, but there are '
                                 'other config options specified:')
-            self.assertTrue(e.message.startswith(expected_message),
+            self.assertTrue(e.args[0].startswith(expected_message),
                             "Error message does not start with '%s'" %
                             expected_message)
 
@@ -792,6 +797,74 @@ class TestKmsKeymaster(unittest.TestCase):
             print("Unexpected error: %s" % sys.exc_info()[0])
             raise
 
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
+                'keystone_password.KeystonePassword', MockPassword)
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
+    def test_get_root_secret_multiple_keys(
+            self, mock_castellan_key_manager, mock_readconf,
+            mock_castellan_options, mock_oslo_config,):
+        config = dict(TEST_KMS_KEYMASTER_CONF)
+        config.update({
+            'key_id_foo': 'foo-valid_kms_key_id-123456',
+            'key_id_bar': 'bar-valid_kms_key_id-123456',
+            'key_id_baz': 'zz-valid_unicode_kms_key_id-123456',
+            'key_id_non_ascii': u'\N{SNOWMAN}_unicode_key_id',
+            'active_root_secret_id': 'foo'})
+
+        # Set side_effect functions.
+        mock_castellan_key_manager.API.side_effect = (
+            mock_castellan_api_side_effect)
+        mock_castellan_options.set_defaults.side_effect = (
+            mock_options_set_defaults_side_effect)
+        mock_oslo_config.ConfigOpts.side_effect = (
+            mock_config_opts_side_effect)
+
+        # Return valid Barbican configuration parameters.
+        mock_readconf.return_value = config
+
+        self.app = kms_keymaster.KmsKeyMaster(self.swift,
+                                              config)
+
+        expected_secrets = {
+            None: b'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv',
+            'foo': b'ffffffffffffffffffffffffffffffff',
+            'bar': b'bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb',
+            'baz': b'zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz',
+            'non_ascii': b'\xe2\x98\x83' * 32}
+        self.assertDictEqual(self.app._root_secrets, expected_secrets)
+        self.assertEqual(self.app.active_secret_id, 'foo')
+
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.'
+                'keystone_password.KeystonePassword', MockPassword)
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.cfg')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.options')
+    @mock.patch('swift.common.middleware.crypto.keymaster.readconf')
+    @mock.patch('swift.common.middleware.crypto.kms_keymaster.key_manager')
+    def test_get_root_secret_legacy_key_id(
+            self, mock_castellan_key_manager, mock_readconf,
+            mock_castellan_options, mock_oslo_config):
+
+        # Set side_effect functions.
+        mock_castellan_key_manager.API.side_effect = (
+            mock_castellan_api_side_effect)
+        mock_castellan_options.set_defaults.side_effect = (
+            mock_options_set_defaults_side_effect)
+        mock_oslo_config.ConfigOpts.side_effect = (
+            mock_config_opts_side_effect)
+
+        # Return valid Barbican configuration parameters.
+        mock_readconf.return_value = TEST_KMS_KEYMASTER_CONF
+
+        self.app = kms_keymaster.KmsKeyMaster(self.swift,
+                                              TEST_KMS_KEYMASTER_CONF)
+
+        expected_secrets = {None: b'vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv'}
+        self.assertDictEqual(self.app._root_secrets, expected_secrets)
+        self.assertIsNone(self.app.active_secret_id)
+
 
 if __name__ == '__main__':
     unittest.main()
