@@ -12,16 +12,16 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 # This stuff can't live in test/unit/__init__.py due to its swob dependency.
-
-from collections import defaultdict, namedtuple
-from six.moves.urllib import parse
+from collections import defaultdict
+from urllib import parse
 from swift.common import swob
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.request_helpers import is_user_meta, \
-    is_object_transient_sysmeta, resolve_etag_is_at_header
-from swift.common.swob import HTTPNotImplemented
+    is_object_transient_sysmeta, resolve_etag_is_at_header, \
+    resolve_ignore_range_header
+from swift.common.storage_policy import POLICIES
+from swift.common.swob import HTTPMethodNotAllowed
 from swift.common.utils import split_path, md5
 
 from test.debug_logger import debug_logger
@@ -47,13 +47,73 @@ class LeakTrackingIter(object):
             self.mark_read(self.key)
             raise
 
-    next = __next__  # for py2
-
     def close(self):
         self.mark_closed(self.key)
 
 
-FakeSwiftCall = namedtuple('FakeSwiftCall', ['method', 'path', 'headers'])
+class FakeSwiftCall(object):
+    """
+    Encapsulate properties of a request captured by FakeSwift.
+    """
+    DUMMY_VALUE = object()
+
+    def __init__(self, req):
+        self.req = req
+        self.method = req.method
+        path = req.environ['PATH_INFO']
+        if req.environ.get('QUERY_STRING'):
+            path += '?' + req.environ['QUERY_STRING']
+        self.path = normalize_path(path)
+        self.headers = HeaderKeyDict(req.headers)
+        # footers is populated if/when it is passed by FakeSwift to any
+        # update_footers callback that has been set in the request environment.
+        self.footers = HeaderKeyDict()
+        self._env = self._partial_copy(req.environ)
+        # leave FakeSwift to read body and set this attribute after the call
+        # has been captured
+        self.body = None
+
+    def _partial_copy(self, value):
+        # Ideally we want a snapshot of the environ, with deep copies of
+        # mutable values. However, some things (e.g. EncInputWrapper) won't
+        # deepcopy. To avoid a confusing mixed of copied and original
+        # values, replace un-copied values with DUMMY_VALUE.
+        if value is None:
+            return value
+        elif isinstance(value, (bool, int, float, str, bytes)):
+            return value
+        elif isinstance(value, (list, tuple, set)):
+            return value.__class__([self._partial_copy(v) for v in value])
+        elif isinstance(value, dict):
+            return dict([(k, self._partial_copy(v))
+                         for k, v in value.items()])
+        else:
+            return self.DUMMY_VALUE
+
+    @property
+    def env(self):
+        """
+        Returns a partial deepcopy of the request environ as it was received by
+        ``FakeSwift``.
+
+        It may not be possible to deepcopy everything in a request environ, so
+        only values of type ``bool``, ``int``, ``float``, ``str``, ``bytes``,
+        ``list``, ``tuple``, ``set``, or ``dict`` are copied. Other values are
+        replaced with a ``FakeSwiftCall.DUMMY_VALUE`` sentinel so that tests
+        can still check for that key being in ``env``, but not mistakenly
+        assume that the value is a copy.
+
+        Tests that need to make assertions about values not copied into
+        ``FakeSwiftCall.env`` can access the original request environ via
+        ``FakeSwiftCall.req.environ``.
+
+        Writing tests that assert the equality of ``env`` is discouraged (e.g.
+        ``self.assertEqual(expected, call.env)``) because those assertions will
+        break when new default keys are added to the request environ. Tests
+        should instead make assertions about individual items in ``env`` (e.g.
+        ``self.assertEqual(expected, call.env['swift.source')``).
+        """
+        return self._env
 
 
 def normalize_query_string(qs):
@@ -62,6 +122,7 @@ def normalize_query_string(qs):
     if not qs:
         return ''
     else:
+        # N.B. sort params so app.call asserts can hard code qs
         return '?%s' % parse.urlencode(sorted(parse.parse_qsl(qs)))
 
 
@@ -122,17 +183,15 @@ class FakeSwift(object):
     container_existence_skip_cache = 0.0
     account_existence_skip_cache = 0.0
 
-    def __init__(self):
+    def __init__(self, capture_unexpected_calls=True):
+        self.capture_unexpected_calls = capture_unexpected_calls
         self._calls = []
-        self.req_bodies = []
         self._unclosed_req_keys = defaultdict(int)
         self._unread_req_paths = defaultdict(int)
-        self.req_method_paths = []
-        self.swift_sources = []
-        self.txn_ids = []
         self.uploaded = {}
         # mapping of (method, path) --> (response class, headers, body)
         self._responses = {}
+        self._sticky_headers = {}
         self.logger = debug_logger('fake-swift')
         self.account_ring = FakeRing()
         self.container_ring = FakeRing()
@@ -140,26 +199,26 @@ class FakeSwift(object):
         self.auto_create_account_prefix = '.'
         self.backend_user_agent = "fake_swift"
         self._pipeline_final_app = self
-        # some tests want to opt in to mimicking the
-        # X-Backend-Ignore-Range-If-Metadata-Present header behavior,
-        # but default to old-swift behavior
-        self.can_ignore_range = False
+        # Object Servers learned to resolve_ignore_range_header in Jan-2020,
+        # and although we still maintain some middleware tests that assert
+        # proper behavior across rolling upgrades, having a FakeSwift not act
+        # like modern swift is now opt-in.
+        self.can_ignore_range = True
 
     def _find_response(self, method, path):
         path = normalize_path(path)
-        resp = self._responses[(method, path)]
-        if isinstance(resp, list):
-            try:
-                resp = resp.pop(0)
-            except IndexError:
-                raise IndexError("Didn't find any more %r "
-                                 "in allowed responses" % (
-                                     (method, path),))
-        return resp
+        resps = self._responses[(method, path)]
+        if len(resps) == 1:
+            # we'll return the last registered response forever
+            return resps[0]
+        else:
+            return resps.pop(0)
 
-    def _select_response(self, env, method, path):
+    def _select_response(self, env):
         # in some cases we can borrow different registered response
         # ... the order is brittle and significant
+        method = env['REQUEST_METHOD']
+        path = self._parse_path(env)[0]
         preferences = [(method, path)]
         if env.get('QUERY_STRING'):
             # we can always reuse response w/o query string
@@ -189,22 +248,51 @@ class FakeSwift(object):
             # HEAD resp never has body
             body = None
 
-        return resp_class, HeaderKeyDict(headers), body
+        try:
+            is_success = resp_class().is_success
+        except Exception:
+            # test_reconciler passes in an exploding response
+            is_success = False
+        if is_success and method in ('GET', 'HEAD'):
+            # update sticky resp headers with headers from registered resp
+            sticky_headers = self._sticky_headers.get(env['PATH_INFO'], {})
+            resp_headers = HeaderKeyDict(sticky_headers)
+            resp_headers.update(headers)
+        else:
+            # error responses don't get sticky resp headers
+            resp_headers = HeaderKeyDict(headers)
 
-    def __call__(self, env, start_response):
-        if self.can_ignore_range:
-            # we might pop off the Range header
-            env = dict(env)
-        method = env['REQUEST_METHOD']
-        if method not in self.ALLOWED_METHODS:
-            raise HTTPNotImplemented()
+        return resp_class, resp_headers, body
 
+    def _get_policy_index(self, acc, cont):
+        path = '/v1/%s/%s' % (acc, cont)
+        env = {'PATH_INFO': path,
+               'REQUEST_METHOD': 'HEAD'}
+        try:
+            resp_class, headers, _ = self._select_response(env)
+            policy_index = headers.get('X-Backend-Storage-Policy-Index')
+        except KeyError:
+            policy_index = None
+        if policy_index is None:
+            policy_index = str(int(POLICIES.default))
+        return policy_index
+
+    def _parse_path(self, env):
         path = env['PATH_INFO']
+
         _, acc, cont, obj = split_path(env['PATH_INFO'], 0, 4,
                                        rest_with_last=True)
         if env.get('QUERY_STRING'):
             path += '?' + env['QUERY_STRING']
         path = normalize_path(path)
+        return path, acc, cont, obj
+
+    def __call__(self, env, start_response):
+        method = env['REQUEST_METHOD']
+        if method not in self.ALLOWED_METHODS:
+            return HTTPMethodNotAllowed()(env, start_response)
+
+        path, acc, cont, obj = self._parse_path(env)
 
         if 'swift.authorize' in env:
             resp = env['swift.authorize'](swob.Request(env))
@@ -212,51 +300,38 @@ class FakeSwift(object):
                 return resp(env, start_response)
 
         req = swob.Request(env)
-        self.swift_sources.append(env.get('swift.source'))
-        self.txn_ids.append(env.get('swift.trans_id'))
-
-        resp_class, headers, body = self._select_response(env, method, path)
-
-        ignore_range_meta = req.headers.get(
-            'x-backend-ignore-range-if-metadata-present')
-        if self.can_ignore_range and ignore_range_meta and set(
-                ignore_range_meta.split(',')).intersection(headers.keys()):
-            req.headers.pop('range', None)
-
-        # Update req.headers before capturing the request
-        if method in ('GET', 'HEAD') and obj:
-            req.headers['X-Backend-Storage-Policy-Index'] = headers.get(
-                'x-backend-storage-policy-index', '2')
 
         # Capture the request before reading the body, in case the iter raises
         # an exception.
-        # note: tests may assume this copy of req_headers is case insensitive
-        # so we deliberately use a HeaderKeyDict
-        req_headers_copy = HeaderKeyDict(req.headers)
-        self._calls.append(
-            FakeSwiftCall(method, path, req_headers_copy))
+        call = FakeSwiftCall(req)
+        try:
+            resp_class, headers, body = self._select_response(env)
+        except KeyError:
+            if self.capture_unexpected_calls:
+                self._calls.append(call)
+            raise
+        else:
+            self._calls.append(call)
 
-        req_body = None  # generally, we don't care and let eventlet discard()
         if (cont and not obj and method == 'UPDATE') or (
                 obj and method == 'PUT'):
-            req_body = b''.join(iter(env['wsgi.input'].read, b''))
+            call.body = b''.join(iter(env['wsgi.input'].read, b''))
 
         # simulate object PUT
         if method == 'PUT' and obj:
             if 'swift.callback.update_footers' in env:
                 footers = HeaderKeyDict()
                 env['swift.callback.update_footers'](footers)
-                req.headers.update(footers)
-                req_headers_copy.update(footers)
-            etag = md5(req_body, usedforsecurity=False).hexdigest()
+                call.footers.update(footers)
+            etag = md5(call.body, usedforsecurity=False).hexdigest()
             headers.setdefault('Etag', etag)
-            headers.setdefault('Content-Length', len(req_body))
+            headers.setdefault('Content-Length', len(call.body))
 
             # keep it for subsequent GET requests later
-            resp_headers = dict(req.headers)
+            metadata = dict(call.headers, **call.footers)
             if "CONTENT_TYPE" in env:
-                resp_headers['Content-Type'] = env["CONTENT_TYPE"]
-            self.uploaded[env['PATH_INFO']] = (resp_headers, req_body)
+                metadata['Content-Type'] = env["CONTENT_TYPE"]
+            self.uploaded[env['PATH_INFO']] = (metadata, call.body)
 
         # simulate object POST
         elif method == 'POST' and obj:
@@ -268,17 +343,29 @@ class FakeSwift(object):
                     is_object_transient_sysmeta(k)))
             # apply from new
             new_metadata.update(
-                dict((k, v) for k, v in req.headers.items()
+                dict((k, v)
+                     for k, v in dict(call.headers, **call.footers).items()
                      if (is_user_meta('object', k) or
                          is_object_transient_sysmeta(k) or
                          k.lower == 'content-type')))
             self.uploaded[env['PATH_INFO']] = new_metadata, data
 
-        self.req_bodies.append(req_body)
+        # Some middlewares (e.g. proxy_logging) inspect the request headers
+        # after it has been handled, so simulate some request headers updates
+        # that the real proxy makes. Do this *after* the request has been
+        # captured in the state it was received.
+        if obj:
+            req.headers.setdefault('X-Backend-Storage-Policy-Index',
+                                   self._get_policy_index(acc, cont))
 
         # Apply conditional etag overrides
         conditional_etag = resolve_etag_is_at_header(req, headers)
 
+        if self.can_ignore_range:
+            # avoid popping range from original environ
+            req = swob.Request(dict(req.environ))
+            resolve_ignore_range_header(req, headers)
+
         # range requests ought to work, hence conditional_response=True
         if isinstance(body, list):
             resp = resp_class(
@@ -290,11 +377,14 @@ class FakeSwift(object):
                 req=req, headers=headers, body=body,
                 conditional_response=req.method in ('GET', 'HEAD'),
                 conditional_etag=conditional_etag)
-        wsgi_iter = resp(env, start_response)
+        wsgi_iter = resp(req.environ, start_response)
         self.mark_opened((method, path))
         return LeakTrackingIter(wsgi_iter, self.mark_closed,
                                 self.mark_read, (method, path))
 
+    def clear_calls(self):
+        del self._calls[:]
+
     def mark_opened(self, key):
         self._unclosed_req_keys[key] += 1
         self._unread_req_paths[key] += 1
@@ -318,28 +408,65 @@ class FakeSwift(object):
                 if count > 0}
 
     @property
-    def calls(self):
-        return [(method, path) for method, path, headers in self._calls]
-
-    @property
-    def headers(self):
-        return [headers for method, path, headers in self._calls]
-
-    @property
-    def calls_with_headers(self):
+    def call_list(self):
+        """
+        Returns a list of instances of ``FakeSwiftCall``.
+        """
         return self._calls
 
+    @property
+    def calls(self):
+        """
+        Returns a list of 2-tuples (<method>, <path>) for each item in
+        ``call_list``.
+        """
+        return [(call.method, call.path) for call in self._calls]
+
+    @property
+    def headers(self):
+        """
+        Returns a list of headers for each item in ``call_list``.
+        """
+        return [call.headers for call in self._calls]
+
+    @property
+    def calls_with_headers(self):
+        """
+        Returns a list of 3-tuples (<method>, <path>, <headers>) for each item
+        in ``call_list``.
+        """
+        return [(call.method, call.path, call.headers)
+                for call in self._calls]
+
     @property
     def call_count(self):
-        return len(self._calls)
+        return len(self.call_list)
+
+    @property
+    def txn_ids(self):
+        return [call.env.get('swift.trans_id') for call in self.call_list]
+
+    @property
+    def swift_sources(self):
+        return [call.env.get('swift.source') for call in self.call_list]
+
+    def update_sticky_response_headers(self, path, headers):
+        """
+        Tests setUp can use this to ensure any successful GET/HEAD response for
+        a given path will include these headers.
+        """
+        sticky_headers = self._sticky_headers.setdefault(path, {})
+        sticky_headers.update(headers)
 
     def register(self, method, path, response_class, headers, body=b''):
         path = normalize_path(path)
-        self._responses[(method, path)] = (response_class, headers, body)
+        self._responses[(method, path)] = [(response_class, headers, body)]
 
-    def register_responses(self, method, path, responses):
-        path = normalize_path(path)
-        self._responses[(method, path)] = list(responses)
+    def register_next_response(self, method, path,
+                               response_class, headers, body=b''):
+        resp_key = (method, normalize_path(path))
+        next_resp = (response_class, headers, body)
+        self._responses.setdefault(resp_key, []).append(next_resp)
 
 
 class FakeAppThatExcepts(object):
