@@ -15,21 +15,21 @@
 import copy
 import base64
 import json
-import logging
 import time
 import unittest
 import uuid
 
-import fixtures
-import mock
+from unittest import mock
 import requests
 from requests_mock.contrib import fixture as rm_fixture
-from six.moves import urllib
+import urllib.parse
 
 from swift.common.middleware.s3api import s3token
 from swift.common.swob import Request, Response
 from swift.common.wsgi import ConfigFileError
 
+from test.debug_logger import debug_logger
+
 GOOD_RESPONSE_V2 = {'access': {
     'user': {
         'username': 'S3_USER',
@@ -72,7 +72,7 @@ GOOD_RESPONSE_V3 = {'token': {
 }}
 
 
-class TestResponse(requests.Response):
+class FakeResponse(requests.Response):
     """Utility class to wrap requests.Response.
 
     Class used to wrap requests.Response and provide some convenience to
@@ -81,7 +81,7 @@ class TestResponse(requests.Response):
 
     def __init__(self, data):
         self._text = None
-        super(TestResponse, self).__init__()
+        super(FakeResponse, self).__init__()
         if isinstance(data, dict):
             self.status_code = data.get('status_code', 200)
             headers = data.get('headers')
@@ -130,8 +130,7 @@ class S3TokenMiddlewareTestBase(unittest.TestCase):
 
     def setUp(self):
         super(S3TokenMiddlewareTestBase, self).setUp()
-        self.logger = fixtures.FakeLogger(level=logging.DEBUG)
-        self.logger.setUp()
+        self.logger = debug_logger()
         self.time_patcher = mock.patch.object(time, 'time', lambda: 1234)
         self.time_patcher.start()
 
@@ -139,15 +138,19 @@ class S3TokenMiddlewareTestBase(unittest.TestCase):
         self.conf = {
             'auth_uri': self.TEST_AUTH_URI,
         }
-        self.middleware = s3token.S3Token(self.app, self.conf)
+        self.middleware = self.make_middleware(self.conf)
 
         self.requests_mock = rm_fixture.Fixture()
         self.requests_mock.setUp()
 
+    def make_middleware(self, conf):
+        with mock.patch('swift.common.middleware.s3api.s3token.get_logger',
+                        return_value=self.logger):
+            return s3token.S3Token(self.app, conf)
+
     def tearDown(self):
         self.requests_mock.cleanUp()
         self.time_patcher.stop()
-        self.logger.cleanUp()
         super(S3TokenMiddlewareTestBase, self).tearDown()
 
     def start_fake_response(self, status, headers):
@@ -196,7 +199,8 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
         self.middleware(req.environ, self.start_fake_response)
         self.assertEqual(self.response_status, 200)
 
-    def _assert_authorized(self, req, account_path='/v1/AUTH_TENANT_ID/'):
+    def _assert_authorized(self, req, account_path='/v1/AUTH_TENANT_ID/',
+                           access_key='access'):
         self.assertTrue(
             req.path.startswith(account_path),
             '%r does not start with %r' % (req.path, account_path))
@@ -221,7 +225,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
         self.assertEqual(1, self.requests_mock.call_count)
         request_call = self.requests_mock.request_history[0]
         self.assertEqual(json.loads(request_call.body), {'credentials': {
-            'access': 'access',
+            'access': access_key,
             'signature': 'signature',
             'token': base64.urlsafe_b64encode(b'token').decode('ascii')}})
 
@@ -267,8 +271,8 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
             '%s/s3tokens' % auth_uri,
             status_code=201, json=GOOD_RESPONSE_V2)
 
-        self.middleware = s3token.filter_factory({
-            'auth_uri': auth_uri})(self.app)
+        self.middleware = self.make_middleware({
+            'auth_uri': auth_uri})
         req = Request.blank('/v1/AUTH_cfa/c/o')
         req.environ['s3api.auth_details'] = {
             'access_key': u'access',
@@ -286,8 +290,8 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
             '%s/s3tokens' % auth_uri,
             status_code=201, json=GOOD_RESPONSE_V2)
 
-        self.middleware = s3token.filter_factory({
-            'auth_uri': auth_uri})(self.app)
+        self.middleware = self.make_middleware({
+            'auth_uri': auth_uri})
         req = Request.blank('/v1/AUTH_cfa/c/o')
         req.environ['s3api.auth_details'] = {
             'access_key': u'access',
@@ -298,8 +302,8 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
         self._assert_authorized(req)
 
     def test_authorized_trailing_slash(self):
-        self.middleware = s3token.filter_factory({
-            'auth_uri': self.TEST_AUTH_URI + '/'})(self.app)
+        self.middleware = self.make_middleware({
+            'auth_uri': self.TEST_AUTH_URI + '/'})
         req = Request.blank('/v1/AUTH_cfa/c/o')
         req.environ['s3api.auth_details'] = {
             'access_key': u'access',
@@ -321,11 +325,11 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
 
     @mock.patch.object(requests, 'post')
     def test_insecure(self, MOCK_REQUEST):
-        self.middleware = s3token.filter_factory(
-            {'insecure': 'True', 'auth_uri': 'http://example.com'})(self.app)
+        self.middleware = self.make_middleware({
+            'insecure': 'True', 'auth_uri': 'http://example.com'})
 
         text_return_value = json.dumps(GOOD_RESPONSE_V2)
-        MOCK_REQUEST.return_value = TestResponse({
+        MOCK_REQUEST.return_value = FakeResponse({
             'status_code': 201,
             'text': text_return_value})
 
@@ -405,12 +409,12 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
 
     @mock.patch.object(requests, 'post')
     def test_http_timeout(self, MOCK_REQUEST):
-        self.middleware = s3token.filter_factory({
+        self.middleware = self.make_middleware({
             'http_timeout': '2',
             'auth_uri': 'http://example.com',
-        })(FakeApp())
+        })
 
-        MOCK_REQUEST.return_value = TestResponse({
+        MOCK_REQUEST.return_value = FakeResponse({
             'status_code': 201,
             'text': json.dumps(GOOD_RESPONSE_V2)})
 
@@ -432,7 +436,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
             middleware = s3token.filter_factory({
                 'http_timeout': val,
                 'auth_uri': 'http://example.com',
-            })(FakeApp())
+            })(self.app)
             self.assertEqual(float(val), middleware._timeout)
 
         bad_values = ['1, 4', '-3', '100', 'foo', '0']
@@ -441,7 +445,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
                 s3token.filter_factory({
                     'http_timeout': val,
                     'auth_uri': 'http://example.com',
-                })(FakeApp())
+                })(self.app)
             self.assertTrue(ctx.exception.args[0].startswith((
                 'invalid literal for float():',
                 'could not convert string to float:',
@@ -450,7 +454,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
 
         # default is 10 seconds
         middleware = s3token.filter_factory({
-            'auth_uri': 'http://example.com'})(FakeApp())
+            'auth_uri': 'http://example.com'})(self.app)
         self.assertEqual(10, middleware._timeout)
 
     def test_bad_auth_uris(self):
@@ -498,6 +502,18 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
         self._assert_authorized(req, account_path='/v1/')
         self.assertEqual(req.environ['PATH_INFO'], '/v1/AUTH_TENANT_ID/c/o')
 
+    def test_authorize_with_unicode_access_key(self):
+        req = Request.blank('/v1/acc\xc3\xa9sskey/c/o')
+        req.environ['s3api.auth_details'] = {
+            'access_key': u'acc\u00e9ss',
+            'signature': u'signature',
+            'string_to_sign': u'token',
+        }
+        req.get_response(self.middleware)
+        self._assert_authorized(req, account_path='/v1/',
+                                access_key=u'acc\u00e9ss')
+        self.assertEqual(req.environ['PATH_INFO'], '/v1/AUTH_TENANT_ID/c/o')
+
     def test_authorize_with_access_key_and_unquote_chars(self):
         req = Request.blank('/v1/access%key=/c/o')
         req.environ['s3api.auth_details'] = {
@@ -514,7 +530,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
     @mock.patch.object(requests, 'post')
     def test_secret_is_cached(self, MOCK_REQUEST, MOCK_KEYSTONE,
                               MOCK_CACHE_FROM_ENV):
-        self.middleware = s3token.filter_factory({
+        self.middleware = self.make_middleware({
             'auth_uri': 'http://example.com',
             'secret_cache_duration': '20',
             'auth_type': 'v3password',
@@ -524,15 +540,16 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
             'project_name': 'service',
             'user_domain_name': 'default',
             'project_domain_name': 'default',
-        })(FakeApp())
+        })
         self.assertEqual(20, self.middleware._secret_cache_duration)
+        self.assertIsNone(MOCK_KEYSTONE.mock_calls[0][2]['region_name'])
 
         cache = MOCK_CACHE_FROM_ENV.return_value
 
         fake_cache_response = ({}, {'id': 'tenant_id'}, 'secret')
         cache.get.return_value = fake_cache_response
 
-        MOCK_REQUEST.return_value = TestResponse({
+        MOCK_REQUEST.return_value = FakeResponse({
             'status_code': 201,
             'text': json.dumps(GOOD_RESPONSE_V2)})
 
@@ -552,7 +569,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
     @mock.patch.object(requests, 'post')
     def test_secret_sets_cache(self, MOCK_REQUEST, MOCK_KEYSTONE,
                                MOCK_CACHE_FROM_ENV):
-        self.middleware = s3token.filter_factory({
+        self.middleware = self.make_middleware({
             'auth_uri': 'http://example.com',
             'secret_cache_duration': '20',
             'auth_type': 'v3password',
@@ -562,8 +579,11 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
             'project_name': 'service',
             'user_domain_name': 'default',
             'project_domain_name': 'default',
-        })(FakeApp())
+            'region_name': 'some-other-region',
+        })
         self.assertEqual(20, self.middleware._secret_cache_duration)
+        self.assertEqual(MOCK_KEYSTONE.mock_calls[0][2]['region_name'],
+                         'some-other-region')
 
         cache = MOCK_CACHE_FROM_ENV.return_value
         cache.get.return_value = None
@@ -571,7 +591,7 @@ class S3TokenMiddlewareTestGood(S3TokenMiddlewareTestBase):
         keystone_client = MOCK_KEYSTONE.return_value
         keystone_client.ec2.get.return_value = mock.Mock(secret='secret')
 
-        MOCK_REQUEST.return_value = TestResponse({
+        MOCK_REQUEST.return_value = FakeResponse({
             'status_code': 201,
             'text': json.dumps(GOOD_RESPONSE_V2).encode('ascii')})
 
@@ -722,7 +742,7 @@ class S3TokenMiddlewareTestDeferredAuth(S3TokenMiddlewareTestBase):
     def setUp(self):
         super(S3TokenMiddlewareTestDeferredAuth, self).setUp()
         self.conf['delay_auth_decision'] = 'yes'
-        self.middleware = s3token.S3Token(FakeApp(), self.conf)
+        self.middleware = self.make_middleware(self.conf)
 
     def test_unauthorized_token(self):
         ret = {"error":
@@ -848,8 +868,8 @@ class S3TokenMiddlewareTestV3(S3TokenMiddlewareTestBase):
             auth_uri + 's3tokens',
             status_code=201, json=GOOD_RESPONSE_V3)
 
-        self.middleware = s3token.filter_factory({
-            'auth_uri': auth_uri})(self.app)
+        self.middleware = self.make_middleware({
+            'auth_uri': auth_uri})
         req = Request.blank('/v1/AUTH_cfa/c/o')
         req.environ['s3api.auth_details'] = {
             'access_key': u'access',
@@ -865,8 +885,8 @@ class S3TokenMiddlewareTestV3(S3TokenMiddlewareTestBase):
             auth_uri + 's3tokens',
             status_code=201, json=GOOD_RESPONSE_V3)
 
-        self.middleware = s3token.filter_factory({
-            'auth_uri': auth_uri})(self.app)
+        self.middleware = self.make_middleware({
+            'auth_uri': auth_uri})
         req = Request.blank('/v1/AUTH_cfa/c/o')
         req.environ['s3api.auth_details'] = {
             'access_key': u'access',
@@ -877,8 +897,8 @@ class S3TokenMiddlewareTestV3(S3TokenMiddlewareTestBase):
         self._assert_authorized(req)
 
     def test_authorized_trailing_slash(self):
-        self.middleware = s3token.filter_factory({
-            'auth_uri': self.TEST_AUTH_URI + '/'})(self.app)
+        self.middleware = self.make_middleware({
+            'auth_uri': self.TEST_AUTH_URI + '/'})
         req = Request.blank('/v1/AUTH_cfa/c/o')
         req.environ['s3api.auth_details'] = {
             'access_key': u'access',
@@ -949,6 +969,18 @@ class S3TokenMiddlewareTestV3(S3TokenMiddlewareTestBase):
         self._assert_authorized(req, account_path='/v1/')
         self.assertEqual(req.environ['PATH_INFO'], '/v1/AUTH_PROJECT_ID/c/o')
 
+    def test_authorize_with_access_key_in_container(self):
+        req = Request.blank('/v1/accesskey/accesskey.c/o')
+        req.environ['s3api.auth_details'] = {
+            'access_key': u'access',
+            'signature': u'signature',
+            'string_to_sign': u'token',
+        }
+        req.get_response(self.middleware)
+        self._assert_authorized(req, account_path='/v1/')
+        self.assertEqual(req.environ['PATH_INFO'],
+                         '/v1/AUTH_PROJECT_ID/accesskey.c/o')
+
     def test_authorize_with_access_key_and_unquote_chars(self):
         req = Request.blank('/v1/ab%c=/c/o')
         req.environ['s3api.auth_details'] = {
