@@ -17,12 +17,12 @@ import functools
 import json
 import os
 import time
-import mock
+from unittest import mock
 import unittest
-from swift.common import swob, utils
+from swift.common import swob, utils, registry
 from swift.common.middleware import versioned_writes, copy
 from swift.common.swob import Request
-from test.unit.common.middleware.helpers import FakeSwift
+from test.unit.common.middleware import helpers
 
 
 class FakeCache(object):
@@ -58,12 +58,14 @@ def local_tz(func):
 
 class VersionedWritesBaseTestCase(unittest.TestCase):
     def setUp(self):
-        self.app = FakeSwift()
+        self.app = helpers.FakeSwift()
         conf = {'allow_versioned_writes': 'true'}
-        self.vw = versioned_writes.filter_factory(conf)(self.app)
+        self.vw = versioned_writes.legacy.VersionedWritesMiddleware(
+            self.app, conf)
 
     def tearDown(self):
         self.assertEqual(self.app.unclosed_requests, {})
+        self.assertEqual(self.app.unread_requests, {})
 
     def call_app(self, req, app=None):
         if app is None:
@@ -392,7 +394,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '201 Created')
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
-        called_method = [method for (method, path, hdrs) in self.app._calls]
+        called_method = [call.method for call in self.app.call_list]
         self.assertNotIn('GET', called_method)
 
     def test_put_request_is_dlo_manifest_with_container_config_true(self):
@@ -422,7 +424,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             ('PUT', '/v1/a/c/o'),
         ], self.app.calls)
         self.assertIn('x-object-manifest',
-                      self.app.calls_with_headers[2].headers)
+                      self.app.call_list[2].headers)
 
     def test_put_version_is_dlo_manifest_with_container_config_true(self):
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk,
@@ -454,7 +456,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             ('PUT', '/v1/a/c/o'),
         ], self.app.calls)
         self.assertIn('x-object-manifest',
-                      self.app.calls_with_headers[1].headers)
+                      self.app.call_list[1].headers)
 
     def test_delete_object_no_versioning_with_container_config_true(self):
         # set False to versions_write obviously and expect no GET versioning
@@ -470,8 +472,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '204 No Content')
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
-        called_method = \
-            [method for (method, path, rheaders) in self.app._calls]
+        called_method = [call.method for call in self.app.call_list]
         self.assertNotIn('PUT', called_method)
         self.assertNotIn('GET', called_method)
         self.assertEqual(1, self.app.call_count)
@@ -590,8 +591,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.app.register(
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
-            'GET',
-            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
+            'GET', helpers.normalize_path(
+                '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on'),
             swob.HTTPNotFound, {}, None)
 
         cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
@@ -609,7 +610,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('DELETE', '/v1/a/c/o'),
         ])
 
@@ -617,8 +619,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.app.register(
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
-            'GET',
-            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
+            'GET', '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {}, '[]')
 
         cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
@@ -633,7 +634,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('DELETE', '/v1/a/c/o'),
         ])
 
@@ -681,7 +683,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('GET', '/v1/a/ver_cont/001o/2?symlink=get'),
             ('PUT', '/v1/a/c/o'),
             ('DELETE', '/v1/a/ver_cont/001o/2'),
@@ -720,17 +723,17 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, self.authorized[0])
         self.assertRequestEqual(req, self.authorized[1])
 
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'HEAD', 'DELETE'],
                          [c.method for c in calls])
 
         self.assertIn('X-Newest', calls[1].headers)
         self.assertEqual('True', calls[1].headers['X-Newest'])
 
-        method, path, req_headers = calls.pop()
-        self.assertTrue(path.startswith('/v1/a/c/o'))
+        call = calls.pop()
+        self.assertTrue(call.path.startswith('/v1/a/c/o'))
         # Since we're deleting the original, this *should* still be present:
-        self.assertEqual('1', req_headers.get('X-If-Delete-At'))
+        self.assertEqual('1', call.headers.get('X-If-Delete-At'))
 
     def test_delete_latest_version_is_marker_success(self):
         # Test popping a delete marker off the stack. So, there's data in the
@@ -738,7 +741,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         # in the base versioned container.
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
+            helpers.normalize_path(
+                '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on'),
             swob.HTTPOk, {},
             '[{"hash": "y", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -775,7 +779,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('HEAD', '/v1/a/c/o'),
             ('GET', '/v1/a/ver_cont/001o/1?symlink=get'),
             ('PUT', '/v1/a/c/o'),
@@ -830,15 +835,15 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, self.authorized[0])
 
         # check that X-If-Delete-At was removed from DELETE request
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'HEAD', 'DELETE'],
                          [c.method for c in calls])
 
-        method, path, req_headers = calls.pop()
-        self.assertTrue(path.startswith('/v1/a/ver_cont/001o/3'))
-        self.assertNotIn('x-if-delete-at', [h.lower() for h in req_headers])
+        call = calls.pop()
+        self.assertTrue(call.path.startswith('/v1/a/ver_cont/001o/3'))
+        self.assertNotIn('x-if-delete-at', [h.lower() for h in call.headers])
 
-    @mock.patch('swift.common.middleware.versioned_writes.time.time',
+    @mock.patch('swift.common.middleware.versioned_writes.legacy.time.time',
                 return_value=1234)
     def test_history_delete_marker_no_object_success(self, mock_time):
         self.app.register(
@@ -863,12 +868,12 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         req.environ['REQUEST_METHOD'] = 'PUT'
         self.assertRequestEqual(req, self.authorized[0])
 
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'PUT', 'DELETE'], [c.method for c in calls])
         self.assertEqual('application/x-deleted;swift_versions_deleted=1',
                          calls[1].headers.get('Content-Type'))
 
-    @mock.patch('swift.common.middleware.versioned_writes.time.time',
+    @mock.patch('swift.common.middleware.versioned_writes.legacy.time.time',
                 return_value=123456789.54321)
     def test_history_delete_marker_over_object_success(self, mock_time):
         self.app.register(
@@ -897,7 +902,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         req.environ['REQUEST_METHOD'] = 'PUT'
         self.assertRequestEqual(req, self.authorized[0])
 
-        calls = self.app.calls_with_headers
+        calls = self.app.call_list
         self.assertEqual(['GET', 'PUT', 'PUT', 'DELETE'],
                          [c.method for c in calls])
         self.assertEqual('/v1/a/ver_cont/001o/1416421142.00000',
@@ -940,7 +945,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('GET', '/v1/a/ver_cont/001o/1?symlink=get'),
             ('PUT', '/v1/a/c/o'),
             ('DELETE', '/v1/a/ver_cont/001o/1'),
@@ -988,11 +994,14 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
-            ('GET', '/v1/a/ver_cont/001o/2?symlink=get'),
-            ('GET', '/v1/a/ver_cont/001o/1?symlink=get'),
-            ('PUT', '/v1/a/c/o'),
-            ('DELETE', '/v1/a/ver_cont/001o/1'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
+            ('GET', helpers.normalize_path(
+                '/v1/a/ver_cont/001o/2?symlink=get')),
+            ('GET', helpers.normalize_path(
+                '/v1/a/ver_cont/001o/1?symlink=get')),
+            ('PUT', helpers.normalize_path('/v1/a/c/o')),
+            ('DELETE', helpers.normalize_path('/v1/a/ver_cont/001o/1')),
         ])
 
     def test_denied_DELETE_of_versioned_object(self):
@@ -1030,7 +1039,8 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
         ])
 
     def test_denied_PUT_of_versioned_object(self):
@@ -1112,8 +1122,10 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
-            ('GET', prefix_listing_prefix + 'marker=001o/2'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/2')),
             ('GET', '/v1/a/ver_cont/001o/2?symlink=get'),
             ('PUT', '/v1/a/c/o'),
             ('DELETE', '/v1/a/ver_cont/001o/2'),
@@ -1165,8 +1177,10 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
-            ('GET', prefix_listing_prefix + 'marker=001o/2'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/2')),
             ('GET', '/v1/a/ver_cont/001o/2?symlink=get'),
             ('GET', '/v1/a/ver_cont/001o/1?symlink=get'),
             ('PUT', '/v1/a/c/o'),
@@ -1215,8 +1229,10 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, authorize_call[0])
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
-            ('GET', prefix_listing_prefix + 'marker=001o/2'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/2')),
         ])
 
     def test_partially_upgraded_cluster(self):
@@ -1281,14 +1297,19 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '204 No Content')
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('GET', '/v1/a/ver_cont/001o/4?symlink=get'),
             ('GET', '/v1/a/ver_cont/001o/3?symlink=get'),
             ('GET', '/v1/a/ver_cont/001o/2?symlink=get'),
-            ('GET', prefix_listing_prefix + 'marker=001o/2&reverse=on'),
-            ('GET', prefix_listing_prefix + 'marker=&end_marker=001o/2'),
-            ('GET', prefix_listing_prefix + 'marker=001o/0&end_marker=001o/2'),
-            ('GET', prefix_listing_prefix + 'marker=001o/1&end_marker=001o/2'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/2&reverse=on')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&end_marker=001o/2')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/0&end_marker=001o/2')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/1&end_marker=001o/2')),
             ('GET', '/v1/a/ver_cont/001o/1?symlink=get'),
             ('PUT', '/v1/a/c/o'),
             ('DELETE', '/v1/a/ver_cont/001o/1'),
@@ -1353,13 +1374,18 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '204 No Content')
         prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
-            ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&reverse=on')),
             ('GET', '/v1/a/ver_cont/001o/4?symlink=get'),
             ('GET', '/v1/a/ver_cont/001o/3?symlink=get'),
-            ('GET', prefix_listing_prefix + 'marker=001o/3&reverse=on'),
-            ('GET', prefix_listing_prefix + 'marker=&end_marker=001o/3'),
-            ('GET', prefix_listing_prefix + 'marker=001o/1&end_marker=001o/3'),
-            ('GET', prefix_listing_prefix + 'marker=001o/2&end_marker=001o/3'),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/3&reverse=on')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=&end_marker=001o/3')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/1&end_marker=001o/3')),
+            ('GET', helpers.normalize_path(
+                prefix_listing_prefix + 'marker=001o/2&end_marker=001o/3')),
             ('GET', '/v1/a/ver_cont/001o/2?symlink=get'),
             ('PUT', '/v1/a/c/o'),
             ('DELETE', '/v1/a/ver_cont/001o/2'),
@@ -1370,7 +1396,7 @@ class VersionedWritesCopyingTestCase(VersionedWritesBaseTestCase):
     # verify interaction of copy and versioned_writes middlewares
 
     def setUp(self):
-        self.app = FakeSwift()
+        self.app = helpers.FakeSwift()
         conf = {'allow_versioned_writes': 'true'}
         self.vw = versioned_writes.filter_factory(conf)(self.app)
         self.filter = copy.filter_factory({})(self.vw)
@@ -1488,19 +1514,19 @@ class VersionedWritesCopyingTestCase(VersionedWritesBaseTestCase):
 
 class TestSwiftInfo(unittest.TestCase):
     def setUp(self):
-        utils._swift_info = {}
-        utils._swift_admin_info = {}
+        registry._swift_info = {}
+        registry._swift_admin_info = {}
 
     def test_registered_defaults(self):
         versioned_writes.filter_factory({})('have to pass in an app')
-        swift_info = utils.get_swift_info()
+        swift_info = registry.get_swift_info()
         # in default, versioned_writes is not in swift_info
         self.assertNotIn('versioned_writes', swift_info)
 
     def test_registered_explicitly_set(self):
         versioned_writes.filter_factory(
             {'allow_versioned_writes': 'true'})('have to pass in an app')
-        swift_info = utils.get_swift_info()
+        swift_info = registry.get_swift_info()
         self.assertIn('versioned_writes', swift_info)
         self.assertEqual(
             swift_info['versioned_writes'].get('allowed_flags'),
