@@ -13,19 +13,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import configparser
 import os
-from six import StringIO
+from io import StringIO
 import time
 import unittest
 from getpass import getuser
 import logging
-from test.unit import tmpfile
-import mock
+from test.unit import tmpfile, with_tempdir
+from unittest import mock
 import signal
 from contextlib import contextmanager
 import itertools
 from collections import defaultdict
 import errno
+from textwrap import dedent
 
 from swift.common import daemon, utils
 from test.debug_logger import debug_logger
@@ -58,7 +60,7 @@ class TestDaemon(unittest.TestCase):
     def test_create(self):
         d = daemon.Daemon({})
         self.assertEqual(d.conf, {})
-        self.assertTrue(isinstance(d.logger, utils.LogAdapter))
+        self.assertIsInstance(d.logger, utils.logs.SwiftLogAdapter)
 
     def test_stubs(self):
         d = daemon.Daemon({})
@@ -147,9 +149,12 @@ class TestRunDaemon(unittest.TestCase):
         ])
 
     def test_run_daemon(self):
+        logging.logThreads = 1  # reset to default
         sample_conf = "[my-daemon]\nuser = %s\n" % getuser()
         with tmpfile(sample_conf) as conf_file, \
-                mock.patch('swift.common.daemon.use_hub') as mock_use_hub:
+                mock.patch('swift.common.utils.eventlet') as _utils_evt, \
+                mock.patch('eventlet.hubs.use_hub') as mock_use_hub, \
+                mock.patch('eventlet.debug') as _debug_evt:
             with mock.patch.dict('os.environ', {'TZ': ''}), \
                     mock.patch('time.tzset') as mock_tzset:
                 daemon.run_daemon(MyDaemon, conf_file)
@@ -159,6 +164,12 @@ class TestRunDaemon(unittest.TestCase):
                 self.assertEqual(mock_use_hub.mock_calls,
                                  [mock.call(utils.get_hub())])
             daemon.run_daemon(MyDaemon, conf_file, once=True)
+            _utils_evt.patcher.monkey_patch.assert_called_with(all=False,
+                                                               socket=True,
+                                                               select=True,
+                                                               thread=True)
+            self.assertEqual(0, logging.logThreads)  # fixed in monkey_patch
+            _debug_evt.hub_exceptions.assert_called_with(False)
             self.assertEqual(MyDaemon.once_called, True)
 
             # test raise in daemon code
@@ -195,7 +206,9 @@ class TestRunDaemon(unittest.TestCase):
 
             sample_conf = "[my-daemon]\nuser = %s\n" % getuser()
             with tmpfile(sample_conf) as conf_file, \
-                    mock.patch('swift.common.daemon.use_hub'):
+                    mock.patch('swift.common.utils.eventlet'), \
+                    mock.patch('eventlet.hubs.use_hub'), \
+                    mock.patch('eventlet.debug'):
                 daemon.run_daemon(MyDaemon, conf_file)
                 self.assertFalse(MyDaemon.once_called)
                 self.assertTrue(MyDaemon.forever_called)
@@ -207,6 +220,113 @@ class TestRunDaemon(unittest.TestCase):
             os.environ['TZ'] = old_tz
             time.tzset()
 
+    @with_tempdir
+    def test_run_deamon_from_conf_file(self, tempdir):
+        conf_path = os.path.join(tempdir, 'test-daemon.conf')
+        conf_body = """
+        [DEFAULT]
+        conn_timeout = 5
+        client_timeout = 1
+        [my-daemon]
+        CONN_timeout = 10
+        client_timeout = 2
+        """
+        contents = dedent(conf_body)
+        with open(conf_path, 'w') as f:
+            f.write(contents)
+        with mock.patch('swift.common.utils.eventlet'), \
+                mock.patch('eventlet.hubs.use_hub'), \
+                mock.patch('eventlet.debug'):
+            d = daemon.run_daemon(MyDaemon, conf_path)
+        # my-daemon section takes priority (!?)
+        self.assertEqual('2', d.conf['client_timeout'])
+        self.assertEqual('10', d.conf['CONN_timeout'])
+        self.assertEqual('5', d.conf['conn_timeout'])
+
+    @with_tempdir
+    def test_run_daemon_from_conf_file_with_duplicate_var(self, tempdir):
+        conf_path = os.path.join(tempdir, 'test-daemon.conf')
+        conf_body = """
+        [DEFAULT]
+        client_timeout = 3
+        [my-daemon]
+        CLIENT_TIMEOUT = 2
+        client_timeout = 1
+        conn_timeout = 1.1
+        conn_timeout = 1.2
+        """
+        contents = dedent(conf_body)
+        with open(conf_path, 'w') as f:
+            f.write(contents)
+        with mock.patch('swift.common.utils.eventlet'), \
+                mock.patch('eventlet.hubs.use_hub'), \
+                mock.patch('eventlet.debug'):
+            with self.assertRaises(
+                    configparser.DuplicateOptionError) as ctx:
+                daemon.run_daemon(MyDaemon, tempdir)
+            msg = str(ctx.exception)
+            self.assertIn('conn_timeout', msg)
+            self.assertIn('already exists', msg)
+
+    @with_tempdir
+    def test_run_deamon_from_conf_dir(self, tempdir):
+        conf_files = {
+            'default': """
+            [DEFAULT]
+            conn_timeout = 5
+            client_timeout = 1
+            """,
+            'daemon': """
+            [DEFAULT]
+            CONN_timeout = 3
+            CLIENT_TIMEOUT = 4
+            [my-daemon]
+            CONN_timeout = 10
+            client_timeout = 2
+            """,
+        }
+        for filename, conf_body in conf_files.items():
+            path = os.path.join(tempdir, filename + '.conf')
+            with open(path, 'wt') as fd:
+                fd.write(dedent(conf_body))
+        with mock.patch('swift.common.utils.eventlet'), \
+                mock.patch('eventlet.hubs.use_hub'), \
+                mock.patch('eventlet.debug'):
+            d = daemon.run_daemon(MyDaemon, tempdir)
+        # my-daemon section takes priority (!?)
+        self.assertEqual('2', d.conf['client_timeout'])
+        self.assertEqual('10', d.conf['CONN_timeout'])
+        self.assertEqual('5', d.conf['conn_timeout'])
+
+    @with_tempdir
+    def test_run_daemon_from_conf_dir_with_duplicate_var(self, tempdir):
+        conf_files = {
+            'default': """
+            [DEFAULT]
+            client_timeout = 3
+            """,
+            'daemon': """
+            [my-daemon]
+            client_timeout = 2
+            CLIENT_TIMEOUT = 4
+            conn_timeout = 1.1
+            conn_timeout = 1.2
+            """,
+        }
+        for filename, conf_body in conf_files.items():
+            path = os.path.join(tempdir, filename + '.conf')
+            with open(path, 'wt') as fd:
+                fd.write(dedent(conf_body))
+        with mock.patch('swift.common.utils.eventlet'), \
+                mock.patch('eventlet.hubs.use_hub'), \
+                mock.patch('eventlet.debug'):
+            with self.assertRaises(
+                    configparser.DuplicateOptionError) as ctx:
+                daemon.run_daemon(MyDaemon, tempdir)
+            msg = str(ctx.exception)
+            self.assertIn('conn_timeout', msg)
+            self.assertIn('already exists', msg)
+
     @contextmanager
     def mock_os(self, child_worker_cycles=3):
         self.waitpid_calls = defaultdict(int)
@@ -228,6 +348,7 @@ class TestRunDaemon(unittest.TestCase):
             yield
 
     def test_fork_workers(self):
+        utils.logging_monkey_patch()  # needed to log at notice
         d = MyWorkerDaemon({'workers': 3})
         strategy = daemon.DaemonStrategy(d, d.logger)
         with self.mock_os():
