@@ -14,15 +14,17 @@
 # limitations under the License.
 
 """Tests for swift.common.request_helpers"""
-
+import argparse
 import unittest
-from swift.common.swob import Request, HTTPException, HeaderKeyDict
+from swift.common.swob import Request, HTTPException, HeaderKeyDict, HTTPOk
 from swift.common.storage_policy import POLICIES, EC_POLICY, REPL_POLICY
 from swift.common import request_helpers as rh
 from swift.common.constraints import AUTO_CREATE_ACCOUNT_PREFIX
 
+from test.debug_logger import debug_logger
 from test.unit import patch_policies
 from test.unit.common.test_utils import FakeResponse
+from test.unit.common.middleware.helpers import FakeSwift
 
 
 server_types = ['account', 'container', 'object']
@@ -30,6 +32,22 @@ server_types = ['account', 'container', 'object']
 
 class TestRequestHelpers(unittest.TestCase):
 
+    def test_append_log_info(self):
+        req = Request.blank('/v/a/c/o')
+        self.assertNotIn('swift.log_info', req.environ)
+        rh.append_log_info(req.environ, 'msg1')
+        self.assertEqual(['msg1'], req.environ.get('swift.log_info'))
+        rh.append_log_info(req.environ, 'msg2')
+        self.assertEqual(['msg1', 'msg2'], req.environ.get('swift.log_info'))
+
+    def test_get_log_info(self):
+        req = Request.blank('/v/a/c/o')
+        self.assertEqual('', rh.get_log_info(req.environ))
+        req.environ['swift.log_info'] = ['msg1']
+        self.assertEqual('msg1', rh.get_log_info(req.environ))
+        rh.append_log_info(req.environ, 'msg2')
+        self.assertEqual('msg1,msg2', rh.get_log_info(req.environ))
+
     def test_constrain_req_limit(self):
         req = Request.blank('')
         self.assertEqual(10, rh.constrain_req_limit(req, 10))
@@ -42,6 +60,70 @@ class TestRequestHelpers(unittest.TestCase):
             rh.constrain_req_limit(req, 10)
         self.assertEqual(raised.exception.status_int, 412)
 
+    def test_validate_params(self):
+        req = Request.blank('')
+        actual = rh.validate_params(req, ('limit', 'marker', 'end_marker'))
+        self.assertEqual({}, actual)
+
+        req = Request.blank('', query_string='limit=1&junk=here&marker=foo')
+        actual = rh.validate_params(req, ())
+        self.assertEqual({}, actual)
+
+        req = Request.blank('', query_string='limit=1&junk=here&marker=foo')
+        actual = rh.validate_params(req, ('limit', 'marker', 'end_marker'))
+        expected = {'limit': '1', 'marker': 'foo'}
+        self.assertEqual(expected, actual)
+
+        req = Request.blank('', query_string='limit=1&junk=here&marker=')
+        actual = rh.validate_params(req, ('limit', 'marker', 'end_marker'))
+        expected = {'limit': '1', 'marker': ''}
+        self.assertEqual(expected, actual)
+
+        # ignore bad junk
+        req = Request.blank('', query_string='limit=1&junk=%ff&marker=foo')
+        actual = rh.validate_params(req, ('limit', 'marker', 'end_marker'))
+        expected = {'limit': '1', 'marker': 'foo'}
+        self.assertEqual(expected, actual)
+
+        # error on bad wanted parameter
+        req = Request.blank('', query_string='limit=1&junk=here&marker=%ff')
+        with self.assertRaises(HTTPException) as raised:
+            rh.validate_params(req, ('limit', 'marker', 'end_marker'))
+        self.assertEqual(raised.exception.status_int, 400)
+
+    def test_validate_container_params(self):
+        req = Request.blank('')
+        actual = rh.validate_container_params(req)
+        self.assertEqual({'limit': 10000}, actual)
+
+        req = Request.blank('', query_string='limit=1&junk=here&marker=foo')
+        actual = rh.validate_container_params(req)
+        expected = {'limit': 1, 'marker': 'foo'}
+        self.assertEqual(expected, actual)
+
+        req = Request.blank('', query_string='limit=1&junk=here&marker=')
+        actual = rh.validate_container_params(req)
+        expected = {'limit': 1, 'marker': ''}
+        self.assertEqual(expected, actual)
+
+        # ignore bad junk
+        req = Request.blank('', query_string='limit=1&junk=%ff&marker=foo')
+        actual = rh.validate_container_params(req)
+        expected = {'limit': 1, 'marker': 'foo'}
+        self.assertEqual(expected, actual)
+
+        # error on bad wanted parameter
+        req = Request.blank('', query_string='limit=1&junk=here&marker=%ff')
+        with self.assertRaises(HTTPException) as raised:
+            rh.validate_container_params(req)
+        self.assertEqual(raised.exception.status_int, 400)
+
+        # error on bad limit
+        req = Request.blank('', query_string='limit=10001')
+        with self.assertRaises(HTTPException) as raised:
+            rh.validate_container_params(req)
+        self.assertEqual(raised.exception.status_int, 412)
+
     def test_is_user_meta(self):
         m_type = 'meta'
         for st in server_types:
@@ -124,6 +206,21 @@ class TestRequestHelpers(unittest.TestCase):
         self.assertFalse('c' in to_req.headers)
         self.assertFalse('C' in to_req.headers)
 
+    def test_is_use_replication_network(self):
+        self.assertFalse(rh.is_use_replication_network())
+        self.assertFalse(rh.is_use_replication_network({}))
+        self.assertFalse(rh.is_use_replication_network(
+            {'x-backend-use-replication-network': 'false'}))
+        self.assertFalse(rh.is_use_replication_network(
+            {'x-backend-use-replication-network': 'no'}))
+
+        self.assertTrue(rh.is_use_replication_network(
+            {'x-backend-use-replication-network': 'true'}))
+        self.assertTrue(rh.is_use_replication_network(
+            {'x-backend-use-replication-network': 'yes'}))
+        self.assertTrue(rh.is_use_replication_network(
+            {'X-Backend-Use-Replication-Network': 'True'}))
+
     def test_get_ip_port(self):
         node = {
             'ip': '1.2.3.4',
@@ -137,6 +234,17 @@ class TestRequestHelpers(unittest.TestCase):
         self.assertEqual(('1.2.3.4', 6000), rh.get_ip_port(node, {
             rh.USE_REPLICATION_NETWORK_HEADER: 'false'}))
 
+        # node trumps absent header and False header
+        node['use_replication'] = True
+        self.assertEqual(('5.6.7.8', 7000), rh.get_ip_port(node, {}))
+        self.assertEqual(('5.6.7.8', 7000), rh.get_ip_port(node, {
+            rh.USE_REPLICATION_NETWORK_HEADER: 'false'}))
+
+        # True header trumps node
+        node['use_replication'] = False
+        self.assertEqual(('5.6.7.8', 7000), rh.get_ip_port(node, {
+            rh.USE_REPLICATION_NETWORK_HEADER: 'true'}))
+
     @patch_policies(with_ec_default=True)
     def test_get_name_and_placement_object_req(self):
         path = '/device/part/account/container/object'
@@ -381,6 +489,46 @@ class TestRequestHelpers(unittest.TestCase):
         self.assertEqual(str(ctx.exception),
                          'Invalid reserved name')
 
+    def test_is_open_expired(self):
+        app = argparse.Namespace(allow_open_expired=False)
+        req = Request.blank('/v1/a/c/o', headers={'X-Open-Expired': 'yes'})
+        self.assertFalse(rh.is_open_expired(app, req))
+        req = Request.blank('/v1/a/c/o', headers={'X-Open-Expired': 'no'})
+        self.assertFalse(rh.is_open_expired(app, req))
+        req = Request.blank('/v1/a/c/o', headers={})
+        self.assertFalse(rh.is_open_expired(app, req))
+
+        app = argparse.Namespace(allow_open_expired=True)
+        req = Request.blank('/v1/a/c/o', headers={'X-Open-Expired': 'no'})
+        self.assertFalse(rh.is_open_expired(app, req))
+        req = Request.blank('/v1/a/c/o', headers={})
+        self.assertFalse(rh.is_open_expired(app, req))
+
+        req = Request.blank('/v1/a/c/o', headers={'X-Open-Expired': 'yes'})
+        self.assertTrue(rh.is_open_expired(app, req))
+
+    def test_is_backend_open_expired(self):
+        req = Request.blank('/v1/a/c/o', headers={
+            'X-Backend-Open-Expired': 'yes'
+        })
+        self.assertTrue(rh.is_backend_open_expired(req))
+        req = Request.blank('/v1/a/c/o', headers={
+            'X-Backend-Open-Expired': 'no'
+        })
+        self.assertFalse(rh.is_backend_open_expired(req))
+
+        req = Request.blank('/v1/a/c/o', headers={
+            'X-Backend-Replication': 'yes'
+        })
+        self.assertTrue(rh.is_backend_open_expired(req))
+        req = Request.blank('/v1/a/c/o', headers={
+            'X-Backend-Replication': 'no'
+        })
+        self.assertFalse(rh.is_backend_open_expired(req))
+
+        req = Request.blank('/v1/a/c/o', headers={})
+        self.assertFalse(rh.is_backend_open_expired(req))
+
 
 class TestHTTPResponseToDocumentIters(unittest.TestCase):
     def test_200(self):
@@ -563,3 +711,107 @@ class TestHTTPResponseToDocumentIters(unittest.TestCase):
         do_test()
         metadata = dict((k.upper(), v) for k, v in metadata.items())
         do_test()
+
+    def test_ignore_range_header(self):
+        req = Request.blank('/v/a/c/o')
+        self.assertIsNone(req.headers.get(
+            'X-Backend-Ignore-Range-If-Metadata-Present'))
+        rh.update_ignore_range_header(req, 'X-Static-Large-Object')
+        self.assertEqual('X-Static-Large-Object', req.headers.get(
+            'X-Backend-Ignore-Range-If-Metadata-Present'))
+        rh.update_ignore_range_header(req, 'X-Static-Large-Object')
+        self.assertEqual(
+            'X-Static-Large-Object,X-Static-Large-Object',
+            req.headers.get('X-Backend-Ignore-Range-If-Metadata-Present'))
+        rh.update_ignore_range_header(req, 'X-Object-Sysmeta-Slo-Etag')
+        self.assertEqual(
+            'X-Static-Large-Object,X-Static-Large-Object,'
+            'X-Object-Sysmeta-Slo-Etag',
+            req.headers.get('X-Backend-Ignore-Range-If-Metadata-Present'))
+
+    def test_resolove_ignore_range_header(self):
+        # no ignore header is no-op
+        req = Request.blank('/v/a/c/o', headers={'Range': 'bytes=0-4'})
+        self.assertEqual(str(req.range), 'bytes=0-4')
+        rh.resolve_ignore_range_header(req, {
+            'X-Static-Large-Object': True,
+            'X-Object-Meta-Color': 'blue',
+        })
+        self.assertEqual(str(req.range), 'bytes=0-4')
+
+        # missing matching metadata is no-op
+        rh.update_ignore_range_header(req, 'X-Static-Large-Object')
+        rh.resolve_ignore_range_header(req, {
+            'X-Object-Meta-Color': 'blue',
+        })
+        self.assertEqual(str(req.range), 'bytes=0-4')
+
+        # matching metadata pops range
+        rh.resolve_ignore_range_header(req, {
+            'X-Static-Large-Object': True,
+            'X-Object-Meta-Color': 'blue',
+        })
+        self.assertIsNone(req.range)
+
+    def test_multiple_resolove_ignore_range_header(self):
+        req = Request.blank('/v/a/c/o', headers={'Range': 'bytes=0-4'})
+        rh.update_ignore_range_header(req, 'X-Static-Large-Object')
+        rh.update_ignore_range_header(req, 'X-Object-Sysmeta-Slo-Etag')
+        rh.resolve_ignore_range_header(req, {
+            'X-Static-Large-Object': True,
+            'X-Object-Meta-Color': 'blue',
+        })
+        self.assertIsNone(req.range)
+
+
+class TestSegmentedIterable(unittest.TestCase):
+
+    def setUp(self):
+        self.logger = debug_logger()
+        self.app = FakeSwift()
+        self.expected_unread_requests = {}
+
+    def tearDown(self):
+        self.assertFalse(self.app.unclosed_requests)
+        self.assertEqual(self.app.unread_requests,
+                         self.expected_unread_requests)
+
+    def test_simple_segments_app_iter(self):
+        self.app.register('GET', '/a/c/seg1', HTTPOk, {}, 'segment1')
+        self.app.register('GET', '/a/c/seg2', HTTPOk, {}, 'segment2')
+        req = Request.blank('/v1/a/c/mpu')
+        listing_iter = [
+            {'path': '/a/c/seg1', 'first_byte': None, 'last_byte': None},
+            {'path': '/a/c/seg2', 'first_byte': None, 'last_byte': None},
+        ]
+        si = rh.SegmentedIterable(req, self.app, listing_iter, 60, self.logger,
+                                  'test-agent', 'test-source')
+        body = b''.join(si.app_iter)
+        self.assertEqual(b'segment1segment2', body)
+
+    def test_simple_segments_app_iter_ranges(self):
+        self.app.register('GET', '/a/c/seg1', HTTPOk, {}, 'segment1')
+        self.app.register('GET', '/a/c/seg2', HTTPOk, {}, 'segment2')
+        req = Request.blank('/v1/a/c/mpu')
+        listing_iter = [
+            {'path': '/a/c/seg1', 'first_byte': None, 'last_byte': None},
+            {'path': '/a/c/seg2', 'first_byte': None, 'last_byte': None},
+        ]
+        si = rh.SegmentedIterable(req, self.app, listing_iter, 60, self.logger,
+                                  'test-agent', 'test-source')
+        body = b''.join(si.app_iter_ranges(
+            [(0, 8), (8, 16)], b'app/foo', b'bound', 16))
+        expected = b'\r\n'.join([
+            b'--bound',
+            b'Content-Type: app/foo',
+            b'Content-Range: bytes 0-7/16',
+            b'',
+            b'segment1',
+            b'--bound',
+            b'Content-Type: app/foo',
+            b'Content-Range: bytes 8-15/16',
+            b'',
+            b'segment2',
+            b'--bound--',
+        ])
+        self.assertEqual(expected, body)
