@@ -22,12 +22,14 @@ import time
 
 from io import BytesIO
 
-import six
-from six.moves.urllib.parse import quote
+from urllib.parse import quote
+from unittest import mock
 
 import swift.common.swob as swob
 from swift.common import utils, exceptions
 
+from test.unit.common.middleware.helpers import LeakTrackingIter
+
 
 class TestHeaderEnvironProxy(unittest.TestCase):
     def test_proxy(self):
@@ -354,10 +356,7 @@ class TestEtag(unittest.TestCase):
 
     def test_normalize_bytes(self):
         some_etag = b'"some-etag"'
-        if six.PY2:
-            self.assertEqual('some-etag', swob.normalize_etag(some_etag))
-        else:
-            self.assertRaises(TypeError, swob.normalize_etag, some_etag)
+        self.assertRaises(TypeError, swob.normalize_etag, some_etag)
 
 
 class TestTransferEncoding(unittest.TestCase):
@@ -789,14 +788,8 @@ class TestRequest(unittest.TestCase):
 
         hacker = 'account-name\n\n<b>foo<br>'  # url injection test
         quoted_hacker = quote(hacker)
-        req = swob.Request.blank('/v1/' + hacker)
-        resp = req.get_response(test_app)
-        self.assertEqual(resp.status_int, 401)
-        self.assertTrue('Www-Authenticate' in resp.headers)
-        self.assertEqual('Swift realm="%s"' % quoted_hacker,
-                         resp.headers['Www-Authenticate'])
-
         req = swob.Request.blank('/v1/' + quoted_hacker)
+        self.assertIn(hacker, req.environ['PATH_INFO'])  # sanity check
         resp = req.get_response(test_app)
         self.assertEqual(resp.status_int, 401)
         self.assertTrue('Www-Authenticate' in resp.headers)
@@ -838,7 +831,7 @@ class TestRequest(unittest.TestCase):
         req = swob.Request.blank('/hi/there', body='hi')
 
         req.if_unmodified_since = 0
-        self.assertTrue(isinstance(req.if_unmodified_since, datetime.datetime))
+        self.assertIsInstance(req.if_unmodified_since, datetime.datetime)
         if_unmodified_since = req.if_unmodified_since
         req.if_unmodified_since = if_unmodified_since
         self.assertEqual(if_unmodified_since, req.if_unmodified_since)
@@ -974,11 +967,11 @@ class TestRequest(unittest.TestCase):
         self.assertEqual(_test_split_path('/a/c/', 2), ['a', 'c'])
         self.assertEqual(_test_split_path('/a/c/', 2, 3), ['a', 'c', ''])
         try:
-            _test_split_path('o\nn e', 2)
+            _test_split_path('o%0an e', 2)
         except ValueError as err:
             self.assertEqual(str(err), 'Invalid path: o%0An%20e')
         try:
-            _test_split_path('o\nn e', 2, 3, True)
+            _test_split_path('o%0an e', 2, 3, True)
         except ValueError as err:
             self.assertEqual(str(err), 'Invalid path: o%0An%20e')
 
@@ -993,66 +986,44 @@ class TestRequest(unittest.TestCase):
         self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
         self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
 
-        if six.PY2:
-            # Unicode is encoded to UTF-8 on py2, to paper over deserialized
-            # JSON slipping into subrequests
+        # Arbitrary Unicode *is not* supported on py3 -- only latin-1
+        # encodable is supported, because PEP-3333.
+        with self.assertRaises(UnicodeEncodeError):
             req = swob.Request.blank(u'/\u2661')
-            self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
-            self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
 
-            req = swob.Request.blank('/')
+        req = swob.Request.blank('/')
+        with self.assertRaises(UnicodeEncodeError):
             req.path_info = u'/\u2661'
-            self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
-            self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
+        # Update didn't take
+        self.assertEqual(req.path, '/')
+        self.assertEqual(req.environ['PATH_INFO'], '/')
 
-        else:
-            # Arbitrary Unicode *is not* supported on py3 -- only latin-1
-            # encodable is supported, because PEP-3333.
-            with self.assertRaises(UnicodeEncodeError):
-                req = swob.Request.blank(u'/\u2661')
+        # Needs to be a "WSGI string"
+        req = swob.Request.blank('/\xe2\x99\xa1')
+        self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
+        self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
 
-            req = swob.Request.blank('/')
-            with self.assertRaises(UnicodeEncodeError):
-                req.path_info = u'/\u2661'
-            # Update didn't take
-            self.assertEqual(req.path, '/')
-            self.assertEqual(req.environ['PATH_INFO'], '/')
-
-            # Needs to be a "WSGI string"
-            req = swob.Request.blank('/\xe2\x99\xa1')
-            self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
-            self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
-
-            req = swob.Request.blank('/')
-            req.path_info = '/\xe2\x99\xa1'
-            self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
-            self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
+        req = swob.Request.blank('/')
+        req.path_info = '/\xe2\x99\xa1'
+        self.assertEqual(req.path, quote(u'/\u2661'.encode('utf-8')))
+        self.assertEqual(req.environ['PATH_INFO'], '/\xe2\x99\xa1')
 
     def test_unicode_query(self):
         # Bytes are always OK
         req = swob.Request.blank('/')
         encoded = u'\u2661'.encode('utf-8')
         req.query_string = b'x=' + encoded
-        if six.PY2:
-            self.assertEqual(req.params['x'], encoded)
-        else:
-            self.assertEqual(req.params['x'], encoded.decode('latin1'))
+        self.assertEqual(req.params['x'], encoded.decode('latin1'))
 
-        if six.PY2:
-            # Unicode will be UTF-8-encoded on py2
-            req = swob.Request.blank('/')
+        # Note that py3 requires "WSGI strings"
+        req = swob.Request.blank('/')
+        with self.assertRaises(UnicodeEncodeError):
             req.query_string = u'x=\u2661'
-            self.assertEqual(req.params['x'], encoded)
-        else:
-            # ...but py3 requires "WSGI strings"
-            req = swob.Request.blank('/')
-            with self.assertRaises(UnicodeEncodeError):
-                req.query_string = u'x=\u2661'
-            self.assertEqual(req.params, {})
+        self.assertEqual(req.params, {})
 
-            req = swob.Request.blank('/')
-            req.query_string = 'x=' + encoded.decode('latin-1')
-            self.assertEqual(req.params['x'], encoded.decode('latin-1'))
+        req = swob.Request.blank('/')
+        req.query_string = 'x=' + encoded.decode('latin-1')
+        self.assertEqual(req.params['x'], encoded.decode('latin-1'))
 
     def test_url2(self):
         pi = '/hi/there'
@@ -1238,16 +1209,100 @@ class TestResponse(unittest.TestCase):
         The Response object's __call__ method should be able to reify a
         Request object from the env it gets passed.
         """
+        tracking = {
+            'closed': 0,
+            'read': 0,
+        }
+
+        def mark_closed(*args):
+            tracking['closed'] += 1
+
+        def mark_read(*args):
+            tracking['read'] += 1
+
         def test_app(environ, start_response):
             start_response('200 OK', [])
-            return [b'hi']
+            body = [b'hi']
+            return LeakTrackingIter(body, mark_closed, mark_read, None)
         req = swob.Request.blank('/')
         req.method = 'HEAD'
         status, headers, app_iter = req.call_application(test_app)
         resp = swob.Response(status=status, headers=dict(headers),
                              app_iter=app_iter)
         output_iter = resp(req.environ, lambda *_: None)
-        self.assertEqual(list(output_iter), [b''])
+        with utils.closing_if_possible(output_iter):
+            body = b''.join(output_iter)
+        self.assertEqual(body, b'')
+        self.assertEqual(tracking, {
+            'closed': 1,
+            'read': 1,
+        })
+
+    def test_swob_drains_small_HEAD_resp_iter(self):
+        tracking = {
+            'closed': 0,
+            'read': 0,
+        }
+
+        def mark_closed(*args):
+            tracking['closed'] += 1
+
+        def mark_read(*args):
+            tracking['read'] += 1
+
+        def test_app(environ, start_response):
+            start_response('200 OK', [])
+            body = [b'hello', b'world']
+            return LeakTrackingIter(body, mark_closed, mark_read, None)
+
+        req = swob.Request.blank('/', method='HEAD')
+        status, headers, app_iter = req.call_application(test_app)
+        resp = swob.Response(status=status, headers=dict(headers),
+                             app_iter=app_iter)
+        # sanity, swob drains small HEAD responses
+        output_iter = resp(req.environ, lambda *_: None)
+        with utils.closing_if_possible(output_iter):
+            # regardless what the app returns swob's HEAD response is empty
+            body = b''.join(output_iter)
+        self.assertEqual(body, b'')
+        self.assertEqual(tracking, {
+            'closed': 1,
+            'read': 1,
+        })
+
+    def test_swob_closes_large_HEAD_resp_iter(self):
+        tracking = {
+            'closed': 0,
+            'read': 0,
+        }
+
+        def mark_closed(*args):
+            tracking['closed'] += 1
+
+        def mark_read(*args):
+            tracking['read'] += 1
+
+        def test_app(environ, start_response):
+            start_response('200 OK', [])
+            body = [b'hello', b'world']
+            return LeakTrackingIter(body, mark_closed, mark_read, None)
+
+        req = swob.Request.blank('/', method='HEAD')
+        status, headers, app_iter = req.call_application(test_app)
+        resp = swob.Response(status=status, headers=dict(headers),
+                             app_iter=app_iter)
+        # N.B. if we call next a third time (i.e. len(helloworld) < read_limit)
+        # then leak tracker will notice StopIteration and count it drained.
+        with mock.patch.object(utils, 'DEFAULT_DRAIN_LIMIT', 10):
+            output_iter = resp(req.environ, lambda *_: None)
+        with utils.closing_if_possible(output_iter):
+            # regardless what the app returns swob's HEAD response is empty
+            body = b''.join(output_iter)
+        self.assertEqual(body, b'')
+        self.assertEqual(tracking, {
+            'closed': 1,
+            'read': 0,
+        })
 
     def test_call_preserves_closeability(self):
         def test_app(environ, start_response):
