@@ -19,11 +19,11 @@ import tempfile
 from textwrap import dedent
 import unittest
 
-import mock
+from unittest import mock
 
 from swift.common.utils import config
 
-from six import StringIO
+from io import StringIO
 from test import annotate_failure
 from test.unit import temptree
 
