@@ -12,11 +12,11 @@
 # limitations under the License.
 
 import json
-import numbers
 import shutil
+from functools import partial
 from tempfile import mkdtemp
 
-import mock
+from unittest import mock
 import operator
 import time
 import unittest
@@ -29,8 +29,7 @@ import eventlet
 
 from collections import defaultdict
 from datetime import datetime
-import six
-from six.moves import urllib
+import urllib.parse
 from swift.common.storage_policy import StoragePolicy, ECStoragePolicy
 from swift.common.swob import Request
 
@@ -39,7 +38,8 @@ from swift.container.server import gen_resp_headers, ContainerController
 from swift.common.direct_client import ClientException
 from swift.common import swob
 from swift.common.header_key_dict import HeaderKeyDict
-from swift.common.utils import split_path, Timestamp, encode_timestamps, mkdirs
+from swift.common.utils import split_path, Timestamp, encode_timestamps, \
+    mkdirs, UTC
 
 from test.debug_logger import debug_logger
 from test.unit import FakeRing, fake_http_connect, patch_policies, \
@@ -48,8 +48,8 @@ from test.unit.common.middleware import helpers
 
 
 def timestamp_to_last_modified(timestamp):
-    return datetime.utcfromtimestamp(
-        float(Timestamp(timestamp))).strftime('%Y-%m-%dT%H:%M:%S.%f')
+    dt = datetime.fromtimestamp(float(Timestamp(timestamp)), UTC)
+    return dt.strftime('%Y-%m-%dT%H:%M:%S.%f')
 
 
 def container_resp_headers(**kwargs):
@@ -59,7 +59,8 @@ def container_resp_headers(**kwargs):
 class FakeStoragePolicySwift(object):
 
     def __init__(self):
-        self.storage_policy = defaultdict(helpers.FakeSwift)
+        self.storage_policy = defaultdict(
+            partial(helpers.FakeSwift, capture_unexpected_calls=False))
         self._mock_oldest_spi_map = {}
 
     def __getattribute__(self, name):
@@ -117,8 +118,6 @@ class FakeInternalClient(reconciler.InternalClient):
             else:
                 timestamp, content_type = timestamp, 'application/x-put'
             storage_policy_index, path = item
-            if six.PY2 and isinstance(path, six.text_type):
-                path = path.encode('utf-8')
             account, container_name, obj_name = split_path(
                 path, 0, 3, rest_with_last=True)
             self.accounts[account][container_name].append(
@@ -140,6 +139,8 @@ class FakeInternalClient(reconciler.InternalClient):
                         continue
                     obj_path = swob.str_to_wsgi(
                         container_path + '/' + obj_name)
+                    # some tests setup mock listings using floats, some use
+                    # strings, so normalize here
                     ts = Timestamp(timestamp)
                     headers = {'X-Timestamp': ts.normal,
                                'X-Backend-Timestamp': ts.internal}
@@ -149,20 +150,12 @@ class FakeInternalClient(reconciler.InternalClient):
                     self.app.storage_policy[storage_policy_index].register(
                         'DELETE', obj_path, swob.HTTPNoContent, {})
                     # container listing entry
-                    last_modified = timestamp_to_last_modified(timestamp)
-                    # some tests setup mock listings using floats, some use
-                    # strings, so normalize here
-                    if isinstance(timestamp, numbers.Number):
-                        timestamp = '%f' % timestamp
-                    if six.PY2:
-                        obj_name = obj_name.decode('utf-8')
-                        timestamp = timestamp.decode('utf-8')
                     obj_data = {
                         'bytes': 0,
                         # listing data is unicode
                         'name': obj_name,
-                        'last_modified': last_modified,
-                        'hash': timestamp,
+                        'last_modified': ts.isoformat,
+                        'hash': ts.internal,
                         'content_type': content_type,
                     }
                     container_listing_data.append(obj_data)
@@ -944,8 +937,8 @@ class TestReconciler(unittest.TestCase):
             with mock.patch('time.time', lambda: next(mock_time_iter)):
                 self.reconciler.run_once()
 
-        return [c[1][1:4] for c in
-                mocks['direct_delete_container_entry'].mock_calls]
+        return [tuple(list(c[1][1:4]) + [c[2]['headers']['X-Timestamp']])
+                for c in mocks['direct_delete_container_entry'].mock_calls]
 
     def test_no_concurrency(self):
         self._mock_listing({
@@ -968,8 +961,10 @@ class TestReconciler(unittest.TestCase):
         self.assertEqual(order_recieved, ['o1', 'o2'])
         # process in order recieved
         self.assertEqual(deleted_container_entries, [
-            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1'),
-            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o2'),
+            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+             Timestamp(3618.84187, offset=2).internal),
+            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o2',
+             Timestamp(3724.23456, offset=2).internal)
         ])
 
     def test_concurrency(self):
@@ -997,8 +992,10 @@ class TestReconciler(unittest.TestCase):
         self.assertEqual(order_recieved, ['o1', 'o2'])
         # ... and so we finish o2 first
         self.assertEqual(deleted_container_entries, [
-            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o2'),
-            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1'),
+            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o2',
+             Timestamp(3724.23456, offset=2).internal),
+            ('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+             Timestamp(3618.84187, offset=2).internal),
         ])
 
     def test_multi_process_should_process(self):
@@ -1097,7 +1094,8 @@ class TestReconciler(unittest.TestCase):
         # so pop the queue for that one
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(3618.84187, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_queue_name_with_policy_index_delimiter_in_name(self):
@@ -1142,7 +1140,8 @@ class TestReconciler(unittest.TestCase):
         # and pop the queue for that one
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries, [(
-            '.misplaced_objects', '3600', '1:/%s' % obj_path)])
+            '.misplaced_objects', '3600', '1:/%s' % obj_path,
+            Timestamp(3618.84187, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_unable_to_direct_get_oldest_storage_policy(self):
@@ -1240,9 +1239,9 @@ class TestReconciler(unittest.TestCase):
             [('HEAD', '/v1/AUTH_bob/c/o1'),
              ('PUT', '/v1/AUTH_bob/c/o1')])
         put_headers = self.fake_swift.storage_policy[0].headers[1]
-        # we PUT the object in the right place with q_ts + offset 2
+        # we PUT the object in the right place with q_ts + offset 3
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(3618.84187, offset=2))
+                         Timestamp(3618.84187, offset=3))
         # cleans up the old
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         self.assertEqual(self.reconciler.stats['cleanup_success'], 1)
@@ -1253,7 +1252,8 @@ class TestReconciler(unittest.TestCase):
         # and when we're done, we pop the entry from the queue
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(3618.84187, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_the_other_direction(self):
@@ -1287,9 +1287,9 @@ class TestReconciler(unittest.TestCase):
             [('HEAD', '/v1/AUTH_bob/c/o1'),  # 1
              ('PUT', '/v1/AUTH_bob/c/o1')])  # 3
         put_headers = self.fake_swift.storage_policy[1].headers[1]
-        # we PUT the object in the right place with q_ts + offset 2
+        # we PUT the object in the right place with q_ts + offset 3
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(3618.84187, offset=2).internal)
+                         Timestamp(3618.84187, offset=3).internal)
         # cleans up the old
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         self.assertEqual(self.reconciler.stats['cleanup_success'], 1)
@@ -1300,7 +1300,8 @@ class TestReconciler(unittest.TestCase):
         # and when we're done, we pop the entry from the queue
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '0:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '0:/AUTH_bob/c/o1',
+                           Timestamp(3618.84187, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_with_unicode_and_spaces(self):
@@ -1308,10 +1309,7 @@ class TestReconciler(unittest.TestCase):
         # functions where we call them with (account, container, obj)
         obj_name = u"AUTH_bob/c \u062a/o1 \u062a"
         # anytime we talk about a call made to swift for a path
-        if six.PY2:
-            obj_path = obj_name.encode('utf-8')
-        else:
-            obj_path = obj_name.encode('utf-8').decode('latin-1')
+        obj_path = obj_name.encode('utf-8').decode('latin-1')
         # this mock expects unquoted unicode because it handles container
         # listings as well as paths
         self._mock_listing({
@@ -1346,9 +1344,9 @@ class TestReconciler(unittest.TestCase):
             [('HEAD', '/v1/%s' % obj_path),  # 1
              ('PUT', '/v1/%s' % obj_path)])  # 3
         put_headers = self.fake_swift.storage_policy[0].headers[1]
-        # we PUT the object in the right place with q_ts + offset 2
+        # we PUT the object in the right place with q_ts + offset 3
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(3618.84187, offset=2).internal)
+                         Timestamp(3618.84187, offset=3).internal)
         # cleans up the old
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         self.assertEqual(self.reconciler.stats['cleanup_success'], 1)
@@ -1362,7 +1360,8 @@ class TestReconciler(unittest.TestCase):
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         # this mock received the name, it's encoded down in buffered_http
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/%s' % obj_name)])
+                         [('.misplaced_objects', '3600', '1:/%s' % obj_name,
+                           Timestamp(3618.84187, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_delete(self):
@@ -1403,9 +1402,9 @@ class TestReconciler(unittest.TestCase):
             [('HEAD', '/v1/AUTH_bob/c/o1'),
              ('DELETE', '/v1/AUTH_bob/c/o1')])
         reconcile_headers = self.fake_swift.storage_policy[0].headers[1]
-        # we DELETE the object in the right place with q_ts + offset 2
+        # we DELETE the object in the right place with q_ts + offset 3
         self.assertEqual(reconcile_headers.get('X-Timestamp'),
-                         Timestamp(q_ts, offset=2).internal)
+                         Timestamp(q_ts, offset=3).internal)
         # cleans up the old
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         self.assertEqual(self.reconciler.stats['cleanup_success'], 1)
@@ -1416,7 +1415,8 @@ class TestReconciler(unittest.TestCase):
         # and when we're done, we pop the entry from the queue
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(q_ts, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_enqueued_for_the_correct_dest_noop(self):
@@ -1440,7 +1440,8 @@ class TestReconciler(unittest.TestCase):
         # so we just pop the queue
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(3618.84187, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_src_object_newer_than_queue_entry(self):
@@ -1475,10 +1476,10 @@ class TestReconciler(unittest.TestCase):
             self.fake_swift.storage_policy[0].calls,
             [('HEAD', '/v1/AUTH_bob/c/o1'),  # 1
              ('PUT', '/v1/AUTH_bob/c/o1')])  # 3
-        # .. with source timestamp + offset 2
+        # .. with source timestamp + offset 3
         put_headers = self.fake_swift.storage_policy[0].headers[1]
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(3600.234567, offset=2))
+                         Timestamp(3600.234567, offset=3))
         # src object is cleaned up
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         self.assertEqual(self.reconciler.stats['cleanup_success'], 1)
@@ -1488,7 +1489,8 @@ class TestReconciler(unittest.TestCase):
         # and queue is popped
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(3600.123456, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_src_object_older_than_queue_entry(self):
@@ -1539,7 +1541,7 @@ class TestReconciler(unittest.TestCase):
         self._mock_oldest_spi({'c': 0})
         self.fake_swift.storage_policy[1].register(
             'GET', '/v1/AUTH_bob/c/o1', swob.HTTPNotFound,
-            {'X-Backend-Timestamp': Timestamp(q_ts, offset=2).internal})
+            {'X-Backend-Timestamp': Timestamp(q_ts, offset=3).internal})
         deleted_container_entries = self._run_once()
 
         # found a misplaced object
@@ -1679,10 +1681,10 @@ class TestReconciler(unittest.TestCase):
             self.fake_swift.storage_policy[0].calls,
             [('HEAD', '/v1/AUTH_bob/c/o1'),  # 1
              ('PUT', '/v1/AUTH_bob/c/o1')])  # 3
-        # .. with source timestamp + offset 2
+        # .. with source timestamp + offset 3
         put_headers = self.fake_swift.storage_policy[0].headers[1]
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(3600.123457, offset=2))
+                         Timestamp(3600.123457, offset=3))
         # we try to cleanup
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         # ... with q_ts + offset 1
@@ -1728,7 +1730,8 @@ class TestReconciler(unittest.TestCase):
         # so... we're done
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(q_ts, offset=2).internal)])
         # dunno if this is helpful, but FWIW we don't throw tombstones?
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 0)
         self.assertEqual(self.reconciler.stats['success'], 1)  # lol
@@ -1769,7 +1772,8 @@ class TestReconciler(unittest.TestCase):
         # and wipe our hands of it
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(3679.2019, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_dest_object_newer_than_queue_entry(self):
@@ -1810,7 +1814,8 @@ class TestReconciler(unittest.TestCase):
         # and since we cleaned up the old object, so this counts as done
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '3600', '1:/AUTH_bob/c/o1',
+                           Timestamp(3679.2019, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_dest_object_older_than_queue_entry(self):
@@ -1844,10 +1849,10 @@ class TestReconciler(unittest.TestCase):
             self.fake_swift.storage_policy[0].calls,
             [('HEAD', '/v1/AUTH_bob/c/o1'),  # 1
              ('PUT', '/v1/AUTH_bob/c/o1')])  # 3
-        # ... with a q_ts + offset 2
+        # ... with a q_ts + offset 3
         put_headers = self.fake_swift.storage_policy[0].headers[1]
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(36123.38393, offset=2))
+                         Timestamp(36123.38393, offset=3))
         # then clean the dark matter
         self.assertEqual(self.reconciler.stats['cleanup_attempt'], 1)
         self.assertEqual(self.reconciler.stats['cleanup_success'], 1)
@@ -1858,7 +1863,8 @@ class TestReconciler(unittest.TestCase):
         # and pop the queue
         self.assertEqual(self.reconciler.stats['pop_queue'], 1)
         self.assertEqual(deleted_container_entries,
-                         [('.misplaced_objects', '36000', '1:/AUTH_bob/c/o1')])
+                         [('.misplaced_objects', '36000', '1:/AUTH_bob/c/o1',
+                           Timestamp(36123.38393, offset=2).internal)])
         self.assertEqual(self.reconciler.stats['success'], 1)
 
     def test_object_move_put_fails(self):
@@ -1896,9 +1902,9 @@ class TestReconciler(unittest.TestCase):
             [('HEAD', '/v1/AUTH_bob/c/o1'),  # 1
              ('PUT', '/v1/AUTH_bob/c/o1')])  # 3
         put_headers = self.fake_swift.storage_policy[0].headers[1]
-        # ...with q_ts + offset 2 (20-microseconds)
+        # ...with q_ts + offset 3 (20-microseconds)
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(36123.383925, offset=2))
+                         Timestamp(36123.383925, offset=3))
         # but it failed
         self.assertEqual(self.reconciler.stats['copy_success'], 0)
         self.assertEqual(self.reconciler.stats['copy_failed'], 1)
@@ -1947,9 +1953,9 @@ class TestReconciler(unittest.TestCase):
             [('HEAD', '/v1/AUTH_bob/c/o1'),  # 1
              ('PUT', '/v1/AUTH_bob/c/o1')])  # 3
         put_headers = self.fake_swift.storage_policy[0].headers[1]
-        # ...with q_ts + offset 2 (20-microseconds)
+        # ...with q_ts + offset 3 (20-microseconds)
         self.assertEqual(put_headers.get('X-Timestamp'),
-                         Timestamp(36123.383925, offset=2))
+                         Timestamp(36123.383925, offset=3))
         # but it blows up hard
         self.assertEqual(self.reconciler.stats['unhandled_error'], 1)
         # so we don't cleanup
@@ -2026,7 +2032,8 @@ class TestReconciler(unittest.TestCase):
         # probably been reaped, so we'll just give up
         self.assertEqual(
             deleted_container_entries,
-            [('.misplaced_objects', container, '1:/AUTH_jeb/c/o1')])
+            [('.misplaced_objects', container, '1:/AUTH_jeb/c/o1',
+              Timestamp(queue_ts, offset=2).internal)])
 
     def test_delete_old_empty_queue_containers(self):
         ts = time.time() - self.reconciler.reclaim_age * 1.1
