@@ -15,9 +15,8 @@
 
 import unittest
 import json
-import mock
+from unittest import mock
 import os
-import pkg_resources
 import signal
 import string
 import sys
@@ -27,6 +26,8 @@ from shutil import rmtree
 from tempfile import mkdtemp
 import textwrap
 from os.path import dirname, basename
+
+from test import BaseTestCase
 from test.debug_logger import debug_logger
 from test.unit import (
     DEFAULT_TEST_EC_TYPE, make_timestamp_iter, patch_policies,
@@ -40,7 +41,8 @@ from swift.obj.diskfile import (
     get_auditor_status, HASH_FILE, HASH_INVALIDATIONS_FILE)
 from swift.common.exceptions import ClientException
 from swift.common.utils import (
-    mkdirs, normalize_timestamp, Timestamp, readconf, md5, PrefixLoggerAdapter)
+    mkdirs, normalize_timestamp, Timestamp, readconf, md5)
+from swift.common.utils.logs import SwiftLogAdapter
 from swift.common.storage_policy import (
     ECStoragePolicy, StoragePolicy, POLICIES, EC_POLICY)
 
@@ -65,13 +67,6 @@ def works_only_once(callable_thing, exception):
     return only_once
 
 
-def no_audit_watchers(group, name=None):
-    if group == 'swift.object_audit_watcher':
-        return iter([])
-    else:
-        return pkg_resources.iter_entry_points(group, name)
-
-
 class FakeRing1(object):
 
     def __init__(self, swift_dir, ring_name=None):
@@ -111,7 +106,7 @@ class FakeRing2(object):
         return (1, nodes)
 
 
-class TestAuditorBase(unittest.TestCase):
+class TestAuditorBase(BaseTestCase):
 
     def setUp(self):
         skip_if_no_xattrs()
@@ -224,6 +219,11 @@ class TestAuditor(TestAuditorBase):
                     'X-Timestamp': timestamp,
                     'Content-Length': str(os.fstat(writer._fd).st_size),
                 }
+                if disk_file.policy.policy_type == EC_POLICY:
+                    metadata.update({
+                        'X-Object-Sysmeta-Ec-Frag-Index': '1',
+                        'X-Object-Sysmeta-Ec-Etag': 'fake-etag',
+                    })
                 writer.put(metadata)
                 writer.commit(Timestamp(timestamp))
                 pre_quarantines = auditor_worker.quarantines
@@ -368,6 +368,8 @@ class TestAuditor(TestAuditorBase):
                     'ETag': etag,
                     'X-Timestamp': timestamp,
                     'Content-Length': len(data),
+                    'X-Object-Sysmeta-Ec-Frag-Index': '1',
+                    'X-Object-Sysmeta-Ec-Etag': 'fake-etag',
                 }
                 writer.put(metadata)
                 writer.commit(Timestamp(timestamp))
@@ -908,7 +910,7 @@ class TestAuditor(TestAuditorBase):
         kwargs['zero_byte_fps'] = 50
         self.auditor.run_audit(**kwargs)
         self.assertFalse(os.path.isdir(quarantine_path))
-        del(kwargs['zero_byte_fps'])
+        del (kwargs['zero_byte_fps'])
         clear_auditor_status(self.devices, 'objects')
         self.auditor.run_audit(**kwargs)
         self.assertTrue(os.path.isdir(quarantine_path))
@@ -1586,7 +1588,6 @@ class TestAuditor(TestAuditorBase):
                          .format(outstanding_pids))
 
 
-@mock.patch('pkg_resources.iter_entry_points', no_audit_watchers)
 @patch_policies(_mocked_policies)
 class TestAuditWatchers(TestAuditorBase):
 
@@ -1639,6 +1640,8 @@ class TestAuditWatchers(TestAuditorBase):
                 'X-Timestamp': timestamp.internal,
                 'Content-Length': str(len(frag_0)),
                 'X-Object-Meta-Flavor': 'peach',
+                'X-Object-Sysmeta-Ec-Frag-Index': '1',
+                'X-Object-Sysmeta-Ec-Etag': 'fake-etag',
             }
             writer.put(metadata)
             writer.commit(timestamp)
@@ -1694,8 +1697,13 @@ class TestAuditWatchers(TestAuditorBase):
         self.assertEqual(len(calls), 6)
 
         self.assertEqual(calls[0], ["__init__", conf, mock.ANY])
-        self.assertIsInstance(calls[0][2], PrefixLoggerAdapter)
-        self.assertIs(calls[0][2].logger, self.logger)
+        watcher_logger = calls[0][2]
+        self.assertIsInstance(calls[0][2], SwiftLogAdapter)
+        self.assertIs(calls[0][2].logger, self.logger.logger)
+        self.assertEqual(watcher_logger.server, my_auditor.logger.server)
+        self.assertEqual('', my_auditor.logger.prefix)  # sanity check
+        self.assertEqual('[audit-watcher test_watcher1] ',
+                         watcher_logger.prefix)
 
         self.assertEqual(calls[1], ["start", "ZBF"])
 
@@ -1706,33 +1714,38 @@ class TestAuditWatchers(TestAuditorBase):
         # irrelevant; what matters is that it finds all the things.
         calls[2:5] = sorted(calls[2:5], key=lambda item: item[1]['name'])
 
-        self.assertDictContainsSubset({'name': '/a/c/o0',
-                                       'X-Object-Meta-Flavor': 'banana'},
-                                      calls[2][1])
+        self._assertDictContainsSubset({'name': '/a/c/o0',
+                                        'X-Object-Meta-Flavor': 'banana'},
+                                       calls[2][1])
         self.assertIn('node/sda/objects/0/', calls[2][2])  # data_file_path
         self.assertTrue(calls[2][2].endswith('.data'))  # data_file_path
         self.assertEqual({}, calls[2][3])
 
-        self.assertDictContainsSubset({'name': '/a/c/o1',
-                                       'X-Object-Meta-Flavor': 'orange'},
-                                      calls[3][1])
+        self._assertDictContainsSubset({'name': '/a/c/o1',
+                                        'X-Object-Meta-Flavor': 'orange'},
+                                       calls[3][1])
         self.assertIn('node/sda/objects/0/', calls[3][2])  # data_file_path
         self.assertTrue(calls[3][2].endswith('.data'))  # data_file_path
         self.assertEqual({}, calls[3][3])
 
-        self.assertDictContainsSubset({'name': '/a/c_ec/o',
-                                       'X-Object-Meta-Flavor': 'peach'},
-                                      calls[4][1])
+        self._assertDictContainsSubset({'name': '/a/c_ec/o',
+                                        'X-Object-Meta-Flavor': 'peach'},
+                                       calls[4][1])
         self.assertIn('node/sda/objects-2/0/', calls[4][2])  # data_file_path
         self.assertTrue(calls[4][2].endswith('.data'))  # data_file_path
         self.assertEqual({}, calls[4][3])
 
         self.assertEqual(calls[5], ["end"])
 
+        # check that watcher has not mutated the prefix of the auditor logger
+        my_auditor.logger.debug('the auditor has no logger prefix')
         log_lines = self.logger.get_lines_for_level('debug')
         self.assertIn(
             "[audit-watcher test_watcher1] getting started",
             log_lines)
+        self.assertIn(
+            "the auditor has no logger prefix",
+            log_lines)
 
     def test_builtin_watchers(self):
 
@@ -1758,7 +1771,8 @@ class TestAuditWatchers(TestAuditorBase):
             writer.commit(timestamp)
 
         def fake_direct_get_container(node, part, account, container,
-                                      prefix=None, limit=None):
+                                      prefix=None, limit=None,
+                                      extra_params=None, headers=None):
             self.assertEqual(part, 1)
             self.assertEqual(limit, 1)
 
@@ -1875,7 +1889,8 @@ class TestAuditWatchers(TestAuditorBase):
         for cur in scenario:
 
             def fake_direct_get_container(node, part, account, container,
-                                          prefix=None, limit=None):
+                                          prefix=None, limit=None,
+                                          extra_params=None, headers=None):
                 self.assertEqual(part, 1)
                 self.assertEqual(limit, 1)
 
@@ -1934,6 +1949,275 @@ class TestAuditWatchers(TestAuditorBase):
                              ' '.join(words[3:]), cur['cr'])
                 self.fail(msg=msg)
 
+    def test_dark_data_with_sharding(self):
+
+        # We use the EC object because it's all alone in its fake container.
+        main_acc = self.disk_file_ec._account
+        shard_acc = ".shards_%s" % main_acc
+        cont = self.disk_file_ec._container
+
+        def fake_direct_get_container(node, part, account, container,
+                                      prefix=None, limit=None,
+                                      extra_params=None, headers=None):
+            self.assertEqual(part, 1)
+            self.assertEqual(limit, 1)
+            self.assertIn('X-Backend-Record-Type', headers)
+            self.assertEqual(headers['X-Backend-Record-Type'], 'auto')
+
+            if account == shard_acc:
+                # Listing shards - just shortcut with a made-up response.
+                entry = {'bytes': 30968411,
+                         'hash': '60303f4122966fe5925f045eb52d1129',
+                         'name': '%s' % prefix,
+                         'content_type': 'video/mp4',
+                         'last_modified': '2017-08-15T03:30:57.693210'}
+                return {'X-Backend-Record-Type': 'object'}, [entry]
+
+            else:
+                if account == main_acc and container == cont:
+                    # The root container has no listing but has a shard range.
+                    entry = {
+                        'name': '%s/%s' % (shard_acc, cont),
+                        'timestamp': '1630106063.23826',
+                        'lower': '',
+                        'upper': '',
+                        'object_count': 1,
+                        'bytes_used': 1024,
+                        'meta_timestamp': '1630106063.23826',
+                        'deleted': 0,
+                        'state': 'sharded',
+                        'state_timestamp': '1630106063.23826',
+                        'epoch': None,
+                        'reported': 1,
+                        'tombstones': -1}
+                    return {'X-Backend-Record-Type': 'shard'}, [entry]
+
+                else:
+                    # It's an un-sharded container, no tricks.
+                    entry = {'bytes': 30968411,
+                             'hash': '60303f4122966fe5925f045eb52d1129',
+                             'name': '%s' % prefix,
+                             'content_type': 'video/mp4',
+                             'last_modified': '2017-08-15T03:30:57.693210'}
+                    return {}, [entry]
+
+        conf = self.conf.copy()
+        conf['watchers'] = 'test_watcher1'
+        conf['__file__'] = '/etc/swift/swift.conf'
+
+        ret_config = {'test_watcher1': {'action': 'log', 'grace_age': '0'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[DarkDataWatcher]):
+            my_auditor = auditor.ObjectAuditor(conf, logger=self.logger)
+
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1), \
+                mock.patch("swift.obj.watchers.dark_data.direct_get_container",
+                           fake_direct_get_container):
+            my_auditor.run_audit(mode='once')
+
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertIn(
+            '[audit-watcher test_watcher1] total unknown 0 ok 3 dark 0',
+            log_lines)
+
+    def test_dark_data_with_sharding_fallback_to_root(self):
+
+        # We use the EC object because it's all alone in its fake container.
+        main_acc = self.disk_file_ec._account
+        shard_acc = ".shards_%s" % main_acc
+        cont = self.disk_file_ec._container
+        call_stack = []
+
+        def fake_direct_get_container(node, part, account, container,
+                                      prefix=None, limit=None,
+                                      extra_params=None, headers=None):
+            self.assertEqual(part, 1)
+            self.assertEqual(limit, 1)
+            call_stack.append((account, container, headers))
+
+            if account == shard_acc:
+                # return a shard listing that actaully points to the root OSR
+                entry = {
+                    'name': '%s/%s' % (main_acc, cont),
+                    'timestamp': '1630106063.23826',
+                    'lower': '',
+                    'upper': '',
+                    'object_count': 1,
+                    'bytes_used': 1024,
+                    'meta_timestamp': '1630106063.23826',
+                    'deleted': 0,
+                    'state': 'sharded',
+                    'state_timestamp': '1630106063.23826',
+                    'epoch': None,
+                    'reported': 1,
+                    'tombstones': -1}
+                return {'X-Backend-Record-Type': 'shard'}, [entry]
+
+            else:
+                if account == main_acc and container == cont:
+                    if headers['X-Backend-Record-Type'] == 'auto':
+                        # The root container has no listing but has a shard
+                        # range.
+                        entry = {
+                            'name': '%s/%s' % (shard_acc, cont),
+                            'timestamp': '1630106063.23826',
+                            'lower': '',
+                            'upper': '',
+                            'object_count': 1,
+                            'bytes_used': 1024,
+                            'meta_timestamp': '1630106063.23826',
+                            'deleted': 0,
+                            'state': 'sharded',
+                            'state_timestamp': '1630106063.23826',
+                            'epoch': None,
+                            'reported': 1,
+                            'tombstones': -1}
+                        return {'X-Backend-Record-Type': 'shard'}, [entry]
+                    else:
+                        # we've come back with a direct record-type = object
+                        self.assertEqual(headers['X-Backend-Record-Type'],
+                                         'object')
+                        # let's give them the obj, they've tried hard enough.
+                        entry = {'bytes': 30968411,
+                                 'hash': '60303f4122966fe5925f045eb52d1129',
+                                 'name': '%s' % prefix,
+                                 'content_type': 'video/mp4',
+                                 'last_modified': '2017-08-15T03:30:57.693210'}
+                        return {'X-Backend-Record-Type': 'object'}, [entry]
+
+                else:
+                    # It's an un-sharded container, no tricks.
+                    entry = {'bytes': 30968411,
+                             'hash': '60303f4122966fe5925f045eb52d1129',
+                             'name': '%s' % prefix,
+                             'content_type': 'video/mp4',
+                             'last_modified': '2017-08-15T03:30:57.693210'}
+                    return {}, [entry]
+
+        conf = self.conf.copy()
+        conf['watchers'] = 'test_watcher1'
+        conf['__file__'] = '/etc/swift/swift.conf'
+
+        ret_config = {'test_watcher1': {'action': 'log', 'grace_age': '0'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[DarkDataWatcher]):
+            my_auditor = auditor.ObjectAuditor(conf, logger=self.logger)
+
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1), \
+                mock.patch("swift.obj.watchers.dark_data.direct_get_container",
+                           fake_direct_get_container):
+            my_auditor.run_audit(mode='once')
+
+        self.assertEqual(
+            call_stack[-3:],
+            [(main_acc, cont, {'X-Backend-Record-Type': 'auto'}),
+             (shard_acc, cont, {'X-Backend-Record-Type': 'auto'}),
+             (main_acc, cont, {'X-Backend-Record-Type': 'object'})])
+
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertIn(
+            '[audit-watcher test_watcher1] total unknown 0 ok 3 dark 0',
+            log_lines)
+
+    def test_dark_data_with_sharding_fallback_to_root_no_objects(self):
+
+        # We use the EC object because it's all alone in its fake container.
+        main_acc = self.disk_file_ec._account
+        shard_acc = ".shards_%s" % main_acc
+        cont = self.disk_file_ec._container
+        call_stack = []
+
+        def fake_direct_get_container(node, part, account, container,
+                                      prefix=None, limit=None,
+                                      extra_params=None, headers=None):
+            self.assertEqual(part, 1)
+            self.assertEqual(limit, 1)
+            call_stack.append((account, container, headers))
+
+            if account == shard_acc:
+                # return a shard listing that actaully points to the root OSR
+                entry = {
+                    'name': '%s/%s' % (main_acc, cont),
+                    'timestamp': '1630106063.23826',
+                    'lower': '',
+                    'upper': '',
+                    'object_count': 1,
+                    'bytes_used': 1024,
+                    'meta_timestamp': '1630106063.23826',
+                    'deleted': 0,
+                    'state': 'sharded',
+                    'state_timestamp': '1630106063.23826',
+                    'epoch': None,
+                    'reported': 1,
+                    'tombstones': -1}
+                return {'X-Backend-Record-Type': 'shard'}, [entry]
+
+            else:
+                if account == main_acc and container == cont:
+                    if headers['X-Backend-Record-Type'] == 'auto':
+                        # The root container has no listing but has a shard
+                        # range.
+                        entry = {
+                            'name': '%s/%s' % (shard_acc, cont),
+                            'timestamp': '1630106063.23826',
+                            'lower': '',
+                            'upper': '',
+                            'object_count': 1,
+                            'bytes_used': 1024,
+                            'meta_timestamp': '1630106063.23826',
+                            'deleted': 0,
+                            'state': 'sharded',
+                            'state_timestamp': '1630106063.23826',
+                            'epoch': None,
+                            'reported': 1,
+                            'tombstones': -1}
+                        return {'X-Backend-Record-Type': 'shard'}, [entry]
+                    else:
+                        # we've come back with a direct record-type = object
+                        self.assertEqual(headers['X-Backend-Record-Type'],
+                                         'object')
+                        return {'X-Backend-Record-Type': 'object'}, []
+
+                else:
+                    # It's an un-sharded container, no tricks.
+                    entry = {'bytes': 30968411,
+                             'hash': '60303f4122966fe5925f045eb52d1129',
+                             'name': '%s' % prefix,
+                             'content_type': 'video/mp4',
+                             'last_modified': '2017-08-15T03:30:57.693210'}
+                    return {}, [entry]
+
+        conf = self.conf.copy()
+        conf['watchers'] = 'test_watcher1'
+        conf['__file__'] = '/etc/swift/swift.conf'
+
+        ret_config = {'test_watcher1': {'action': 'log', 'grace_age': '0'}}
+        with mock.patch('swift.obj.auditor.parse_prefixed_conf',
+                        return_value=ret_config), \
+                mock.patch('swift.obj.auditor.load_pkg_resource',
+                           side_effect=[DarkDataWatcher]):
+            my_auditor = auditor.ObjectAuditor(conf, logger=self.logger)
+
+        with mock.patch('swift.obj.watchers.dark_data.Ring', FakeRing1), \
+                mock.patch("swift.obj.watchers.dark_data.direct_get_container",
+                           fake_direct_get_container):
+            my_auditor.run_audit(mode='once')
+
+        self.assertEqual(
+            call_stack[-3:],
+            [(main_acc, cont, {'X-Backend-Record-Type': 'auto'}),
+             (shard_acc, cont, {'X-Backend-Record-Type': 'auto'}),
+             (main_acc, cont, {'X-Backend-Record-Type': 'object'})])
+
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertIn(
+            '[audit-watcher test_watcher1] total unknown 0 ok 2 dark 1',
+            log_lines)
+
 
 if __name__ == '__main__':
     unittest.main()
